//=======================================================
//=======================================================
#ifndef CLUSTERING_H
#define CLUSTERING_H

#include "Primitive.H"
#include "GameObjWrapper.H"
#include "GlobalDataBase.H"

namespace {
  const int MAX_UNIT_COUNT = 64;
  const int MAX_CLUSTER_COUNT = 16;
}

class Cluster;

class Clustering {
  public:
    static void Update(); // Group units and keep all the clusters.

    static inline const int& GetClusterCount() { return clusterCount; }
    static inline Cluster* const* GetClusters() { return clusters; }
    static inline Cluster* const* GetClustersEnd() { return clusters + clusterCount; }

  private:
    static int initEnemyCount;

    static int clusterCount;
    static Cluster* clusters[MAX_UNIT_COUNT];

    static const int dummy;
    static int StaticInit();

};

class Cluster {
  public:
    Cluster(int id_);

    int GetID() const { return id; }

    int GetSize() const { return clusterSize; }
    EnemyTank** GetUnits() { return units; }
    EnemyTank** GetUnitsEnd() { return units + clusterSize; }

    inline WorldCoor GetCenterOfMass() const { return centerOfMass; }
    inline sint4 GetVariance() const { return variance; }
    
    inline int GetHullSize() const { return hullSize; }
    const WorldCoor* GetHull() const { return hull; }
    const WorldCoor* GetHullEnd() const { return hull + hullSize; }

    // This is for General AI
    bool vertexSquadSet[MAX_UNIT_COUNT];
    int numMarineEngaged;

  private:

    int id;

    int clusterSize;
    EnemyTank* units[MAX_UNIT_COUNT];

    int hullSize;
    WorldCoor hull[MAX_UNIT_COUNT];

    WorldCoor centerOfMass;
    sint4 variance;

    void AddUnit(EnemyTank* unit);
    void Clear(); // Clear any information related to this cluster
    void CalcStat(); // Calculate all relevant statistics

    friend class Clustering;
};
#endif
