#include "DebugHeader.H"
#include "FFData.H"
#include "FFEngine.H"
#include "XTerrainEngine.H"
#include "GlobalDataBase.H"

/* 
   Copyright 2007 Man Hon Chan, Zhangbo Liu, Simon Suyadi
   This code is Licensed under GPL.
 */

DEF_FERR("FFData");

inline void AddCCBlock(const MapCoor& leftTop, const MapCoor& rightBottom, MapCoor& door) {
  const int c1 = leftTop.c, c2 = rightBottom.c;
  const int r1 = leftTop.r, r2 = rightBottom.r;
  MapCoor center(r1 + 2, c1 + 2); 
  XTerrainEngine::Set(r1, c1, C_BR);
  XTerrainEngine::Set(r1, c2, C_BL);
  XTerrainEngine::Set(r2, c1, C_TR);
  XTerrainEngine::Set(r2, c2, C_TL);
  for (int r = r1+1; r < r2; ++r) {
    XTerrainEngine::Set(r, c1, C_R);
    XTerrainEngine::Set(r, c2, C_L);
  }
  for (int c = c1+1; c < c2; ++c) {
    XTerrainEngine::Set(r1, c, C_B);
    XTerrainEngine::Set(r2, c, C_T);
  }
  for (int r = r1 + 1; r < r2; ++r)
    for (int c = c1 + 1; c < c2; ++c)
      XTerrainEngine::Set(r, c, T_BLOCK);
  const int dice = ((GlobalDataBase::GetRandom())->rand_uint4()) % 4;
  switch(dice) {
    case 0: //top
      door.r = center.r - 1;
      door.c = center.c;
      break;
    case 1: //bottom
      door.r = center.r + 1;
      door.c = center.c;
      break;
    case 2: //left
      door.r = center.r;
      door.c = center.c - 1;
      break;
    case 3:
      door.r = center.r;
      door.c = center.c + 1;
      break;
  }

  MapBox::AddTerrain(MapCoor(r1 + 1, c1 + 1),
      MapCoor(r2 - 1, c2 - 1));
  MapBox::RemoveTerrain(door);
  MapBox::RemoveTerrain(center);

  MapBox::AddTaboo(MapCoor(r1, c1));
  MapBox::AddTaboo(MapCoor(r1, c2));
  MapBox::AddTaboo(MapCoor(r2, c1));
  MapBox::AddTaboo(MapCoor(r2, c2));

  /*
  MapBox::AddTaboo(MapCoor(r1 + 1, c1 + 1), MapCoor(r1 + 1, c1 + 1));
  MapBox::AddTaboo(MapCoor(r1 + 1, c2 - 1), MapCoor(r1 + 1, c2 - 1));
  MapBox::AddTaboo(MapCoor(r2 - 1, c2 - 1), MapCoor(r2 - 1, c2 - 1));
  MapBox::AddTaboo(MapCoor(r2 - 1, c1 + 1), MapCoor(r1 - 1, c2 + 1));
  */
}

inline void RemoveCCBlock(const MapCoor& leftTop, const MapCoor& rightBottom, const MapCoor& door) {
  const int c1 = leftTop.c, c2 = rightBottom.c;
  const int r1 = leftTop.r, r2 = rightBottom.r;
  MapCoor center(r1 + 2, c1 + 2); 

  MapBox::AddTerrain(door);
  MapBox::AddTerrain(center);
  MapBox::RemoveTerrain(MapCoor(r1 + 1, c1 + 1),
      MapCoor(r2 - 1, c2 - 1));

  MapBox::RemoveTaboo(MapCoor(r1, c1));
  MapBox::RemoveTaboo(MapCoor(r1, c2));
  MapBox::RemoveTaboo(MapCoor(r2, c1));
  MapBox::RemoveTaboo(MapCoor(r2, c2));

  /*
  MapBox::RemoveTaboo(MapCoor(r1 + 1, c1 + 1), MapCoor(r1 + 1, c1 + 1));
  MapBox::RemoveTaboo(MapCoor(r1 + 1, c2 - 1), MapCoor(r1 + 1, c2 - 1));
  MapBox::RemoveTaboo(MapCoor(r2 - 1, c2 - 1), MapCoor(r2 - 1, c2 - 1));
  MapBox::RemoveTaboo(MapCoor(r2 - 1, c1 + 1), MapCoor(r1 - 1, c2 + 1));
  */

  for (int r = r1; r <= r2; ++r)
    for (int c = c1; c <= c2; ++c)
      XTerrainEngine::Set(r, c, T_EMPTY);
}

void FFEnemyData::CleanUp() {
  FFEngine::ClearEnemyData(this);
}

void FFNeutralData::CleanUp() {
  FFEngine::ClearNeutralData(this);
}

void FFPlayerData::CleanUp() {
  FFEngine::ClearPlayerData(this);
}

void FFStaticData::Init() {
  leftTop = MapBox::GetMapCoor(
      ((PlayerCC*)gameObjWrapper)->GetLeftTop() );
  rightBottom = MapBox::GetMapCoor(
      ((PlayerCC*)gameObjWrapper)->GetRightBottom() );
  AddCCBlock(leftTop, rightBottom, door);
}

void FFStaticData::CleanUp() {
  RemoveCCBlock(leftTop, rightBottom, door);
}

void FFEnemyStaticData::Init() {
  leftTop = MapBox::GetMapCoor(
      ((EnemyCC*)gameObjWrapper)->GetLeftTop() );
  rightBottom = MapBox::GetMapCoor(
      ((EnemyCC*)gameObjWrapper)->GetRightBottom() );
  AddCCBlock(leftTop, rightBottom, door);
}

void FFEnemyStaticData::CleanUp() {
  RemoveCCBlock(leftTop, rightBottom, door);
}

