#ifndef FF_DATA_H
#define FF_DATA_H

#include "Primitive.H"
#include "GameObjWrapper.H"
#include "PlayerTank.H"
#include "EnemyObj.H"
#include "NeutralObj.H"
#include "PlayerCC.H"

class FFEngine;
// These are all structures that are used by FFEngine to record data
// They are not supposed to be used anywhere else

// The global base class
class FFData {
  public:
    virtual void CleanUp() = 0;
    virtual ~FFData() {}
  protected:
    FFData(GameObjWrapper* game_obj_wrapper):gameObjWrapper(game_obj_wrapper) {}
    GameObjWrapper* gameObjWrapper;
    friend class FFEngine;
};

class FFStaticData:public FFData {
  public:
    void CleanUp();
    void Init();
  private:
    FFStaticData(PlayerCC* playerCC):FFData(playerCC) {}
    MapCoor leftTop, rightBottom;
    MapCoor door;
    friend class FFEngine;
};

class FFEnemyStaticData:public FFData {
  public:
    void CleanUp();
    void Init();
  private:
    FFEnemyStaticData(EnemyCC* enemyCC):FFData(enemyCC) {}
    MapCoor leftTop, rightBottom;
    MapCoor door;
    friend class FFEngine;
};

class FFDynamicData: public FFData {
  public:
    virtual ~FFDynamicData() {}
  protected:
    FFDynamicData(GameObjWrapper* game_obj_wrapper):FFData(game_obj_wrapper) {}
    MapCoor prevMapCoor;
    friend class FFEngine;
};

class FFEnemyData: public FFDynamicData {
  public:
    void CleanUp();
  private:
    FFEnemyData(EnemyTank* tank):FFDynamicData(tank) {}
    friend class FFEngine;
};

class FFNeutralData: public FFDynamicData {
  public:
    void CleanUp();
  private:
    FFNeutralData(NeutralObj* sheep):FFDynamicData(sheep) {}
    friend class FFEngine;
};

class FFPlayerData: public FFDynamicData {
  public:
    void CleanUp();

    // For ordering, only valid after update
    inline bool operator<(const FFPlayerData& other) const {
      if (estimatedMapDist < other.estimatedMapDist) return true;
      else if (estimatedMapDist > other.estimatedMapDist) return false;
      if (sqrEuclidDist < other.sqrEuclidDist) return true;
      else if (sqrEuclidDist > other.sqrEuclidDist) return false;
      return manDist < other.manDist;
    }
  protected:
    FFPlayerData(PlayerTank* tank):FFDynamicData(tank) {}
    WorldCoor predictedWorldCoor;
    MapCoor predictedMapCoor;
    sint4 estimatedMapDist, sqrEuclidDist, manDist;
    bool hasRoute;
    sint4 dx, dy; // The direction that it wants to move

    // stuck mechanism
    int stuckCount;
    WorldCoor prevPosition;

    int tabooCount; // If this counts down to zero, remove
    MapCoor tabooCoor; // The position I added my taboo
    friend class FFEngine;
};

#endif
