#ifndef FF_ENGINE_H
#define FF_ENGINE_H

#include "FFData.H"
#include "NeutralObj.H"
#include "PlayerTank.H"
#include "EnemyObj.H"

namespace {
  const sint4 NUM_MAP_ROW = 64;
  const sint4 NUM_MAP_COL = 64;

  const int MAX_NUM_TANK = 64;

  const int PLAYER_BUCKET_SIZE = 4;
  const int ENEMY_BUCKET_SIZE = 4;
  const int NEUTRAL_BUCKET_SIZE = 8;
  const int PREDICTED_BUCKET_SIZE = 6;
}

class FFPlayerData;

class FFEngine {
  public:

    // Called after GlobalDataBase is built
    static void Install();

    // Called after everything on the GlobalDataBase is initialized
    static void Init();

    // Calculate all the force and set the movements
    static void Run();

    static void ClearPlayerData(const FFPlayerData* const &data);
    static void ClearNeutralData(const FFNeutralData* const &data);
    static void ClearEnemyData(const FFEnemyData* const &data);

  private:

    static void ProcessPlayer(PlayerTank* tank);
    static void UpdatePlayerData(PlayerTank* tank);

    static void RebucketPlayer(PlayerTank* tank);
    static void RebucketEnemy(EnemyTank* tank);
    static void RebucketNeutral(NeutralObj* sheep);

    static void AddPlayerData(const MapCoor& coor, FFPlayerData* data);
    static void RemovePlayerData(const FFPlayerData* const &data);

    static void AddPredictedData(const WorldCoor& coor, FFPlayerData* data);
    static void RemovePredictedData(const FFPlayerData* const &data);

    static void AddNeutralData(const MapCoor& coor, FFNeutralData* data);
    static void RemoveNeutralData(const FFNeutralData* const &data);

    static void AddEnemyData(const MapCoor& coor, FFEnemyData* data);
    static void RemoveEnemyData(const FFEnemyData* const &data);

    static FFPlayerData** playerBucketsEnd[NUM_MAP_ROW][NUM_MAP_COL];
    static FFPlayerData* playerBuckets[NUM_MAP_ROW][NUM_MAP_COL][PLAYER_BUCKET_SIZE];
    static FFPlayerData** predictedBucketsEnd[NUM_MAP_ROW][NUM_MAP_COL];
    static FFPlayerData* predictedBuckets[NUM_MAP_ROW][NUM_MAP_COL][PREDICTED_BUCKET_SIZE];

    static FFEnemyData** enemyBucketsEnd[NUM_MAP_ROW][NUM_MAP_COL];
    static FFEnemyData* enemyBuckets[NUM_MAP_ROW][NUM_MAP_COL][ENEMY_BUCKET_SIZE];

    static FFNeutralData** neutralBucketsEnd[NUM_MAP_ROW][NUM_MAP_COL];
    static FFNeutralData* neutralBuckets[NUM_MAP_ROW][NUM_MAP_COL][NEUTRAL_BUCKET_SIZE];

    static FFPlayerData* processQueue[MAX_NUM_TANK];
    static FFPlayerData** processQueueEnd;
};

#endif
