#ifndef GLOBAL_DATA_BASE_H
#define GLOBAL_DATA_BASE_H

#include "Primitive.H"
#include "Game.H"
#include "GameChanges.H"
#include "GameObjWrapper.H"
#include "PlayerInfo.H"
#include "PlayerCC.H"
#include "PlayerTank.H"
#include "EnemyObj.H"
#include "NeutralObj.H"

class GlobalDataBase {
  public:
    // InitGame is the entry point of everything
    // Once it is connected, we should simply call InitGame to
    // post initialize everything else
    // Including the MapBox and alike
    // This is the single entry point to the whole system
    static void InitGame(Game* game_);
    static void CleanUp();

    static inline PlayerInfo& GetPlayerInfo() {
      return game->get_cplayer_info();
    }

    static inline PlayerTank** GetPlayerTanks() { return playerTanks; }
    static inline EnemyTank** GetEnemyTanks() { return enemyTanks; }
    static inline NeutralObj** GetNeutralObjs() { return neutralObjs; }
    static inline PlayerCC** GetPlayerCCs() { return playerCCs; }
    static inline EnemyCC** GetEnemyCCs() { return enemyCCs; }
    static inline EnemyObj** GetEnemyUnits() { return enemyUnits; }

    static inline int GetPlayerTankCount() {
      return playerTanksEnd - playerTanks;
    }
    static inline int GetEnemyTankCount() {
      return enemyTanksEnd - enemyTanks;
    }
    static inline int GetNeutralObjCount() {
      return neutralObjsEnd - neutralObjs;
    }
    static inline int GetPlayerCCCount() {
      return playerCCsEnd - playerCCs;
    }
    static inline int GetEnemyCCCount() {
      return enemyCCsEnd - enemyCCs;
    }
    static inline int GetEnemyUnitCount() {
      return enemyUnitsEnd - enemyUnits;
    }

    static inline PlayerTank** GetPlayerTanksEnd() { return playerTanksEnd; }
    static inline EnemyTank** GetEnemyTanksEnd() { return enemyTanksEnd; }
    static inline NeutralObj** GetNeutralObjsEnd() { return neutralObjsEnd; }
    static inline PlayerCC** GetPlayerCCsEnd() { return playerCCsEnd; }
    static inline EnemyCC** GetEnemyCCsEnd() { return enemyCCsEnd; }
    static inline EnemyObj** GetEnemyUnitsEnd() { return enemyUnitsEnd; }

    static inline Game* GetGame() { return game; }

    static inline void SetRandom(Random* random_) { random = random_; }
    static inline Random* GetRandom() { return random; }

    static inline sint4 GetGameTick() { return gameTick; }
    static inline sint4 GetPlayerID() { return playerID; }
    static inline sint4 GetEnemyID() { return enemyID; }

    // Line of sight information
    static inline bool CanSee(const PlayerTank* const &tank, const EnemyObj* const &obj) {
      return canSeePE[tank->arrayIndex][obj->enemyIndex];
    }

    static inline bool CanSee(const PlayerCC* const &center, const EnemyObj* const &obj) {
      return canSeeCE[center->arrayIndex][obj->enemyIndex];
    }

    static inline bool CanSee(const EnemyTank* const &tank, const EnemyObj* const &obj) {
      return canSeeEE[tank->enemyIndex][obj->enemyIndex];
    }

    // Update all internal information
    static void Update(const GameChanges& game_changes);
    
    static void Display();

  private:
    static Game* game;

    static PlayerTank*  playerTanks[64];
    static PlayerTank** playerTanksEnd;

    static EnemyTank*  enemyTanks[64];
    static EnemyTank** enemyTanksEnd;

    static NeutralObj*  neutralObjs[128];
    static NeutralObj** neutralObjsEnd;

    static PlayerCC*  playerCCs[8];
    static PlayerCC** playerCCsEnd;

    static EnemyCC*  enemyCCs[8];
    static EnemyCC** enemyCCsEnd;

    static EnemyObj*  enemyUnits[64];
    static EnemyObj** enemyUnitsEnd;

    // Nobody is supposed to access the graveyard anyway...
    static GameObjWrapper* graveyard[128];
    static int corpseCount;

    static Random* random;

    static sint4 gameTick;
    static sint4 playerID, enemyID;

    static bool canSeePE[64][64];
    static bool canSeeCE[ 8][64];
    static bool canSeeEE[64][64];

    const static int dummy;
    static int StaticInit();
};

#endif
