#include "PlayerCC.H"
#include "GlobalDataBase.H"
#include "Clustering.H"

/* 
   Copyright 2007 Man Hon Chan, Zhangbo Liu, Simon Suyadi
   This code is Licensed under GPL.
 */

namespace {
  const sint4 HARASS_RADIUS = 168;
  const sint4 SQR_HARASS_RADIUS = HARASS_RADIUS * HARASS_RADIUS;
}

void PlayerCC::Init(GameObj* game_obj) {
  GameObjWrapper::Init(game_obj);

  leftTop = WorldCoor(*game_obj->sod.x1, *game_obj->sod.y1);
  rightBottom = WorldCoor(*game_obj->sod.x2, *game_obj->sod.y2);
  worldCoor = WorldCoor((leftTop.x + rightBottom.x) / 2,
      (leftTop.y + rightBottom.y) / 2 );
  mapCoor = MapBox::GetMapCoor(worldCoor);
  hp = *gameObj->get_int_ptr("hp");
  RefreshStat();

}

bool PlayerCC::Update() {
  if (! (dead = gameObj->is_dead() || (!gameObj->sod.in_game) ) )
    hp = *gameObj->get_int_ptr("hp");
  return !dead;
}

void PlayerCC::RefreshStat() {
  harassSet = 0;
  harassTankTotalHP = 0;
  harassTankCount = 0;
  for (EnemyTank* const* i = GlobalDataBase::GetEnemyTanks(),
      * const* ei = GlobalDataBase::GetEnemyTanksEnd(); i != ei; ++i)
    if (GlobalDataBase::CanSee(this, *i)
       && worldCoor.SqrDist( (*i)->GetWorldCoor() ) < SQR_HARASS_RADIUS ) {
      if ( (*i)->GetCluster() )
        harassSet |= (1 << (*i)->GetCluster()->GetID() );
      else
        ++harassTankCount;
      harassTankTotalHP += (*i)->GetHP();
    }

  int bk = harassSet;
  while(bk) {
    int last_bit = bk ^ (bk & (bk - 1));
    bk ^= last_bit;
    last_bit = __builtin_ctz(last_bit);
    harassTankCount += Clustering::GetClusters()[last_bit]->GetSize();
  }
}
