#ifndef PRIMITIVE_H
#define PRIMITIVE_H

#include <iostream>
#include "Game.H"

struct MapCoor {
  sint4 r, c;
  MapCoor(sint4 rr = -1, sint4 cc = -1):r(rr), c(cc) {}

  inline bool operator== (const MapCoor& other) const {
    return r == other.r && c == other.c;
  }
  inline bool operator!= (const MapCoor& other) const {
    return r != other.r || c != other.c;
  }
  inline bool operator< (const MapCoor& other) const {
    return (r < other.r) || (r == other.r && c < other.c);
  }
};

struct WorldCoor {
  sint4 x, y;
  WorldCoor(sint4 xx = -1, sint4 yy = -1): x(xx), y(yy) {}
  WorldCoor(const MapCoor& mc) : x(mc.c * 16 + 8), y(mc.r * 16 + 8) {}

  inline bool operator== (const WorldCoor& other) const {
    return x == other.x && y == other.y;
  }
  inline bool operator!= (const WorldCoor& other) const {
    return x != other.x || y != other.y;
  }
  inline bool operator< (const WorldCoor& other) const {
    return (x < other.x) || (x == other.x && y < other.y);
  }

  inline sint4 SqrDist(const WorldCoor& other) const {
    sint4 dx(other.x - x), dy(other.y - y);
    return dx * dx + dy * dy;
  }

  inline sint4 ManDist(const WorldCoor& other) const {
    return abs(other.x - x) + abs(other.y - y);
  }

  inline double dist(const WorldCoor& other) const {
    sint4 dx(other.x - x), dy(other.y - y);
    return sqrt(dx * dx + dy * dy);
  }

  inline WorldCoor Offset(const WorldCoor& origin) const {
    return WorldCoor(x - origin.x, y - origin.y);
  }
  inline MapCoor GetMapCoor() const {
    return MapCoor(y / 16, x / 16);
  }
};

struct MapDirection {
  sint4 dr, dc;
  MapDirection(sint4 dr_ = 0, sint4 dc_ = 0):dr(dr_), dc(dc_) {}

  inline bool operator==(const MapDirection& other) const {
    return dr == other.dr && dc == other.dc;
  }
  inline bool operator!=(const MapDirection& other) const {
    return dr != other.dr && dc != other.dc;
  }
  inline bool operator<(const MapDirection& other) const {
    return (dr < other.dr) || (dr == other.dr && dc < other.dc);
  }
};

inline MapCoor operator+(const MapCoor& coor, const MapDirection& dir) {
  return MapCoor(coor.r + dir.dr, coor.c + dir.dc);
}

const int NUM_OTHOGONAL_DIR = 4;
const MapDirection ORTHOGONAL_DIR[] = {
  MapDirection(-1, 0), MapDirection(0, -1), MapDirection(0, 1), MapDirection(1, 0) };
const int NUM_DIAGONAL_DIR = 4;
const MapDirection DIAGONAL_DIR[] = {
  MapDirection(-1, -1), MapDirection(-1, 1), MapDirection(1, -1), MapDirection(1, 1) };

std::ostream& operator<<(std::ostream& os, const MapCoor& coor);
std::ostream& operator<<(std::ostream& os, const WorldCoor& coor);
std::ostream& operator<<(std::ostream& os, const MapDirection& dir);

inline double Dist(const WorldCoor& a, const WorldCoor& b) {
  sint4 dx(a.x - b.x), dy(a.y - b.y);
  return sqrt(dx * dx + dy * dy);
}

#endif
