#include "DebugHeader.H"
#include "SquadAI.H"
#include "GlobalDataBase.H"
#include "Clustering.H"
#include "PathFinder.H"

using namespace std;

DEF_FERR("SquadAI");

#define CLEAR_DEAD(v) for (size_t i = 0; i < v.size(); ) if (v[i]->IsDead()) { v[i] = v.back(); v.pop_back(); } else ++i;

const sint4 HALF_RANGE = 112 / 2;
const sint4 RANGE = 112;
const sint4 HYPER_RANGE = 112 * 3 / 2;
const sint4 CC_NEAR_SQR_RANGE = HYPER_RANGE * HYPER_RANGE;

SquadAI::SquadAI(sint4 id)
  : defaultActionWaypoint(0, 0), aiMode(AI_DEFAULT)
  /*, inBattle(false)*/ {
  squadTanks.reserve(20);
  pathFinder = new PathFinder();
  targetCooldown = TARGET_COOLDOWN_TIMER;
  target = NULL;
  nearestEnemyCC = NULL;
};

void SquadAI::AddTank(PlayerTank* tank) {
  if(tank!=NULL) {
    squadTanks.push_back(tank);
    tank->SetSquad(this);
  }
}

void SquadAI::Act() {
  //Scan and remove dead tanks from squad
  CLEAR_DEAD(squadTanks);
  for (int i = 0; i < squadTanks.size(); i++) {
    //tanks are assumed to be alive
    //LOG(ferr << "Working with squad member " << i << endl);
    EnemyObj* temp_target = NULL;
    //LOG(ferr << "Before looking for the temp_target" << endl);
    temp_target = squadTanks[i]->WeakestInRange(RANGE);
    //LOG(ferr << "Before setting target" << endl);
    squadTanks[i]->SetTarget(temp_target);
    //LOG(ferr << "Before setting CC" << endl);
  }
} // End of SquadAI::Act

void SquadAI::DefaultTargetAction() {
  switch(aiMode) {
    // just find nearest CC and attack
    case AI_DEFAULT:
      {
        sint4 min_dist = 99999999;
        sint4 dist = 0;
        EnemyCC** enemyCCs = GlobalDataBase::GetEnemyCCs();
        for (int i = 0; i < GlobalDataBase::GetEnemyCCCount(); i++) {
          // to counter taboos
          if ( !MapBox::Walkable(enemyCCs[i]->GetMapCoor()) ) {
            //if(min_dist == 99999999) {
            //  min_dist = 99999998;
            //  nearestEnemyCC = enemyCCs[i];
            //}
            continue;
          }
          dist = enemyCCs[i]->GetWorldCoor().SqrDist(centerOfMass)+1;
          //Assume squadTanks[0] is not NULL
          if (dist < min_dist) {
            min_dist = dist;
            nearestEnemyCC = enemyCCs[i];
          }
        }
        if(min_dist < 99999999) {
          int t_dist = 9999999;
          defaultActionWaypoint = nearestEnemyCC->GetMapCoor();
          for(vector<PlayerTank*>::iterator i = squadTanks.begin(); i != squadTanks.end(); i++) {
            t_dist <?= nearestEnemyCC->GetWorldCoor().SqrDist((*i)->GetWorldCoor());
          }
          if(t_dist < CC_NEAR_SQR_RANGE) {
            targetCooldown >?= 3;
          }
        } else {
          // DO NOT DO THIS!
          // may point to terrain
          //defaultActionWaypoint = centerOfMass.GetMapCoor();
        }
      }
      break;
    case AI_DEFENSIVE:
      //cerr << " Squad Defensive Mode ";
      // move towards that coordinate
      break;
  }
  //cerr << " goto (" << defaultActionWaypoint.c << "," << defaultActionWaypoint.r << ") ";
}

void SquadAI::Update() {
  // First find their center of mass
  int cum_x = 0, cum_y = 0;
  int size = squadTanks.size();
  for (int i = 0; i < size; ++i) {
    cum_x += squadTanks[i]->GetWorldCoor().x;
    cum_y += squadTanks[i]->GetWorldCoor().y;
  }
  //cerr << " size = " << size;
  centerOfMass = WorldCoor( cum_x / size, cum_y / size );
  // if there is no target, just attack nearest enemy cc
  if(target != NULL) {
    target->Update(this);
    LOG(target->PrintTarget(cerr));
    if(!target->IsValid() || !MapBox::Walkable(target->GetWaypoint())) {
      LOG(ferr << " target changed" << endl);
      delete target;
      target = NULL;
      //targetCooldown = 0;
    }
  } else {
    // if an enemy CC is in range, increment cooldown
    DefaultTargetAction();
  }
  if(targetCooldown > 0) targetCooldown--;
  //cerr << endl;
}

SquadAI* SquadAI::Split(const WorldCoor& pt, int ntank) {
  int N = squadTanks.size();
  ntank >?= 4;
  if(N - ntank <= 4) return NULL;
  std::vector<std::pair<int, int> > data(squadTanks.size());
  for(int i = 0; i < N; i++) {
    data[i] = make_pair(Dist(pt, squadTanks[i]->GetWorldCoor()), i);
  }
  sort(data.begin(), data.end());
  SquadAI* sq = new SquadAI(0);
  std::vector<PlayerTank*> tmp;
  swap(squadTanks, tmp);
  for(int i = 0; i < ntank; i++) {
    sq->AddTank( tmp[ data[i].second ] );
  }
  for(int i = ntank; i < N; i++) {
    AddTank( tmp[ data[i].second ] );
  }
  return sq;
}

int SquadAI::SqrDist(const SquadAI* sq) const {
  int min_dist = 999999;
  for(int i = 0; i < squadTanks.size(); i++) {
    for(int j = 0; j < sq->squadTanks.size(); j++) {
      min_dist <?= squadTanks[i]->GetDist(sq->squadTanks[j]);
    }
  }
  return min_dist;
}

// optimized version of the above
bool SquadAI::TestDist(const SquadAI* sq, const int& dist) const {
  for(int i = 0; i < squadTanks.size(); i++) {
    for(int j = 0; j < sq->squadTanks.size(); j++) {
      if(squadTanks[i]->GetWorldCoor().SqrDist(sq->squadTanks[j]->GetWorldCoor()) < dist) return true;
    }
  }
  return false;
}

void SquadAI::Merge(SquadAI* other) {
  int n = other->squadTanks.size();
  for (int i = 0; i < n; ++i)
    AddTank(other->squadTanks[i]);
  other->squadTanks.clear();
  LOG(ferr << "Merge" << endl);
  delete other;
}

#ifdef EXPERIMENTAL
void SquadAI::RefreshHarassSet() {
  harassSet = 0;
  harassTankTotalHP = 0;
  harassTankCount = 0;
  for (EnemyTank* const* i = GlobalDataBase::GetEnemyTanks(),
      * const* ei = GlobalDataBase::GetEnemyTanksEnd(); i != ei; ++i)
    if (GlobalDataBase::CanSee(this, *i)
        && worldCoor.SqrDist( (*i)->GetWorldCoor() ) < SQR_HARASS_RADIUS ) {
      harassSet |= (1 << (*i)->GetCluster()->GetID() );
      harassTankTotalHP += (*i)->GetHP();
    }

  int bk = harassSet;
  while(bk) {
    int last_bit = bk ^ (bk & (bk - 1));
    bk ^= last_bit;
    last_bit = __builtin_ctz(last_bit);
    harassTankCount += Clustering::GetClusters()[last_bit]->GetSize();
  }
}
void SquadAI::CheckBattleStatus() {
  for (int i = 0, ei = squadTanks.size(); i < ei; i++) {
    if (squadTanks.at(i)->IsInBattle()) {
      inBattle = true;
      return;
    }
  }
}
void SquadAI::DoProcess(vector< vector<int> >& player_sand, const int& pv_index, vector< vector<int> >& enemy_sand, const int& ev_index) {

  isProcessed = true;
}
#endif
