//=================================================================
// SquadAI.H
// Responsible for making tactic decisions for one squad of tank
//=================================================================
// TODO:
// Personality of SquadAI, to be or not to be
// Prevent overkilling
//=================================================================
#ifndef SQUADAI_H
#define SQUADAI_H
#include "PlayerTank.H"
#include "EnemyObj.H"
#include "Target.H"
using namespace std;

namespace {
  enum AIMode { AI_DEFAULT, AI_DEFENSIVE };
  const int TARGET_COOLDOWN_TIMER = 5;
}

class PathFinder;

class SquadAI {
  public:
    SquadAI(sint4 id);
    void AddTank(PlayerTank* tank);
    void Act();

    bool IsAlive() const { return squadTanks.size();}
    inline sint4 NumOfTanks() const { return squadTanks.size(); }
    inline sint4 Size() const { return squadTanks.size(); }

    inline const MapCoor& GetDest() const {
      if(target == NULL) return defaultActionWaypoint;
      return target->GetWaypoint();
    }

    //inline sint4 GetID() { return ID; }
    //inline void SetID(const sint4& id) { ID = id; } 

    inline vector<PlayerTank*>& GetTanks() { return squadTanks; }
    int SqrDist(const SquadAI* sq) const;
    bool TestDist(const SquadAI* sq, const int& dist) const;

    // targetting system
    inline void SetTarget(Target* t) {
      if(target != NULL) delete target;
      target = t; targetCooldown = TARGET_COOLDOWN_TIMER;
      target->Update(this); //force!
    }
    inline void ResetTargetCooldown() {
      targetCooldown = TARGET_COOLDOWN_TIMER;
    }
    inline bool HasTarget() const {
      return (target != NULL) || (targetCooldown > 0);
    }

    void Update();
    void Merge(SquadAI* other);
    inline WorldCoor GetCenterOfMass() const { return centerOfMass; }
    PathFinder* pathFinder;

    inline void SetAIMode(const AIMode& m) { aiMode = m; }
    inline void SetDefaultActionWaypoint(const MapCoor& wp) {
      // NOTE: gets recalculated unless defensive mode
      defaultActionWaypoint = wp;
    }

    //split off this amount of tanks to go to pt
    SquadAI* Split(const WorldCoor& pt, int ntank);
  protected:
  private:
    std::vector<PlayerTank*> squadTanks; 

    Target* target;
    int targetCooldown;
    EnemyCC* nearestEnemyCC;

    void DefaultTargetAction();
    MapCoor defaultActionWaypoint;

    AIMode aiMode;

    //states
    WorldCoor centerOfMass;

    // all nearby enemy clusters
    std::vector<Cluster*> enemyClusters;
};

#endif
