#ifndef TARGET_H
#define TARGET_H

#include<iostream>
using std::ostream;
using std::endl;

#include "GlobalDataBase.H"

class PlayerTank;
class PlayerCC;
class SquadAI;

namespace {
  enum TargetType {
    target_playercc,
    target_area,
    target_enemycc,
    target_squad,
    target_cooldown
  };
};

class Target {
  public:
    Target() : isvalid(true) {}
    virtual ~Target() {}
    inline const MapCoor& GetWaypoint() const { return waypoint; }
    virtual void Update(const SquadAI*) = 0;
    virtual void PrintTarget(ostream& out) = 0;
    inline TargetType GetTargetType() const { return type; }
    inline bool IsValid() const { return isvalid; }
    inline bool CanMerge() const { return canmerge; }
  protected:
    TargetType type;
    MapCoor waypoint;
    // properties
    bool isvalid; //target dead or no longer exist
    bool canmerge; //can merge
};

class Target_EnemyCC : public Target {
  public:
    Target_EnemyCC() {
      type = target_enemycc;
    }
    void Update(const SquadAI*);
    void PrintTarget(ostream& out);
};

class Target_PlayerCC : public Target {
  public:
    Target_PlayerCC(PlayerCC* c);
    ~Target_PlayerCC();
    void Update(const SquadAI*);
    void PrintTarget(ostream& out);
  private:
    PlayerCC* cc;
};

class Target_Area : public Target {
  public:
    Target_Area(const MapCoor& wp)
      : worldCoor(wp)  { 
      waypoint = wp; 
      type = target_area;
    }
    void Update(const SquadAI*);
    void PrintTarget(ostream& out);
  private:
    sint4 harassSet;
    sint4 harassTankCount;
    sint4 harassTankTotalHP;
    WorldCoor worldCoor;
};

/*
   class Target_Squad : public Target {
   public:
   Target_Squad(SquadAI* s) : tanks(s->GetTanks()) { 
   type = target_squad;
   }
   void Update(const SquadAI*);
   private:
   std::vector<PlayerTank*> tanks; //iterate to find squad
   };
 */

#endif
