#ifndef Game2EventHandler_H
#define Game2EventHandler_H

// $Id: Game2EventHandler.H,v 1.1 2005/10/19 21:25:38 orts_mburo Exp $

// This is an ORTS file (c) Michael Buro, licensed under the GPL

#include "EventHandler.H"
#include "Random.H"
#include "MapBox.H"
#include "FFEngine.H"

#include "GlobalDataBase.H"
#include "GeneralAI.H"

class GameStateModule;
class GfxModule;


// This class gives the event handler access to all game and gfx data

struct Game2AIState {

  GameStateModule *gsm;
  GfxModule       *gfxm; // !=0 : gfx used
  bool just_drew;  // just drew a gfx frame
  bool quit;       // true => quit game
  bool error;      // true <=> error occurred
  
  Game2AIState(GameStateModule *gsm_, GfxModule *gfxm_) :
    gsm(gsm_), gfxm(gfxm_)
  {
    just_drew = quit = error = false;
  }

};


// This class handles server events
// In particular it generates game object actions

class Game2EventHandler : public EventHandler {

public:
  
  Game2AIState &state;

  Game2EventHandler(Game2AIState &state_, sint4 seed) : state(state_), 
  postInitialized(false), prevFFTick(-1) {
    random.srand(seed);
    GlobalDataBase::SetRandom( &random );
    FFEngine::Install();
  }

  bool handle_event(const Event &e);
  

private:
  
  bool postInitialized;
  Random random; // local RNG
  sint4 prevFFTick;
  GeneralAI* generalAI;

  void ComputeActions();
  void PostInitialize();

};

#endif
