#ifndef GAME_OBJ_WRAPPER_H
#define GAME_OBJ_WRAPPER_H

#include "Primitive.H"
#include "MapBox.H"
#include "GameObj.H"
#include "Game.H"

class FFData;
class GlobalDataBase;

class GameObjWrapper {
  public:
    static GameObjWrapper* GetWrapper(GameObj* game_obj);

    // update should be called per frame
    // It returns if the object is still valid.
    // If not, the corresponding destructor will be called.
    virtual bool Update();
    virtual void CleanUp();

    virtual ~GameObjWrapper(); // This is doing nothing right now, but we must allow it to have a destructor

    inline const WorldCoor& GetWorldCoor() const { return worldCoor; }

    inline const MapCoor& GetMapCoor() const { return mapCoor; }

    inline bool IsDead() const { return dead; }

    inline const GameObj* GetGameObj() const { return gameObj; }
    inline GameObj* GetGameObj() { return gameObj; }

    /* For external library use */

    inline sint4 GetClusterID() const { return clusterID; }
    inline void SetClusterID(const sint4& cluster_id ) {
      clusterID = cluster_id;
    }

    inline real8 GetDist(const GameObjWrapper* obj) const {
      return Game::distance( *gameObj, *obj->gameObj );
    }

    // For GlobalDatabase to maintain the arrays
    int arrayIndex;

    // For Force field pathing
    FFData* ffData;

    /* End of external library hack */

  protected:
    void Init(GameObj* game_obj);
    GameObjWrapper(); // Just new

    GameObj* gameObj;
    WorldCoor worldCoor;
    MapCoor mapCoor;
    bool dead;
    
    sint4 clusterID;
};

inline GameObjWrapper::GameObjWrapper() {
  ffData = NULL;
}

inline bool GameObjWrapper::Update() {
  dead = (gameObj->is_dead()) || (!gameObj->sod.in_game);

  worldCoor.x = *gameObj->sod.x;
  worldCoor.y = *gameObj->sod.y;

  mapCoor = MapBox::GetMapCoor(worldCoor);
  return !dead;
}

#endif
