#include "DebugHeader.H"
#include "PlayerTank.H"
#include "EnemyObj.H"
#include "GlobalDataBase.H"
#include "ScriptObj.H"
#include <algorithm>

DEF_FERR("PlayerTank");

namespace {
  const sint4 SQR_RANGE = 110 * 110;
}
Vector<sint4> PlayerTank::attackArgs(1);
Vector<sint4> PlayerTank::moveArgs(3);

PlayerTank::PlayerTank():target(NULL), nextFireTime(-1)/*,inBattle(false)*/ {
}

void PlayerTank::Init(GameObj* game_obj) {
  GameObjWrapper::Init(game_obj);
  dest = mapCoor; // My own mapCoor
  wayPoint = worldCoor; // My current location
}

bool PlayerTank::Update() {
  if (!GameObjWrapper::Update()) return false;
  hp = *gameObj->get_int_ptr("hp");
  return true;
}

void PlayerTank::SetDest(const MapCoor& coor) {
  if (dest != coor) {
    dest = coor;
    // Should we reset the FFData?
  }
}

sint4 PlayerTank::GetCooldown() const {
  const ScriptObj* weapon = gameObj->component("weapon");
  const sintptr *lf = weapon->get_int_ptr("last_fired");
  const sintptr *cd = weapon->get_int_ptr("cooldown");
  const sint4 game_tick = GlobalDataBase::GetGameTick();
  return std::max(sint4(*cd + *lf - game_tick), 0);
}

void PlayerTank::Act() {
  if (target != NULL
      && !GetCooldown()
      && GlobalDataBase::GetGameTick() >= nextFireTime
      && (!target->IsDead()) ) {
    LOG(ferr << "Attacking" << endl);
    attackArgs[0] = target->GetID();
    gameObj->component("weapon")->set_action("attack", attackArgs);
    nextFireTime = GlobalDataBase::GetGameTick() + 5;
  } else { // Move to the way point
    LOG(ferr << "Moving towards " << wayPoint << " with speed " << speed << endl);
    moveArgs[0] = wayPoint.x;
    moveArgs[1] = wayPoint.y;
    moveArgs[2] = speed;
    if (target != NULL) moveArgs[2] <?= 2; // Can't move faster
    gameObj->set_action("move", moveArgs);
  }
}

EnemyObj* PlayerTank::WeakestInRange(sint4 range) const {

  LOG(ferr << "Hello World! " << this << endl);
  
  WorldCoor pos = GetWorldCoor();
  EnemyObj* weakest = NULL;
  sint4 weakest_hp = 2000;

  //LOG(ferr << "Before main loop" << endl);

  real8 tmp;
  for(EnemyObj** enemy_unit = GlobalDataBase::GetEnemyUnits(),
      ** enemy_units_end = GlobalDataBase::GetEnemyUnitsEnd();
      enemy_unit != enemy_units_end; ++enemy_unit) {
    //LOG(ferr << "Checking " << (*enemy_unit) << endl);
    //tmp = (*enemy_unit)->GetWorldCoor().SqrDist(pos);
    tmp = GetDist(*enemy_unit);
    if (tmp < range && (*enemy_unit)->GetHP() < weakest_hp)
      weakest_hp = (weakest = (*enemy_unit))->GetHP();

  }
  /*
  if (weakest!=NULL) {
    inBattle = true;
  }
  else {
    inBattle = false;
  }
  */
  return weakest;
}
