#include<iostream>
#include<fstream>
#include<cmath>
using namespace std;

// just for that ANGLE_DIV thing

typedef unsigned long long uint8;
typedef unsigned int uint4;

namespace {
  // T_TL means Top Left edge is blocked
  enum XTerrain { T_EMPTY, T_BLOCK, T_TL, T_TR, T_BL, T_BR };
  const int ANGLE_DIV = 24; //fine tune this to performance
  const int TANK_RAD = 7; //fixed radius
  const int SHEEP_RAD = 4;
  const int BIT_COUNT = 32;
  const int MAX_STEP = 6;
  const float pi = acos(-1.0f);
  const float DANGLE = 2 * pi / ANGLE_DIV;
  const uint4 BIT_BASE = ((uint4)1 << ANGLE_DIV) - 1;
}

#define LOG( x ) x
ofstream fout("CollisionPrecompute.log");

// CONSTANTS
// generate the direction

typedef float ld;

// how wide the angles for each step as a fraction
const float CollisionPathPercent[MAX_STEP + 1] = {
  1.0f, 1.0f, 0.8f, 0.5f, 0.4f, 0.3f, 0.2f
};

//*****************************

struct CollisionData {
  void Rotate(int a); // rotations
  void AddForward();
  void Find(int& a, int& s) const;
  void Add(const CollisionData& step_data); // AND the bitmasks together

  //FOR DEBUGGING AND INITIALIZATION ONLY
  void Reset();
  void Clear();
  bool get(int a, int s);
  void set(int a, int s);
  void clr(int a, int s);
  void view();
  int dir(); //get 8-direction
  void printdir(int l);
  uint4 d[MAX_STEP + 1];
};

//**********************************

// DATA BEGIN HERE

CollisionData forward;

CollisionData tdata[ANGLE_DIV + 2][MAX_STEP + 1]; //for collision with tanks
CollisionData sdata[ANGLE_DIV + 2][MAX_STEP + 1]; //for collision with sheeps

//16x16 terrain, 6 possible tiles, at 9 possible locations
// size = (4 * 6) * (16 * 16 * 6 * 9) == 331,776 Bytes = 0.33 MBytes
CollisionData terrain[3][3][6][16][16];

//**********************************

void CollisionData::Find(int& a, int& s) const {
  for(s = MAX_STEP; s > 0; s--) {
    if(d[s] != 0) {
      int l = __builtin_ctz(d[s]);
      int r = __builtin_clz(d[s]) - (BIT_COUNT - ANGLE_DIV);
      if(l <= r) a = l;
      else a = ANGLE_DIV - r - 1;
      return;
    }
  }
  // s = 0
}

void CollisionData::Add(const CollisionData& step_data) {
  for(int i = 1; i <= MAX_STEP; i++) d[i] &= step_data.d[i];
}

void CollisionData::AddForward() {
  Add(forward);
}

void CollisionData::Reset() {
  for(int i = 1; i <= MAX_STEP; i++) d[i] = BIT_BASE;
}

void CollisionData::Clear() {
  for(int i = 1; i <= MAX_STEP; i++) d[i] = 0;
}

//rotates the data
void CollisionData::Rotate(int a) {
  if(a < 0) a += ANGLE_DIV;
  for(int i = 1; i <= MAX_STEP; i++) {
    // the CollisionTemplate ensures the back is padded with 0
    d[i] = ((d[i] << a) | (d[i] >> (ANGLE_DIV - a))) & BIT_BASE;
  }
}

void CollisionData::view() {
  for(int s = 1; s <= MAX_STEP; s++) {
    fout << " s=" << s << " >";
    for(int a = 0; a < ANGLE_DIV; a++) {
      if(get(a, s)) fout << "# ";
      else fout << ". ";
    }
    fout << endl;
  }
}

bool CollisionData::get(int a, int s) {
  return d[s] & ((uint4)1 << a);
}

//sets the bitmask to 1
//at Collision a, distance s
void CollisionData::set(int a, int s) {
  d[s] |= (uint4)1 << a;
}

//sets the bitmask to 0
//at Collision a, distance s
void CollisionData::clr(int a, int s) {
  d[s] &= ~((uint4)1 << a);
}

// this is a debugging function
// gets bitmask of 8 - directions only
int CollisionData::dir() {
  int x = 0;
  int div = 8, step = ANGLE_DIV / div;
  for(int i = 0; i < div; i++) {
    if(get(i * step, 6)) x |= (1 << i);
  }
  return x;
}

void CollisionData::printdir(int l) {
  int x = 0;
  int div = 8, step = ANGLE_DIV / div;
  char c[] = "R/U\\L/D\\";
  for(int i = 0; i < div; i++) {
    if(get(i * step, l)) fout << c[i];
    else fout << ".";
  }
}

//**********************************

namespace {
  istream& operator>> ( istream& is, CollisionData& cd ) {
    for(int i = 0; i < MAX_STEP + 1; i++) is >> cd.d[i];
    return is;
  }

  ostream& operator<<(ostream& os, const CollisionData& cd) {
    for(int i = 0; i < MAX_STEP + 1; i++) os << cd.d[i] << " ";
    return os;
  }
}

void data_load() {
  LOG( cout << " data load started" << endl; )
    ifstream din("data.txt");
  din >> forward;
  for(int i = 0; i < MAX_STEP + 1; i++) for(int a = 0; a < ANGLE_DIV + 2; a++)
    din >> tdata[a][i] >> sdata[a][i];
  for(int x = 0; x < 3; x++) for(int y = 0; y < 3; y++) for(int k = 0; k < 6; k++)
    for(int i = 0; i < 16; i++) for(int j = 0; j < 16; j++)
      din >> terrain[x][y][k][i][j];
  LOG( cout << " data load ended" << endl; )
}

void data_save() {
  ofstream dout("data.txt");
  dout << forward;
  for(int i = 0; i < MAX_STEP + 1; i++) {
    for(int a = 0; a < ANGLE_DIV + 2; a++) {
      dout << tdata[a][i] << sdata[a][i];
    }
  }
  for(int x = 0; x < 3; x++) for(int y = 0; y < 3; y++) for(int k = 0; k < 6; k++)
    for(int i = 0; i < 16; i++) for(int j = 0; j < 16; j++)
      dout << terrain[x][y][k][i][j];
}

//**********************************
// GENERATION STARTS HERE

void generate0() {
  cout << " calculating default bitmask" << endl;
  forward.Reset();
  for(int s = 0; s <= MAX_STEP; s++) {
    for(int a = 0; a < ANGLE_DIV; a++) {
      //b is the 'integer' angle from the centre
      int b = min(a, ANGLE_DIV - a);
      if(b <= CollisionPathPercent[s] * ANGLE_DIV / 2) forward.set(a, s);
      else forward.clr(a, s);
    }
  }
  LOG( forward.view(); )
}

// tank and sheep collision generation
void generate1() {
  cout << " calculating tank and sheep collision bitmask" << endl;

  for(int i = 0; i < ANGLE_DIV; i++)
    for(int d = 0; d <= MAX_STEP; d++) {
      sdata[i][d].Reset();
      tdata[i][d].Reset();
    }

  //now our tank is facing east
  //Collision is measured from horizontal axis
  for(int i = 0; i < ANGLE_DIV; i++) {
    ld a = DANGLE * i;
    for(int d = 0; d <= MAX_STEP; d++) {
      int fs, fa;
      //TANK PART
      //now at Collision a
      //distance d between our tank and another tank
      //so the object is located at position x1, y1
      ld x1 = (d + TANK_RAD * 2) * cos(a);
      ld y1 = -(d + TANK_RAD * 2) * sin(a);

      //now check if this object at position x1, y1 is going to intersect anything
      //check each Collision
      for(int j = 0; j < ANGLE_DIV; j++) {
        ld b = DANGLE * j;
        for(int e = 1; e <= MAX_STEP; e++) {
          //now we move our big tank at Collision b, distance e
          //Collision b, distance e
          ld x2 = (e) * cos(b);
          ld y2 = -(e) * sin(b);
          //distance between them
          ld dist = sqrt(pow(x1 - x2, 2) + pow(y1 - y2, 2));
          if(dist < TANK_RAD * 2) {
            //cout << dist << endl;
            //this position will intersect the target circle
            //so mark the rest as invalid
            for(; e <= MAX_STEP; e++) tdata[i][d].clr(j, e);
          }
        }
      }

      LOG( fout << " tank "; )
        LOG( fout << " i=" << i << " d=" << d; )
        LOG( fout << " x1=" << x1 << " y1=" << y1; )
        LOG( tdata[i][d].Find(fa, fs); )
        LOG( fout << " found a=" << fa << " s=" << fs; )
        LOG( fout << endl; )
        LOG( tdata[i][d].view(); )

        //SHEEP PART
        ld x3 = (d + TANK_RAD + SHEEP_RAD) * cos(a);
      ld y3 = -(d + TANK_RAD + SHEEP_RAD) * sin(a);

      //now check if this object at position x1, y1 is going to intersect anything
      //check each Collision
      for(int j = 0; j < ANGLE_DIV; j++) {
        ld b = DANGLE * j;
        for(int e = 1; e <= MAX_STEP; e++) {
          //now we move our big tank at Collision b, distance e
          //Collision b, distance e
          ld x4 = (e) * cos(b);
          ld y4 = -(e) * sin(b);
          //distance between them
          ld dist = sqrt(pow(x3 - x4, 2) + pow(y3 - y4, 2));
          if(dist < (TANK_RAD + SHEEP_RAD)) {
            //this position will intersect the target circle
            //so mark the rest as invalid
            for(; e <= MAX_STEP; e++) sdata[i][d].clr(j, e);
          }
        }
      }

      LOG( fout << " sheep "; )
        LOG( fout << " i=" << i << " d=" << d; )
        LOG( fout << " x3=" << x3 << " y3=" << y3; )
        LOG( sdata[i][d].Find(fa, fs); )
        LOG( fout << " found a=" << fa << " s=" << fs; )
        LOG( fout << endl; )
        LOG( sdata[i][d].view(); )

    }
  }

  //ALSO:
  for(int d = 1; d <= MAX_STEP; d++) {
    sdata[ANGLE_DIV][d] = sdata[0][d];
    tdata[ANGLE_DIV][d] = tdata[0][d];
  }
}

//bigger tile
char ntile[16 * 3][16 * 3];

void view_tile() {
  for(int j = 0; j < 3 * 16; j++) {
    for(int i = 0; i < 3 * 16; i++) {
      if(ntile[i][j]) fout << "*";
      else fout << ".";
    }
    fout << endl;
  }
}

void view_terrain(int x, int y, int k) {
  for(int j = 0; j < 16; j++) {
    for(int i = 0; i < 16; i++) {
      //int d = terrain[loc][k][i][j].dir();
      //fout << d << " ";
      terrain[x][y][k][i][j].printdir(6);
      fout << " ";
    }
    fout << endl;
  }
}

void create_tile(int X, int Y, int id) {
  memset(ntile, 0, sizeof(ntile));
  switch(id) {
    case T_BLOCK:
      LOG( fout << " T_BLOCK X=" << X << " Y=" << Y << endl; )
        for(int j = 0; j < 16; j++) for(int i = 0; i < 16; i++)
          ntile[i + X][j + Y] = 1;
      break;
    case T_TL:
      LOG( fout << " T_TL X=" << X << " Y=" << Y << endl; )
        for(int j = 0; j < 16; j++) for(int i = 0; i < 16 - j; i++)
          ntile[i + X][j + Y] = 1;
      break;
    case T_TR:
      LOG( fout << " T_TR X=" << X << " Y=" << Y << endl; )
        for(int j = 0; j < 16; j++) for(int i = j; i < 16; i++)
          ntile[i + X][j + Y] = 1;
      break;
    case T_BR:
      LOG( fout << " T_BR X=" << X << " Y=" << Y << endl; )
        for(int j = 0; j < 16; j++) for(int i = 16 - j - 1; i < 16; i++)
          ntile[i + X][j + Y] = 1;
      break;
    case T_BL:
      LOG( fout << " T_BL X=" << X << " Y=" << Y << endl; )
        for(int j = 0; j < 16; j++) for(int i = 0; i <= j; i++)
          ntile[i + X][j + Y] = 1;
      break;
    case T_EMPTY:
      break;
  }
  LOG( view_tile(); )
}

// terrain collision generation
void generate2() {
  cout << " calculating terrain collision bitmask" << endl;
  for (int dr = -1; dr <= 1; ++dr) {
    for (int dc = -1; dc <= 1; ++dc) {
      /*
         for(int x = 0; x < 3; x++) {
         for(int y = 0; y < 3; y++) {
         cout << " x=" << x << " y=" << y << " tile: ";
       */
      cout << " dr=" << dr << " dc=" << dc << " tile: ";
      for(int k = 0; k < 6; k++) {
        //there are 6 possible tiles
        //now test every pixel
        cout << k << " " << flush;

        create_tile( (dc+1) * 16, (dr+1) * 16, k);
        for(int x = 0; x < 16; ++x) {
          for(int y = 0; y < 16; ++y) {
            terrain[dr+1][dc+1][k][x][y].Reset();

            //test if this place IS VALID
            for(int x2 = 0; x2 < 3 * 16; x2++) {
              for(int y2 = 0; y2 < 3 * 16; y2++) {
                int x1 = 16 + x, y1 = 16 + y;
                ld dist = sqrt(pow((ld)(x1 - x2), 2) + pow((ld)(y1 - y2), 2));
                if(dist >= TANK_RAD) continue; //there is no value
                if(ntile[x2][y2]) {
                  terrain[dr][dc][k][x][y].Clear();
                  goto skippo;
                }
              }
            }

            //test every direction
            for(int z = 0; z < ANGLE_DIV; z++) {
              ld a = DANGLE * z;
              for(int d = 1; d <= MAX_STEP; d++) {
                //we are at location (x, y)
                // do not forget to offset by 16, 16
                ld x1 = d * cos(a) + 16 + i;
                ld y1 = -d * sin(a) + 16 + j;

                //now we test pixel (x2, y2)
                for(int x2 = 0; x2 < 3 * 16; x2++) {
                  for(int y2 = 0; y2 < 3 * 16; y2++) {
                    ld dist = sqrt(pow(x1 - x2, 2) + pow(y1 - y2, 2));
                    if(dist >= TANK_RAD) continue; //there is no value

                    if(ntile[x2][y2]) {
                      //LOG( fout << " collision! " << endl; )
                      for(; d <= MAX_STEP; d++) terrain[x][y][k][i][j].clr(z, d);
                      goto end;
                    }
                  }
                }
end:;
              }
            }

            //NOTE: LOGGING EVERTYHING IS TOO MUCH OF DEBUG
            //LOG( fout << " loc=" << loc << " k=" << k << " i=" << i << " j=" << j << endl; )
            //LOG( terrain[loc][k][i][j].view(); )

skippo:;
          }
        }

        LOG( fout << " x=" << x << " y=" << y << " k=" << k << endl; )
          LOG( view_terrain(x, y, k); )
      }
      cout << endl;
    }
    }
    }

    // DONE GENERATE
    //**********************************


    int main() {
      generate0();
      generate1();
      generate2();

      data_save();
      return 0;
    }

