#include "Clustering.H"


void Clustering::KMedoids(EnemyTank** enemy, const int& n, std::vector<EnemyTank*>& medoids_obj, const int& k, vector<int>& medoids_size) {
//1. Randomly get K objects as initial medoids
  //cerr << "Inside KMedoids" << endl;
  medoids_obj.clear();
  medoids_size.clear();
  EnemyTank* old_medoids[k];
  EnemyTank* medoids[k];
  sint4 index = 0;
  //cerr << "Before random selection" << endl;

  for (int i = 0; i < k; i++) {
    index = GlobalDataBase::GetRandom()->rand_uint4()%n;    // Randomly select an object
    enemy[index]->SetClusterID(i);
    medoids[i] = old_medoids[i] = enemy[index];
  }
  //cerr << "After random selection" << endl;
  Sorting(old_medoids, k);
  //cerr << "Initial means are: " << endl;
  for (int i = 0; i < k; i++) {
    cerr << "(" << old_medoids[i]->GetWorldCoor().x << "," << old_medoids[i]->GetWorldCoor().y << "),";
  }
  cerr << endl;
  cerr <<"-----------------" << endl;
//2. Repeat
  bool repeat = false;
  int c = 0;
  do {
    c++;
  //2.1 Assign every non-medoid object to its closest medoid
    //2.2 Randomly select a non-medoid object O_h                      //TODO: Randomly or FORALL?
    Evolve(enemy, n, medoids, k);
 
    Sorting(medoids, k);
    for (int i = 0; i < k; i++){
      if (medoids[i]!=old_medoids[i]) {
        repeat = true;
        old_medoids[i] = medoids[i];
      }
    }
  } while (repeat && c < 120);

  cerr << "After "<< c << " interations:" << endl;
  int number;
  for (int i = 0; i < k; i++) {
    number = 0;
    medoids_obj.push_back(medoids[i]);
    cerr << "Cluster " << i << " contains:";
    for (int j = 0; j < n; j ++) {
      if (enemy[j]->GetClusterID()==i) {
        cerr << "(" << enemy[j]->GetWorldCoor().x << "," << enemy[j]->GetWorldCoor().y << "),";
        number++;
      }
    }
    cerr << endl;
    medoids_size.push_back(number);
  }
  return;
}

void Clustering::InitAssignClosest(PlayerTank** unit, const int& n,  PlayerCC** medoid, const int& k) {

  int dist, minDist, index;
  for (int i = 0; i < n; i++) {
    dist = 0;
    minDist = 999999999;
    index = 0;
    for (int j = 0; j < k; j++) {
      //if (unit[i]!=medoid[j]) {
        dist = medoid[j]->GetWorldCoor().SqrDist(unit[i]->GetWorldCoor());
        if (dist < minDist) {
          minDist = dist;
          unit[i]->SetClusterID(j);
        }
      //}
    }
  }

}

sint4 Clustering::AssignAndTestClosest(EnemyTank** unit, const int& n,  EnemyTank** medoid, const int& k, const sint4& threshold) {
  sint4 cost = 0;
  int dist, minDist, index;
  for (int i = 0; i < n; i++) {
    dist = 0;
    minDist = 999999999;
    index = 0;
    for (int j = 0; j < k; j++) {
      if (unit[i]!=medoid[j]) {
        dist = medoid[j]->GetWorldCoor().SqrDist(unit[i]->GetWorldCoor());
        if (dist < minDist) {
          minDist = dist;
          unit[i]->SetClusterID(j);
        }
      }
    }
    if ((cost += minDist) > threshold) {
      return threshold + 10;
    }
  }

  return cost;


  for (int j = 0; j < k; j++) {
    cerr << "Cluster " << j << " contains: ";
    for (int i = 0; i < n; i++) {
      if (unit[i]->GetClusterID() == j) {
        cerr << "(" << unit[i]->GetWorldCoor().x << ", " << unit[i]->GetWorldCoor().y << "),";
      }
    }
    cerr << endl;
  }
  

}


void Clustering::Sorting(EnemyTank** unit, const int& k) {
  for (int i = 0; i < k; i++) {
    for (int j = k-1; j > i; j--) {
      if (unit[j] < unit[j-1]) {
        EnemyTank* temp = unit[j];
        unit[j] = unit[j-1];
        unit[j-1] = temp;
      }
    }
  }
}

void Clustering::Evolve(EnemyTank** unit, const int& n, EnemyTank** medoid, const int& k) { //TODO: Optimization needed
  sint4 threshold = AssignAndTestClosest(unit, n, medoid, k, 99999999);
  int index_k = GlobalDataBase::GetRandom()->rand_uint4()%k;
  EnemyTank* saved_medoid = medoid[index_k];
  int index_n;
  bool pass;
  do { 
    index_n = GlobalDataBase::GetRandom()->rand_uint4()%n;
    pass = true;
    for (int i = 0; i < k; i++) {
      if (unit[index_n]==medoid[i]) {
        pass = false;
        break;
      }
    }
  } while(!pass);

  medoid[index_k] = unit[index_n];
  if (threshold < AssignAndTestClosest(unit, n, medoid, k, threshold))
    medoid[index_k] = saved_medoid;
}

void Clustering::Merge(EnemyTank** enemy, vector<EnemyTank*>& medoids_obj, vector<int>& medoids_size, const int& index1, const int& index2) { 
//TODO: Not precise. Simply merge the cluster with smaller size into the bigger one, and use the bigger one's medoid as the new cluster's medoid
  int big, small;
  if (medoids_size.at(index1) > medoids_size.at(index2)) {
    big = index1;
    small = index2;
  }
  else{
    big = index2;
    small = index1;
  }
  medoids_size.at(big) += medoids_size.at(small);
  medoids_size.at(small) = medoids_size.back();
  medoids_size.pop_back();
  medoids_obj.at(small) = medoids_obj.back();
  medoids_obj.pop_back();
  for (int i = 0; i < GlobalDataBase::GetEnemyTankCount(); i++) { //TODO: It is not efficiently to scan all enemy tanks
    if (enemy[i]->GetClusterID() == small) {
      enemy[i]->SetClusterID(big);
    }
  } 
}




