// $Id: Game2EventHandler.C,v 1.12 2006/12/31 04:09:34 orts_furtak Exp $

// This is an ORTS file (c) Michael Buro, licensed under the GPL

// A sample client for ORTS that shows how to access client data

// todo/questions:
//
// game scripts:
// - what to do with workers during build time
// - similarly, how to define mining?
// - how is attack range defined (distance between centers/hulls)?
//
// client:
// - how to access global player data?
// - how to compute the set of visible tiles?
// - how to compute tiles visible from a unit?
// - how to decide what can be attacked by a specific unit?

// FORS/FORALL/ERR macros are defined in orts3/libs/kernel/src/Global.H

#include "DebugHeader.H"
#include "Game2EventHandler.H"
#include "GameStateModule.H"
#include "GfxModule.H"
#include "Game.H"
#include "GameChanges.H"
#include "GameTile.H"
#include "ServerObjData.H"
#include "Options.H"
#include <algorithm>
#include <cstdio>
#include <iostream>
#include "FFEngine.H"

// tile types from core.bp
// fixme: game script reflection would help here
// in the competition we won't use water tiles
// TILE_CLIFF: non-traversable by ground units

namespace {

  static const int NUMBER_OF_SQUADS = 10;
  static const int NUMBER_OF_TANKS = 50;
  static const int NUMBER_OF_BASES = 5;
  const int FF_INTERVAL = 2;
}

bool Game2EventHandler::handle_event(const Event& e)
{

  // Post initialize
  if (!postInitialized) {
    PostInitialize();
  }

  if (e.get_who() == GameStateModule::FROM) {

    //cout << "Event: " << e.who() << " " << e.what() << endl;

    if (e.get_what() == GameStateModule::STOP_MSG) {
      state.quit = true; return true;
    } else if (e.get_what() == GameStateModule::READ_ERROR_MSG) {
      state.quit = state.error = true; return true;
    } else if (e.get_what() == GameStateModule::VIEW_MSG) {

      LOG(cerr << "Before Update" << endl);
      GlobalDataBase::Update(state.gsm->get_changes());
      // Check who to fire

      LOG(cerr << "Before Compute Action" << endl);
      ComputeActions();
      if (GlobalDataBase::GetGameTick() - prevFFTick >= FF_INTERVAL) {
        // Move
        prevFFTick = GlobalDataBase::GetGameTick();
        LOG(cerr << "Before FFEngine::Run()" << endl);
        FFEngine::Run();
      }

      LOG(cerr << "Before Act" << endl);
      // Set all actions
      PlayerTank** tank = GlobalDataBase::GetPlayerTanks();
      PlayerTank** tank_end = GlobalDataBase::GetPlayerTanksEnd();
      for(; tank != tank_end; ++tank) (*tank)->Act();
      LOG(cerr << "Before send action" << endl);
      // All actions set, now send them
      state.gsm->send_actions();

      // REMARK: Here belows are debugging information

      Game &game = state.gsm->get_game();	
      // gfx and frame statistics

      if (state.gfxm) state.gfxm->process_changes();

      sint4 vf = game.get_view_frame();
      sint4 af = game.get_action_frame();
      sint4 sa = game.get_skipped_actions();
      sint4 ma = game.get_merged_actions();
  

      if ((vf % 100) == 0) {
        GlobalDataBase::Display();


      }

      if ((vf % 10) == 0) {
        printf("%05d\n", vf);
        generalAI->Update();

        //cout << "[frame " << vf << "]" << endl;
      }

      if (vf != af || sa || ma > 1) {

        cout << "frame " << vf;
        if (af < 0) cout << " [no action]";
        else if (vf != af) cout << " [behind by " << (vf - af) << " frame(s)]";
        if (sa) cout << " [skipped " << sa << "]";
        if (ma > 1) cout << " [merged " << ma << "]";
        cout << endl;
      }

      if (state.gfxm) {

        // don't draw if we are far behind
        if (abs(vf-af) < 10) {
          state.gfxm->draw();
          state.just_drew = true;
        }
      }

      return true;
    }
  }

  return false;
}

namespace {
  const sint4 SQR_RANGE = 110 * 110;
}

void Game2EventHandler::ComputeActions()
{
  generalAI->Act();
  return;
  // Every tank is going to attack the closest CC in their range
  // or closest enemy
  PlayerTank** tanks = GlobalDataBase::GetPlayerTanks();
  EnemyCC** enemyCCs = GlobalDataBase::GetEnemyCCs();
  EnemyTank** enemyTanks = GlobalDataBase::GetEnemyTanks();

  for (int i = GlobalDataBase::GetPlayerTankCount(); i--; ) {
    WorldCoor pos = tanks[i]->GetWorldCoor();

    // Check out who is the closest enemy to attack
    int closest = 0, closest_dist;
    sint4 weakest_hp = 99999;
    EnemyObj* weakest = NULL;

    closest_dist = enemyCCs[0]->GetWorldCoor().SqrDist( pos ) + 1;
    for (int j = 0; j < GlobalDataBase::GetEnemyCCCount(); ++j) {
      int tmp = enemyCCs[j]->GetWorldCoor().SqrDist( pos );
      if (tmp < closest_dist) {
        closest_dist = tmp;
        closest = j;
      }
      if (tmp < SQR_RANGE && enemyCCs[j]->GetHP() < weakest_hp) {
        weakest_hp = enemyCCs[j]->GetHP();
        weakest = enemyCCs[j];
      }
    }
    
    // Move towards the closest CC
    tanks[i]->SetDest( enemyCCs[closest]->GetMapCoor() );

    for (int j = 0; j < GlobalDataBase::GetEnemyTankCount(); ++j) {
      int tmp = enemyTanks[j]->GetWorldCoor().SqrDist( pos );
      if (tmp < SQR_RANGE && enemyTanks[j]->GetHP() < weakest_hp) {
        weakest_hp = enemyTanks[j]->GetHP();
        weakest = enemyTanks[j];
      }
    }

    tanks[i]->SetTarget( weakest );

  }

}

// Handle everything that depends on the game
void Game2EventHandler::PostInitialize() {
  Game &game = state.gsm->get_game();

  GlobalDataBase::InitGame(&game);
  FFEngine::Init();
  LOG(cerr << "Start init GeneralAI..." << endl);
  generalAI = new GeneralAI();
  generalAI->Init();
  LOG(cerr << "Finish init GeneralAI..." << endl);

  postInitialized = true;
}
