#include "DebugHeader.H"
#include "SquadAI.H"
#include "GlobalDataBase.H"

using namespace std;

DEF_FERR("SquadAI");

#define CLEAR_DEAD(v) for (size_t i = 0; i < v.size(); ) if (v[i]->IsDead()) { v[i] = v.back(); v.pop_back(); } else ++i;


const sint4 HALF_RANGE = 112 / 2;
const sint4 RANGE = 112;
const sint4 HYPER_RANGE = 112 * 3 / 2;


SquadAI::SquadAI(): squadTanks(NULL), missionType(Attack), ID(-1), inBattle(false) {};

void SquadAI::AddTank(PlayerTank* tank) {
  if(tank!=NULL) {
    squadTanks.push_back(tank);
    tank->SetClusterID(ID);
  }
}


void SquadAI::Act() {
  //Scan and remove dead tanks from squad
  CLEAR_DEAD(squadTanks);

  if (missionType==Attack) {
    //Set the closest enemyCC as the squad's dest. This should be one of the choices when the General makes decisions.
    LOG(ferr << "Before ClosestEnemyCC" << endl);
    EnemyCC* closestEnemyCC = ClosestEnemyCC();

    for (int i = 0; i < squadTanks.size(); i++) {//tanks are assumed to be alive
      LOG(ferr << "Working with squad member " << i << endl);
      EnemyObj* temp_target = NULL;
      LOG(ferr << "Before looking for the temp_target" << endl);
      temp_target = squadTanks.at(i)->WeakestInRange(RANGE);
      LOG(ferr << "Before setting target" << endl);
      squadTanks.at(i)->SetTarget(temp_target);
      LOG(ferr << "Before setting CC" << endl);
      if (closestEnemyCC)
        squadTanks.at(i)->SetDest(closestEnemyCC->GetMapCoor());
    }
    LOG(ferr << "Done Attack Mission" << endl);
  } 
  else if (missionType==Intercept) {  //TODO: Redesign needed
    EnemyObj* target = NULL;
    //EnemyObj* potential_target = NULL;
    for (int i = 0; i < squadTanks.size(); i++) {
      EnemyObj* temp_target = NULL;
      temp_target = squadTanks.at(i)->WeakestInRange(RANGE);
      if (temp_target!=NULL) {
        squadTanks.at(i)->SetTarget(temp_target);
        if (target==NULL || target->GetHP() > temp_target->GetHP()) {
          target = temp_target;
        }
      }
      else if (target!=NULL) {
        squadTanks.at(i)->SetDest(target->GetMapCoor());
      }
      else {
        squadTanks.at(i)->SetDest(GetDest());
      }
      
    }
  }
  else if (missionType==Retreat) {
    //TODO: FILL ME
  }
  else {
    cerr << "ERROR: MissionType undefined..." << endl;
  }

} // End of SquadAI::Act


EnemyCC* SquadAI::ClosestEnemyCC() {
  sint4 min_dist = 99999999;
  sint4 dist = 0;
  sint4 closest = -1;
  EnemyCC** enemyCCs = GlobalDataBase::GetEnemyCCs();
  for (int i = 0; i < GlobalDataBase::GetEnemyCCCount(); i++) {
    if ( !MapBox::Walkable(enemyCCs[i]->GetMapCoor()) ) continue;
    /*
    sint4* dist_matrix = MapBox::GetDist(enemyCCs[i]->GetMapCoor());
    if (dist_matrix == NULL) continue;
    dist = 16 * dist_matrix[MapBox::CoorToID(squadTanks[0]->GetMapCoor())]
      + 5 * enemyCCs[i]->GetWorldCoor().SqrDist(squadTanks[0]->GetWorldCoor())+1; //Assume squadTanks[0] is not NULL
     */
    dist = enemyCCs[i]->GetWorldCoor().SqrDist(squadTanks[0]->GetWorldCoor())+1; //Assume squadTanks[0] is not NULL
    if (dist < min_dist) {
      min_dist = dist;
      closest = i;
    }
  }
  if (closest == -1) return NULL;
  return enemyCCs[closest];

}

void SquadAI::EngagingEnemyCluster(int* engaging_map, const int& max_squad_number) {
  for (int i = 0; i < squadTanks.size(); i++) {
    WorldCoor pos = squadTanks.at(i)->GetWorldCoor();
    //sint4 dist;
    for (EnemyTank** enemy_tanks = GlobalDataBase::GetEnemyTanks(), 
        **enemy_tanks_end = GlobalDataBase::GetEnemyTanksEnd();
        enemy_tanks != enemy_tanks_end; enemy_tanks++) {
      //if ((*enemy_tanks)->GetWorldCoor().SqrDist(pos) < RANGE) {
      if ( (*enemy_tanks)->GetDist( squadTanks.at(i) ) < RANGE) {
        engaging_map[ID*max_squad_number + (*enemy_tanks)->GetClusterID()] = 1;  
      }
    } 
  }
}
/*
void SquadAI::CheckBattleStatus() {
  for (int i = 0, ei = squadTanks.size(); i < ei; i++) {
    if (squadTanks.at(i)->IsInBattle()) {
      inBattle = true;
      return;
    }
  }
}
*/
/*
void SquadAI::DoProcess(vector< vector<int> >& player_sand, const int& pv_index, vector< vector<int> >& enemy_sand, const int& ev_index) {
  
  isProcessed = true;
}
*/


/*
WorldCoor SquadAI::GetClosestEnemyCC( GlobalDataBase* gdb) {
  //cerr << "Within GetClosestEnemyCC, before everything" << endl;
  //Get the mean position of the squad rather than the squad leader's
  int pos_x = 0;
  int pos_y = 0;
  vector<TankAI*>::iterator it;
  for (it = squad_tanks.begin(); it != squad_tanks.end(); it++) {
    if (!((*it)->tank->is_dead())) {
      pos_x += *((*it)->tank->sod.x);
      pos_y += *((*it)->tank->sod.y);
    }
  }
  pos_x = pos_x/squad_tanks.size();
  pos_y = pos_y/squad_tanks.size();
  //int pos_x = *(squad_tanks.at(0)->tank->sod.x);
  //int pos_y = *(squad_tanks.at(0)->tank->sod.y); 
  int min_dist = 999999;
  int dist = 0;
  WorldCoor coor;
  //cerr << "Within GetClosestEnemyCC, before for loop" << endl;
  for (int i = 0; i < gdb->enemy_cc.size(); i++) {
    if (!(gdb->enemy_cc.at(i)->is_dead())) { //TODO: This checking might not be necessary
      int cc_x = (*(gdb->enemy_cc.at(i)->sod.x1) + *(gdb->enemy_cc.at(i)->sod.x2))/2;
      int cc_y = (*(gdb->enemy_cc.at(i)->sod.y1) + *(gdb->enemy_cc.at(i)->sod.y2))/2;
      int dist = sqrt((pos_x - cc_x)*(pos_x - cc_x)+(pos_y - cc_y)*(pos_y - cc_y));
      if (dist < min_dist) {
        min_dist = dist;
        coor.x = cc_x;
        coor.y = cc_y;
      }
    }
  }
  return coor;
}
*/

/*
WorldCoor SquadAI::RotateFormation(WorldCoor p, double angle) {
  double radius = sqrt(p.x*p.x + p.y*p.y);
  if (radius == 0) { // Is the central point
    return p;
  }
  else {
    WorldCoor newpoint;
    newpoint.x = p.x*cos(angle) - p.y*sin(angle);
    newpoint.y = p.x*sin(angle) + p.y*cos(angle);
    return newpoint;
  }
}
*/

/*
void SquadAI::GetGoal() { //For testing purpose
  vector<PlayerTank*>::iterator it;
    cerr << "Our goals are: ";
  for (it = squadTanks.begin(); it != squadTanks.end(); it++) {
    cerr << "(" << (*it)->GetGoal().x << ", " << (*it)->GetGoal().y << "), ";
  }  
  cerr << endl;
}
*/
