#include<iostream>
#include<fstream>
#include<cmath>
using namespace std;

// just for that ANGLE_DIV thing

typedef unsigned long long uint8;
typedef unsigned int uint4;

namespace {
  // T_TL means Top Left edge is blocked
  enum XTerrain { T_EMPTY = 0, T_BLOCK, T_TL, T_TR, T_BL, T_BR, C_T, C_B, C_L, C_R, C_TL, C_TR, C_BL, C_BR };
  const int ANGLE_DIV = 24; //fine tune this to performance
  const int TANK_RAD = 7; //fixed radius
  const int SHEEP_RAD = 4;
  const int BIT_COUNT = 32;
  const int MAX_STEP = 6;
  const double pi = acos(-1.0f);
  const double DANGLE = 2 * pi / ANGLE_DIV;
  //const uint4 BIT_BASE = (uint4)(-1);
  const uint4 BIT_BASE = ((uint4)1 << ANGLE_DIV) - 1;
  const int MAX_DIST = MAX_STEP + TANK_RAD * 2;
  const int ALLOWANCE = 1;
  const int TERRAIN_ALLOWANCE = 1;
  const int T_COUNT = 14;
  const int CCR = 8;
}

#ifdef NDEBUG
#define LOG( X )
#else
#define LOG( x ) x
ofstream fout("CollisionPrecompute.log");
#endif

// CONSTANTS
// generate the direction

typedef double ld;

// how wide the angles for each step as a fraction
const float CollisionPathPercent[MAX_STEP + 1] = {
  1.0f, 1.0f, 1.0f, 0.8f, 0.6f, 0.4f, 0.2f
};

//*****************************

struct CollisionData {
  void Rotate(int a); // rotations
  void AddForward();
  void Find(int& a, int& s) const;
  void Add(const CollisionData& step_data); // AND the bitmasks together
  
  //FOR DEBUGGING AND INITIALIZATION ONLY
  void Reset();
  void Clear();
  bool get(int a, int s);
  void set(int a, int s);
  void clr(int a, int s);
  void view();
  int dir(); //get 8-direction
  void printdir(int l);
  bool empty();
  uint4 d[MAX_STEP + 1];
};

//**********************************

// DATA BEGIN HERE

CollisionData forward;

CollisionData tdata[ANGLE_DIV + 2][MAX_DIST + 2]; //for collision with tanks
CollisionData sdata[ANGLE_DIV + 2][MAX_DIST + 2]; //for collision with sheeps

//16x16 terrain, 6 possible tiles, at 9 possible locations
// size = (4 * 6) * (16 * 16 * 6 * 9) == 331,776 Bytes = 0.33 MBytes
CollisionData terrain[3][3][T_COUNT][16][16];
bool          isstuck[3][3][T_COUNT][16][16];

//**********************************

bool CollisionData::empty() {
  for(int i = 0; i < MAX_STEP + 1; i++) if(d[i] != 0) return 0;
  return 1;
}

void CollisionData::Find(int& a, int& s) const {
  for(s = MAX_STEP; s > 0; s--) {
    if(d[s] != 0) {
      int l = __builtin_ctz(d[s]);
      int r = __builtin_clz(d[s]) - (BIT_COUNT - ANGLE_DIV);
      if(l <= r) a = l;
      else a = ANGLE_DIV - r - 1;
      return;
    }
  }
  // s = 0
}

void CollisionData::Add(const CollisionData& step_data) {
  for(int i = 1; i <= MAX_STEP; i++) d[i] &= step_data.d[i];
}

void CollisionData::AddForward() {
  Add(forward);
}

void CollisionData::Reset() {
  for(int i = 1; i <= MAX_STEP; i++) d[i] = BIT_BASE;
}

void CollisionData::Clear() {
  for(int i = 1; i <= MAX_STEP; i++) d[i] = 0;
}

//rotates the data
void CollisionData::Rotate(int a) {
  if(a < 0) a += ANGLE_DIV;
  for(int i = 1; i <= MAX_STEP; i++) {
    // the CollisionTemplate ensures the back is padded with 0
    d[i] = ((d[i] << a) | (d[i] >> (ANGLE_DIV - a))) & BIT_BASE;
  }
}

void CollisionData::view() {
  for(int s = 1; s <= MAX_STEP; s++) {
    LOG( fout << " s=" << s << " >" );
    for(int a = 0; a < ANGLE_DIV; a++) {
      LOG(
      if(get(a, s)) fout << "# ";
      else fout << ". ";
      );
    }
    LOG( fout << endl );
  }
}

bool CollisionData::get(int a, int s) {
  return d[s] & ((uint4)1 << a);
}

//sets the bitmask to 1
//at Collision a, distance s
void CollisionData::set(int a, int s) {
  d[s] |= (uint4)1 << a;
}

//sets the bitmask to 0
//at Collision a, distance s
void CollisionData::clr(int a, int s) {
  d[s] &= ~((uint4)1 << a);
}

// this is a debugging function
// gets bitmask of 8 - directions only
int CollisionData::dir() {
  int x = 0;
  int div = 8, step = ANGLE_DIV / div;
  for(int i = 0; i < div; i++) {
    if(get(i * step, 6)) x |= (1 << i);
  }
  return x;
}

void CollisionData::printdir(int l) {
  int x = 0;
  int div = 8, step = ANGLE_DIV / div;
  char c[] = "R/U\\L/D\\";
  for(int i = 0; i < div; i++) {
    LOG(
        if(get(i * step, l)) fout << c[i];
        else fout << ".";
       );
  }
}

//**********************************

namespace {
  istream& operator>> ( istream& is, CollisionData& cd ) {
    for(int i = 0; i < MAX_STEP + 1; i++) is >> cd.d[i];
    return is;
  }

  ostream& operator<<(ostream& os, const CollisionData& cd) {
    for(int i = 0; i < MAX_STEP + 1; i++) os << cd.d[i] << " ";
    return os;
  }
}

void data_load() {
  LOG( cout << " data load started" << endl; )
    ifstream din("data.txt");
  din >> forward;
  for(int i = 0; i < MAX_DIST + 1; i++)
    for(int a = 0; a < ANGLE_DIV + 2; a++)
      din >> tdata[a][i] >> sdata[a][i];
  for(int x = 0; x < 3; x++) for(int y = 0; y < 3; y++)
    for(int k = 0; k < T_COUNT; k++)
      for(int i = 0; i < 16; i++) for(int j = 0; j < 16; j++)
        din >> terrain[x][y][k][i][j] >> isstuck[x][y][k][i][j];
  LOG( cout << " data load ended" << endl; )
}

void data_save() {
  ofstream dout("data.txt");
  dout << forward;
  for(int i = 0; i < MAX_DIST + 1; i++) {
    for(int a = 0; a < ANGLE_DIV + 2; a++) {
      dout << tdata[a][i] << sdata[a][i];
    }
  }
  for(int x = 0; x < 3; x++) for(int y = 0; y < 3; y++)
    for(int k = 0; k < T_COUNT; k++)
      for(int i = 0; i < 16; i++) for(int j = 0; j < 16; j++) {
        dout << terrain[x][y][k][i][j] << " " << isstuck[x][y][k][i][j] << " ";
      }
}

//**********************************
// GENERATION STARTS HERE

void generate0() {
  cout << " calculating default bitmask" << endl;
  forward.Reset();
  for(int s = 0; s <= MAX_STEP; s++) {
    for(int a = 0; a < ANGLE_DIV; a++) {
      //b is the 'integer' angle from the centre
      int b = min(a, ANGLE_DIV - a);
      if(b <= CollisionPathPercent[s] * ANGLE_DIV / 2) forward.set(a, s);
      else forward.clr(a, s);
    }
  }
  LOG( forward.view(); )
}

// tank and sheep collision generation
void generate1() {
  cout << " calculating tank and sheep collision bitmask" << endl;

  for(int i = 0; i <= ANGLE_DIV; i++)
    for(int d = 0; d <= MAX_DIST; d++) {
      sdata[i][d].Reset();
      tdata[i][d].Reset();
    }

  //now our tank is facing east
  //Collision is measured from horizontal axis
  for(int i = 0; i < ANGLE_DIV; i++) {
    ld a = DANGLE * i;
    for(int d = 0; d <= MAX_DIST; d++) {
      int fs, fa;
      //TANK PART
      //now at Collision a
      //distance d between our tank and another tank
      //so the object is located at position x1, y1
      ld x1 = (d) * cos(a);
      ld y1 = (d) * sin(a);

      //now check if this object at position x1, y1 is going to intersect anything
      //check each Collision
      for(int j = 0; j < ANGLE_DIV; j++) {
        ld b = DANGLE * j;
        for(int e = 1; e <= MAX_STEP; e++) {
          //now we move our big tank at Collision b, distance e
          //Collision b, distance e
          ld x2 = (e) * cos(b);
          ld y2 = (e) * sin(b);
          //distance between them
          ld dist = sqrt((x1 - x2) * (x1 - x2) + (y1 - y2) * (y1 - y2));
          if(dist < TANK_RAD * 2 + ALLOWANCE) {
            //cout << dist << endl;
            //this position will intersect the target circle
            //so mark the rest as invalid
            //for(; e <= MAX_STEP; e++) tdata[i][d].clr(j, e);
            tdata[i][d].clr(j, e);
          }
        }
      }

      LOG( fout << " tank "; );
      LOG( fout << " i=" << i << " d=" << d; );
      LOG( fout << " x1=" << x1 << " y1=" << y1; );
      LOG( tdata[i][d].Find(fa, fs); );
      LOG( fout << " found a=" << fa << " s=" << fs; );
      LOG( fout << endl; );
      LOG( tdata[i][d].view(); );

      //SHEEP PART
      ld x3 = (d) * cos(a);
      ld y3 = (d) * sin(a);

      //now check if this object at position x1, y1 is going to intersect anything
      //check each Collision
      for(int j = 0; j < ANGLE_DIV; j++) {
        ld b = DANGLE * j;
        for(int e = 1; e <= MAX_STEP; e++) {
          //now we move our big tank at Collision b, distance e
          //Collision b, distance e
          ld x4 = (e) * cos(b);
          ld y4 = (e) * sin(b);
          //distance between them
          ld dist = sqrt((x3 - x4) * (x3 - x4) + (y3 - y4) * (y3 - y4));
          if(dist < (TANK_RAD + SHEEP_RAD + ALLOWANCE)) {
            //this position will intersect the target circle
            //so mark the rest as invalid
            //for(; e <= MAX_STEP; e++) sdata[i][d].clr(j, e);
            sdata[i][d].clr(j, e);
          }
        }
      }

      LOG( fout << " sheep "; )
        LOG( fout << " i=" << i << " d=" << d; )
        LOG( fout << " x3=" << x3 << " y3=" << y3; )
        LOG( sdata[i][d].Find(fa, fs); )
        LOG( fout << " found a=" << fa << " s=" << fs; )
        LOG( fout << endl; )
        LOG( sdata[i][d].view(); )

    }
  }

  //ALSO:
  for(int d = 1; d <= MAX_DIST; d++) {
    sdata[ANGLE_DIV][d] = sdata[0][d];
    tdata[ANGLE_DIV][d] = tdata[0][d];
  }
}

//bigger tile
char ntile[16 * 3][16 * 3];

void view_tile() {
  for(int j = 0; j < 3 * 16; j++) {
    for(int i = 0; i < 3 * 16; i++) {
      LOG(
          if(ntile[i][j]) fout << "*";
          else fout << ".";
         );
      if(i % 16 == 15) {
        LOG( fout << " " );
      }
    }
    if(j % 16 == 15) {
      LOG( fout << endl );
    }
    LOG( fout << endl );
  }
}

void view_terrain(int x, int y, int k) {
  for(int j = 0; j < 16; j++) {
    for(int i = 0; i < 16; i++) {
      //int d = terrain[loc][k][i][j].dir();
      //fout << d << " ";
      terrain[x][y][k][i][j].printdir(6);
      LOG( fout << " " );
    }
    LOG( fout << endl );
  }
}

void create_tile(int X, int Y, int id) {
  memset(ntile, 0, sizeof(ntile));
  switch(id) {
    case T_EMPTY:
      break;
    case T_BLOCK:
      LOG( fout << " T_BLOCK X=" << X << " Y=" << Y << endl; )
        for(int j = 0; j < 16; j++) for(int i = 0; i < 16; i++)
          ntile[i + X][j + Y] = 1;
      break;
    case T_TL:
      LOG( fout << " T_TL X=" << X << " Y=" << Y << endl; )
        for(int j = 0; j < 16; j++) for(int i = 0; i < 16 - j; i++)
          ntile[i + X][j + Y] = 1;
      break;
    case T_TR:
      LOG( fout << " T_TR X=" << X << " Y=" << Y << endl; )
        for(int j = 0; j < 16; j++) for(int i = j; i < 16; i++)
          ntile[i + X][j + Y] = 1;
      break;
    case T_BR:
      LOG( fout << " T_BR X=" << X << " Y=" << Y << endl; )
        for(int j = 0; j < 16; j++) for(int i = 16 - j - 1; i < 16; i++)
          ntile[i + X][j + Y] = 1;
      break;
    case T_BL:
      LOG( fout << " T_BL X=" << X << " Y=" << Y << endl; )
        for(int j = 0; j < 16; j++) for(int i = 0; i <= j; i++)
          ntile[i + X][j + Y] = 1;
      break;
    case C_T:
      LOG( fout << " CC TOP X=" << X << " Y=" << Y << endl; )
        for(int j = 0; j < CCR; j++) for(int i = 0; i < 16; i++)
          ntile[i + X][j + Y] = 1;
      break;
    case C_B:
      LOG( fout << " CC BOTTOM X=" << X << " Y=" << Y << endl; )
        for(int j = 16 - CCR; j < 16; j++) for(int i = 0; i < 16; i++)
          ntile[i + X][j + Y] = 1;
      break;
    case C_L:
      LOG( fout << " CC LEFT X=" << X << " Y=" << Y << endl; )
        for(int j = 0; j < 16; j++) for(int i = 0; i < CCR; i++)
          ntile[i + X][j + Y] = 1;
      break;
    case C_R:
      LOG( fout << " CC RIGHT X=" << X << " Y=" << Y << endl; )
        for(int j = 0; j < 16; j++) for(int i = 16 - CCR; i < 16; i++)
          ntile[i + X][j + Y] = 1;
      break;
    case C_TL:
      LOG( fout << " CC TOP LEFT X=" << X << " Y=" << Y << endl; )
        for(int j = 0; j < CCR; j++) for(int i = 0; i < CCR; i++)
          ntile[i + X][j + Y] = 1;
      break;
    case C_TR:
      LOG( fout << " CC TOP RIGHT X=" << X << " Y=" << Y << endl; )
        for(int j = 0; j < CCR; j++) for(int i = 16 - CCR; i < 16; i++)
          ntile[i + X][j + Y] = 1;
      break;
    case C_BL:
      LOG( fout << " CC BOTTOM LEFT X=" << X << " Y=" << Y << endl; )
        for(int j = 16 - CCR; j < 16; j++) for(int i = 0; i < CCR; i++)
          ntile[i + X][j + Y] = 1;
      break;
    case C_BR:
      LOG( fout << " CC BOTTOM RIGHT X=" << X << " Y=" << Y << endl; )
        for(int j = 16 - CCR; j < 16; j++) for(int i = 16 - CCR; i < 16; i++)
          ntile[i + X][j + Y] = 1;
      break;
  }
  LOG( view_tile(); )
}

// terrain collision generation
void generate2() {
  cout << " calculating terrain collision bitmask" << endl;
  for(int x = 0; x < 3; x++) {
    for(int y = 0; y < 3; y++) {
      cout << " x=" << x << " y=" << y << " tile: ";
      for(int k = 0; k < T_COUNT; k++) {
        //there are 6 possible tiles
        //now test every pixel
        cout << k << " ";
        cout.flush();
        create_tile(x * 16, y * 16, k);
        for(int i = 0; i < 16; i++) {
          for(int j = 0; j < 16; j++) {
            int allowance = TERRAIN_ALLOWANCE;
            if(x != 1 && y != 1) allowance++;
            terrain[x][y][k][i][j].Reset();
            isstuck[x][y][k][i][j] = false;

            // test if we get stuck in the terrain
            for(int x2 = 0; x2 < 3 * 16; x2++) {
              for(int y2 = 0; y2 < 3 * 16; y2++) {
                if(!ntile[x2][y2]) continue;
                int x1 = 16 + i, y1 = 16 + j;
                ld dist = sqrt((ld)((x1 - x2) * (x1 - x2) + (y1 - y2) * (y1 - y2)));
                if(dist < TANK_RAD + allowance) {
                  //something wrong with the engine
                  isstuck[x][y][k][i][j] = true;
                  goto here;
                }
              }
            }
here:

            //test every direction
            for(int z = 0; z < ANGLE_DIV; z++) {
              ld a = DANGLE * z;
              for(ld d = 1; d <= MAX_STEP; d += 1) {
                // we are at location (x, y)
                // do not forget to offset by 16, 16
                ld x1 = d * cos(a) + 16 + i;
                ld y1 = d * sin(a) + 16 + j;

                //now we test pixel (x2, y2)
                for(int x2 = 0; x2 < 3 * 16; x2++) {
                  for(int y2 = 0; y2 < 3 * 16; y2++) {
                    if(!ntile[x2][y2]) continue;
                    //if(isstuck[x][y][k][i][j]) allowance += 1;
                    ld dist = sqrt((ld)((x1 - x2) * (x1 - x2) + (y1 - y2) * (y1 - y2)));
                    if(dist < TANK_RAD + allowance) { //too far
                      //use dd here
                      if(isstuck[x][y][k][i][j]) {
                        terrain[x][y][k][i][j].clr(z, d);
                      } else {
                        for(; d <= MAX_STEP; d += 1)
                          terrain[x][y][k][i][j].clr(z, d);
                      }
                      goto end;
                    }
                  }
                }
end:;
              }
            }

            //NOTE: LOGGING EVERTYHING IS TOO MUCH OF DEBUG
            //LOG( fout << " loc=" << loc << " k=" << k << " i=" << i << " j=" << j << endl; )
            //LOG( terrain[loc][k][i][j].view(); )

skippo:;
          }
        }

        // a final correction if the tank is 1 pixel inside the terrain
        /*
           for(int i = 0; i < 16; i++) {
           for(int j = 0; j < 16; j++) {
           if(terrain[x][y][k][i][j].empty()) {
           terrain[x][y][k][i][j].Reset();
        //correction from adjacent tiles
        if(i > 0) {
        terrain[x][y][k][i][j]
        }
        }
        }
        }
         */

        LOG( fout << " x=" << x << " y=" << y << " k=" << k << endl; )
          LOG( view_terrain(x, y, k); )
      }
      cout << endl;
    }
  }
}

// DONE GENERATE
//**********************************


int main() {
  generate0();
  generate1();
  generate2();

  data_save();
  return 0;
}

