#include "DebugHeader.H"
#include<iostream>
#include<fstream>
#include<cmath>
#include<cstring>

using namespace std;

// INSTRUCTIONS
// To convert from angles to index
// if(a < 0) a += 2 * pi
// a / DANGLES
// upper bound: ceil(a / DANGLES)
// lower bound: floor(a / DANGLES)
// for distance:
// d -= TANK_RAD * 2
// bound: floor( d )

// to calculate bitmask for each nearby object
// 1) find dx, dy
// 2) find distance, offset angle
// 3) obtain upper and lower bound
// 4) get floor( distance )
// 5) ADD bitmask

// after all bitmasks applied
// use the 'FIND' method to get the direction and angle

// DEBUGGING

DEF_FOUT("CollisionEngine");

#include "CollisionEngine.H"

#define LOG( x )  

// CONSTANTS
// generate the direction

//**********************************

istream& operator>> ( istream& is, CollisionData& cd ) {
  for(int i = 0; i < MAX_STEP + 1; i++) is >> cd.d[i];
  return is;
}

//**********************************
// DATA BEGIN HERE

CollisionData forward;
CollisionData tdata[ANGLE_DIV + 2][MAX_DIST + 2]; //for collision with tanks
CollisionData sdata[ANGLE_DIV + 2][MAX_DIST + 2]; //for collision with sheeps

//16x16 terrain, 6 possible tiles, at 9 possible locations
// size = (4 * 6) * (16 * 16 * 6 * 9) == 331,776 Bytes = 0.33 MBytes
CollisionData terrain[3][3][T_COUNT][16][16];
bool          isstuck[3][3][T_COUNT][16][16];

bool data_load() {
  LOG( cout << " data load started" << endl );
  ifstream din("data.txt");

  if (!(din >> forward))
    return false;

  for(int i = 0; i < MAX_DIST + 1; i++) for(int a = 0; a < ANGLE_DIV + 2; a++)
    if (!(din >> tdata[a][i] >> sdata[a][i]))
      return false;

  for(int x = 0; x < 3; x++) for(int y = 0; y < 3; y++)
    for(int k = 0; k < 6; k++)
      for(int i = 0; i < 16; i++) for(int j = 0; j < 16; j++) {
        if (!(din >> terrain[x][y][k][i][j] >> isstuck[x][y][k][i][j]))
          return false;
      }

  LOG( cout << " data load ended" << endl );
  return true;
}

//**********************************
// CORE PUBLIC INTERFACE

void CollisionEngine::AddForward() {
  LOG( fout << " before forward" << endl );
  LOG ( data.view() );
  data.Add(forward);
  LOG( fout << " after forward" << endl );
  LOG ( data.view() );
}

const int CollisionEngine::dummy_init = CollisionEngine::StaticInit();

void CollisionEngine::AddTank(const int& dx, const int& dy) {
  LOG( fout << " AddTank " << endl );
  float a = atan2(dy, dx);
  if(a < 0) a += 2 * pi;
  a /= DANGLE;
  int d = (int)(sqrt((float)(dx * dx + dy * dy)) + 0.5);
  LOG( fout << " dx=" << dx << " dy=" << dy << " d=" << d << " a=" << a << endl; );
  if(d > MAX_DIST) return;
  if(d < TANK_RAD * 2) stuck = true;
  LOG( fout << "Before data.Add " << endl);
  LOG( data.view() );
  //data.Add(tdata[(int)floor(a)][d]);
  //LOG( fout << "After floor" << endl);
  //LOG( data.view() );
  //data.Add(tdata[(int)ceil(a)][d]);
  LOG( fout << "After add" << endl);
  data.Add(tdata[(int)(a + 0.5)][d]);
  LOG( data.view() );
  LOG( fout << "After Tank" << endl);
}

void CollisionEngine::AddSheep(const int& dx, const int& dy) {
  LOG( fout << " AddSheep" << endl );
  float a = atan2(dy, dx);
  if(a < 0) a += 2 * pi;
  a /= DANGLE;
  int d = (int)(sqrt((float)(dx * dx + dy * dy)) + 0.5);
  LOG( fout << " dx=" << dx << " dy=" << dy << " d=" << d << " a=" << a << endl; );
  if(d > MAX_DIST) return;
  if(d < TANK_RAD + SHEEP_RAD) stuck = true;
  //data.Add(sdata[(int)floor(a)][d]);
  data.Add(sdata[(int)(a + 0.5)][d]);
  LOG( data.view(); )  
}

// CollisionData terrain[3][3][6][16][16];
void CollisionEngine::AddTerrain(const int& dr, const int& dc, const int& k, const WorldCoor& coor) {
  if(k == 0) return;
  LOG( if(dr < -1 || dr > 1 || dc < -1 || dc > 1 || k < 0 || k >= 6)
      cerr << "Invalid tile data!" << " dr=" << dr << " dc=" << dc << " k=" << k << endl );
  LOG( fout << "Before Terrain dr=" << dr << ", dc=" << dc << ", k=" << k << ", " << coor << endl );
  LOG( data.view() );
  data.Add(terrain[dc+1][dr+1][k][coor.x][coor.y]);
  if(isstuck[dc+1][dr+1][k][coor.x][coor.y]) stuck = true;
  LOG( fout << "After Terrain" << endl);
  LOG( data.view() );
}

//**********************************
// GENERATION STARTS HERE

//initialize the templates
const int CollisionEngine::StaticInit() {
  if (!data_load()) {
    LOG(fout << "Data loading failure" << endl);
    throw -1;
  }
  return 0;
}

void CollisionData::Debug(ostream& fout) const {
  for(int s = 1; s <= MAX_STEP; s++) {
    fout << " s=" << s << " >";
    for(int a = 0; a < ANGLE_DIV; a++) {
      if(get(a, s)) fout << "# ";
      else fout << ". ";
    }
    fout << endl;
  }
}


void CollisionData::view() const {
  for(int s = 1; s <= MAX_STEP; s++) {
    LOG(fout << " s=" << s << " >");
    for(int a = 0; a < ANGLE_DIV; a++) {
      LOG(
          if(get(a, s)) fout << "# ";
          else fout << ". ";
         );
    }
    LOG(fout << endl);
  }
}

bool CollisionData::get(int a, int s) const {
  return d[s] & ((uint4)1 << a);
}
