#ifndef __COLLISION_ENGINE
#define __COLLISION_ENGINE

#include "Primitive.H"
#include "XTerrainEngine.H"
#include "Global.H"

#include <iostream>
using std::ostream;

//typedef unsigned long long uint8;
//typedef unsigned int uint4;

namespace {
  // T_TL means Top Left edge is blocked
  const int ANGLE_DIV = 24; //fine tune this to performance
  const int TANK_RAD = 7; //fixed radius
  const int SHEEP_RAD = 4;
  const int BIT_COUNT = 32;
  const int MAX_STEP = 6;
  const float pi = 3.14159265;
  const float DANGLE = 2 * pi / ANGLE_DIV;
  //const uint4 BIT_BASE = (uint4)(-1);
  const uint4 BIT_BASE = ((uint4)1 << ANGLE_DIV) - 1;
  const int MAX_DIST = MAX_STEP + TANK_RAD * 2;
}

struct CollisionData {
  inline void Rotate(const int& a) {
    for(int i = 1; i <= MAX_STEP; i++) {
      // the CollisionTemplate ensures the back is padded with 0
      d[i] = ((d[i] << a) | (d[i] >> (ANGLE_DIV - a))) & BIT_BASE;
    }
    //view();
  }
  inline void Reset() {
    for(int i = 1; i <= MAX_STEP; i++) d[i] = BIT_BASE;
  }
  inline void Find(int& a, int& s) const {
    for(s = MAX_STEP; s > 0; s--) {
      if(d[s] != 0) {
        int l = __builtin_ctz(d[s]);
        int r = __builtin_clz(d[s]) - (BIT_COUNT - ANGLE_DIV);
        if(l <= r) a = l; // WARNING: note the equal sign here
        else a = ANGLE_DIV - r - 1;
        return;
      }
    }
  }
  inline void FreeFind(int& a, int& s) const {
    for(s = MAX_STEP; s > 0; s--) {
      if(d[s] != 0) {
        int l = __builtin_ctz(d[s]);
        int r = BIT_COUNT - 1 - __builtin_clz(d[s]);
        a = (l + r + 0.5f) / 2;
        return;
      }
    }
  }
  inline void Add(const CollisionData& step_data) { // 'AND' all bitmasks
    //step_data.view();
    for(int i = 1; i <= MAX_STEP; i++) d[i] &= step_data.d[i];
  }
  void view() const;
  void Debug(ostream& fout) const;
  bool get(int a, int s) const;
  uint4 d[MAX_STEP + 1];
};

class CollisionEngine {
  public:

    // BEGIN WITH THIS
    inline void Reset() {
      stuck = false;
      data.Reset();
      /*data.view();*/
    }

    // UPDATE: the angle is facing EAST
    // IMPORTANT: distance MUST BE BETWEEN 0 and 6 (radius to radius)
    // IMPORTANT: angle must be [0, 2pi]
    // there is a sheep at angle a, distance d!
    void AddSheep(const int& dx, const int& dy);

    // UPDATE: the angle is facing EAST
    // there is a tank at angle a, distance d!
    void AddTank(const int& dx, const int& dy);

    // i,j are block coordinates use: [0 - 3, 0 - 3], where (0, 0) is top left
    // for k, see XTerrain enum above
    // x, y are the localized pixel coordinate in the block
    void AddTerrain(const int& dr, const int& dc, const int& k, const WorldCoor& coor);

    // Rotate the bitmask to point to tank direction
    inline void Rotate(const int& a) { data.Rotate(a); }

    // Call this function AFTER ROTATE
    void AddForward();

    // call this function AFTER ADDFORWARD
    // when done with every possible collision
    // call this function to get the angle and step (distance in pixels)
    // float angle = a * (2 * pi / ANGLE_DIV)
    inline void Find(int& a, int& s) const {
      data.Find(a, s);
      /*
      if(stuck) {
        data.FreeFind(a, s);
      } else {
        data.Find(a, s);
      }
      */
    }

    void Debug(ostream& fout) const { data.Debug(fout); }

    inline bool Isstuck() const { return stuck; }

  private:
    CollisionData data;
    bool stuck;

    //BE WARNED! YOU MUST CALL THIS FUNCTION BEFORE ANYTHING ELSE!
    static const int StaticInit();
    static const int dummy_init;	
};

#endif
