#ifndef FF_CONST_H
#define FF_CONST_H

#include <iostream>
#include <fstream>
#include <map>
#include <string>
#include <cmath>
#include "Primitive.H"
#include "FFPrimitive.H"

namespace {
  std::map<std::string, double> nameValueMap;
  int LoadConst() {
    std::string line;
    std::ifstream fname("ff_const.names");
    std::ifstream fvalue("ff_const.values");
    nameValueMap.clear();
    while(getline(fname, line))
      fvalue >> nameValueMap[line];
    return 0; // Done
  }

  double GetConst(std::string name) {
    static int DUMMY_LOADER = LoadConst();
    ++DUMMY_LOADER; // Just to get rid of warning
    if (nameValueMap.find(name) == nameValueMap.end())
      throw "Unknown constant name";
    return nameValueMap[name];
  }

  const double FORCE_THRESHOLD = GetConst("FORCE_THRESHOLD");
  const double GEAR1_THRESHOLD = GetConst("GEAR1_THRESHOLD");
  const double GEAR2_THRESHOLD = GetConst("GEAR2_THRESHOLD");
  const double DENSITY_CHECK_THRESHOLD = GetConst("DENSITY_CHECK_THRESHOLD");
  const double SHAKE_MULTIPLIER = GetConst("SHAKE_MULTIPLIER");
  const double SHAKE_POWER = GetConst("SHAKE_POWER");
  const double REVERSE_SHAKE_MULTIPLIER = GetConst("REVERSE_SHAKE_MULTIPLIER");
  const double REVERSE_SHAKE_POWER = GetConst("REVERSE_SHAKE_POWER");
  const double TERRAIN_MAX_EFFECTIVE_RANGE = GetConst("TERRAIN_MAX_EFFECTIVE_RANGE");
  const double TERRAIN_POWER = GetConst("TERRAIN_POWER");
  const double TERRAIN_MULTIPLIER = GetConst("TERRAIN_MULTIPLIER");
  const double TERRAIN_TRIANGLE_MAX_EFFECTIVE_RANGE = GetConst("TERRAIN_TRIANGLE_MAX_EFFECTIVE_RANGE");
  const double TERRAIN_TRIANGLE_POWER = GetConst("TERRAIN_TRIANGLE_POWER");
  const double TERRAIN_TRIANGLE_MULTIPLIER = GetConst("TERRAIN_TRIANGLE_MULTIPLIER");
  const double FRIENDLY_MAX_EFFECTIVE_RANGE = GetConst("FRIENDLY_MAX_EFFECTIVE_RANGE");
  const double FRIENDLY_POWER = GetConst("FRIENDLY_POWER");
  const double FRIENDLY_MULTIPLIER = GetConst("FRIENDLY_MULTIPLIER");
  const double STATIC_MAX_EFFECTIVE_RANGE = GetConst("STATIC_MAX_EFFECTIVE_RANGE");
  const double STATIC_POWER = GetConst("STATIC_POWER");
  const double STATIC_MULTIPLIER = GetConst("STATIC_MULTIPLIER");
  const double ENEMY_MAX_EFFECTIVE_RANGE = GetConst("ENEMY_MAX_EFFECTIVE_RANGE");
  const double ENEMY_POWER = GetConst("ENEMY_POWER");
  const double ENEMY_MULTIPLIER = GetConst("ENEMY_MULTIPLIER");
  const double NEUTRAL_MAX_EFFECTIVE_RANGE = GetConst("NEUTRAL_MAX_EFFECTIVE_RANGE");
  const double NEUTRAL_POWER = GetConst("NEUTRAL_POWER");
  const double NEUTRAL_MULTIPLIER = GetConst("NEUTRAL_MULTIPLIER");
  const double DEST_MIN_EFFECTIVE_RANGE = GetConst("DEST_MIN_EFFECTIVE_RANGE");
  const double DEST_POWER = GetConst("DEST_POWER");
  const double DEST_COEF = GetConst("DEST_COEF");
  const double DEST_MIN_DRAG = GetConst("DEST_MIN_DRAG");
  const double INERTIA_BUILD_UP = GetConst("INERTIA_BUILD_UP");
  const double INERTIA_DIE_DOWN = GetConst("INERTIA_DIE_DOWN");
  const double INERTIA_MINIMUM = GetConst("INERTIA_MINIMUM");
  const double INERTIA_MAXIMUM = GetConst("INERTIA_MAXIMUM");

  const double MIN_DIST_OFFSET = 0.12;

  const double PI = acos(-1.0);
  const double HALF_PI = 0.5 * PI;
  const double QUARTER_PI = 0.25 * PI;
  const double PI2 = 2 * PI;
  const double SQRT2 = sqrt(2.0);
  const double INV_SQRT2 = 1.0 / SQRT2;
  const double EPS = 1e-5;

  const Force ZERO(0,0);
  const Force N_FORCE(0, -1);
  const Force E_FORCE(1, 0);
  const Force S_FORCE(0, 1);
  const Force W_FORCE(-1, 0);
  const Force NE_FORCE(INV_SQRT2, -INV_SQRT2);
  const Force SE_FORCE(INV_SQRT2, INV_SQRT2);
  const Force SW_FORCE(-INV_SQRT2, INV_SQRT2);
  const Force NW_FORCE(-INV_SQRT2, -INV_SQRT2);
}

#endif
