#include "DebugHeader.H"
#include "GlobalDataBase.H"
#include "MapBox.H"
#include "ServerObjData.H"
#include "XTerrainEngine.H"

namespace {
  const int INIT_PLAYER_TANK_COUNT = 50;
  const int INIT_ENEMY_TANK_COUNT  = 50;
  const int INIT_PLAYER_CC_COUNT   =  5;
  const int INIT_ENEMY_CC_COUNT    =  5;
  // This figure is not at all valid...
  const int INIT_NEUTRAL_OBJ_COUNT = 128;
}

const int GlobalDataBase::dummy = StaticInit();
const Game* GlobalDataBase::game;

PlayerTank*  GlobalDataBase::playerTanks[64];
PlayerTank** GlobalDataBase::playerTanksEnd;

EnemyTank*  GlobalDataBase::enemyTanks[64];
EnemyTank** GlobalDataBase::enemyTanksEnd;

NeutralObj*  GlobalDataBase::neutralObjs[128];
NeutralObj** GlobalDataBase::neutralObjsEnd;

PlayerCC*  GlobalDataBase::playerCCs[8];
PlayerCC** GlobalDataBase::playerCCsEnd;

EnemyCC*  GlobalDataBase::enemyCCs[8];
EnemyCC** GlobalDataBase::enemyCCsEnd;

EnemyObj*  GlobalDataBase::enemyUnits[64];
EnemyObj** GlobalDataBase::enemyUnitsEnd;

// Nobody is supposed to access the graveyard anyway...
GameObjWrapper* GlobalDataBase::graveyard[128];
int GlobalDataBase::corpseCount;

Random* GlobalDataBase::random;

sint4 GlobalDataBase::gameTick;
sint4 GlobalDataBase::playerID;
sint4 GlobalDataBase::enemyID;


#define FORI(n) for (int i = 0; i < (n); ++i)

int GlobalDataBase::StaticInit() {
  memset(neutralObjs, 0, sizeof(neutralObjs));
  memset(playerTanks, 0, sizeof(playerTanks));
  memset(enemyTanks, 0, sizeof(enemyTanks));
  memset(playerCCs, 0, sizeof(playerCCs));
  memset(enemyCCs, 0, sizeof(enemyCCs));

  FORI(INIT_PLAYER_TANK_COUNT) {
    playerTanks[i] = new PlayerTank();
    playerTanks[i]->arrayIndex = i;
  }
  playerTanksEnd = playerTanks + INIT_PLAYER_TANK_COUNT;

  FORI(INIT_ENEMY_TANK_COUNT) {
    enemyUnits[i] = enemyTanks[i] = new EnemyTank();
    enemyTanks[i]->enemyIndex = enemyTanks[i]->arrayIndex = i;
  }
  enemyTanksEnd = enemyTanks + INIT_ENEMY_TANK_COUNT;

  FORI(INIT_PLAYER_CC_COUNT) {
    playerCCs[i] = new PlayerCC();
    playerCCs[i]->arrayIndex = i;
  }
  playerCCsEnd = playerCCs + INIT_PLAYER_CC_COUNT;

  FORI(INIT_ENEMY_CC_COUNT) {
    enemyCCs[i] = new EnemyCC();
    enemyCCs[i]->arrayIndex = i;
    enemyUnits[INIT_ENEMY_TANK_COUNT + i] = enemyCCs[i];
    enemyCCs[i]->enemyIndex = INIT_ENEMY_TANK_COUNT + i;
  }
  enemyCCsEnd = enemyCCs + INIT_ENEMY_CC_COUNT;

  FORI(INIT_NEUTRAL_OBJ_COUNT) {
    neutralObjs[i] = new NeutralObj();
    neutralObjs[i]->arrayIndex = i;
  }
  neutralObjsEnd = neutralObjs + INIT_NEUTRAL_OBJ_COUNT;

  enemyUnitsEnd = enemyUnits + INIT_ENEMY_TANK_COUNT + INIT_ENEMY_CC_COUNT;

  return 0;
}

void GlobalDataBase::InitGame(const Game* game_) {
  game = game_;
  gameTick = game->tick();
  const Map<GameTile> &map = game->get_map();
  MapBox::Init(map);
  for (int r = 0; r < 64; ++r)
    for (int c = 0; c < 64; ++c)
      XTerrainEngine::Add(r, c, map(c,r) );

  playerID = game->get_client_player();
  enemyID = 1 - playerID;

  playerTanksEnd = playerTanks;
  enemyTanksEnd = enemyTanks;
  playerCCsEnd = playerCCs;
  enemyCCsEnd = enemyCCs;
  neutralObjsEnd = neutralObjs;
  enemyUnitsEnd = enemyUnits;


  const Game::ObjCont &player_objs = game->get_objs(playerID);
  const Game::ObjCont &enemy_objs = game->get_objs(enemyID);

  FORALL (player_objs, it) {
    GameObj* game_obj = (*it)->get_GameObj();
    if (!game_obj->sod.in_game) continue;

    if ((game_obj->bp_name() == "controlCenter") && *game_obj->sod.x1 >0)
      (*(playerCCsEnd++))->Init(game_obj);
    else if ((game_obj->bp_name() == "tank") && *game_obj->sod.x > 0)
      (*(playerTanksEnd++))->Init(game_obj);
  }

  FORALL (enemy_objs, it) {
    GameObj* game_obj = (*it)->get_GameObj();
    if (!game_obj->sod.in_game) continue;

    if ((game_obj->bp_name() == "controlCenter") && *game_obj->sod.x1 >0) {
      (*(enemyCCsEnd++))->Init(game_obj);
      ++enemyUnitsEnd;
    } else if ((game_obj->bp_name() == "tank") && *game_obj->sod.x > 0) {
      (*(enemyTanksEnd++))->Init(game_obj);
      ++enemyUnitsEnd;
    }
  }

  const sint4 npcid = game->get_player_num();
  const Game::ObjCont &npc_objs = game->get_objs(npcid);
  FORALL (npc_objs, it) {
    GameObj *game_obj = (*it)->get_GameObj();
    if (game_obj->bp_name() == "sheep")
      (*(neutralObjsEnd++))->Init(game_obj);
  }

}

  void GlobalDataBase::CleanUp() {
    while (playerTanksEnd != playerTanks)
      delete *(--playerTanksEnd);
    while (playerCCsEnd != playerCCs)
      delete *(--playerCCsEnd);
    while (enemyTanksEnd != enemyTanks)
      delete *(--enemyTanksEnd);
    while (enemyCCsEnd != enemyCCs)
      delete *(--enemyCCsEnd);
    while (neutralObjsEnd != neutralObjs)
      delete *(--neutralObjsEnd);

    for (int i = corpseCount; i--; ) delete graveyard[i];
  }

namespace {
  // How do I specify such that this only accepts derived classes of GameObjWrapper
  template<class T>
    inline void RemoveObj(T** array, int& count, GameObjWrapper* obj) {
      int idx = obj->arrayIndex;
      array[idx] = array[--count];
      array[idx]->arrayIndex = idx;
    }
}

void GlobalDataBase::Update(const GameChanges& game_changes) {
  gameTick = game->tick();

  // Deal with the changed objects
  const Vector<ScriptObj::WPtr>& changed_objs = game_changes.changed_objs;
  FORALL(changed_objs, it) {
    GameObj* game_obj = (*it)->get_GameObj();
    if (!game_obj) continue;
    GameObjWrapper* game_obj_wrapper = GameObjWrapper::GetWrapper(game_obj);
    if (game_obj_wrapper)
      // I am ignoring that I may want to check for death here
      game_obj_wrapper->Update();
  }

  // Deal with dead objects
  int tmp_index;
  const Vector<ScriptObj::WPtr>& dead_objs = game_changes.dead_objs;
  FORALL(dead_objs, it) {
    GameObj* game_obj = (*it)->get_GameObj();
    if (!game_obj) continue;
    GameObjWrapper* game_obj_wrapper = GameObjWrapper::GetWrapper(game_obj);

    tmp_index = game_obj_wrapper->arrayIndex;

    // Check what this is
    // I don't care about sheeps....
    if (*game_obj->sod.owner == playerID) {
      // Mine
      if (game_obj->bp_name() == "controlCenter") {
        playerCCs[tmp_index] = *(--playerCCsEnd);
        playerCCs[tmp_index]->arrayIndex = tmp_index;
      } else {
        playerTanks[tmp_index] = *(--playerTanksEnd);
        playerTanks[tmp_index]->arrayIndex = tmp_index;
      }
    } else {
      // Enemy
      if (game_obj->bp_name() == "controlCenter") {
        enemyCCs[tmp_index] = *(--enemyCCsEnd);
        enemyCCs[tmp_index]->arrayIndex = tmp_index;
      } else {
        enemyTanks[tmp_index] = *(--enemyTanksEnd);
        enemyTanks[tmp_index]->arrayIndex = tmp_index;
      }
      // This unit should be removed from the enemy list
      tmp_index = ((EnemyObj*) game_obj_wrapper)->enemyIndex;
      enemyUnits[tmp_index] = *(--enemyUnitsEnd);
      enemyUnits[tmp_index]->enemyIndex = tmp_index;
    }
    game_obj_wrapper->arrayIndex = corpseCount;
    graveyard[corpseCount++] = game_obj_wrapper;
    game_obj_wrapper->CleanUp();
  }

}

void GlobalDataBase::Display() {
#ifdef DEBUG
  LOG(cerr << "Player CC info:" << endl);
  for (const PlayerCC* const* i = playerCCs, * const* ei = playerCCsEnd;
      i != ei; ++i)
    LOG(cerr << (*i)->GetWorldCoor() << "\thp = " << (*i)->GetHP() << endl);

  LOG(cerr << "Enemy CC info:" << endl);
  for (const EnemyCC* const* i = enemyCCs, * const*ei = enemyCCsEnd;
      i != ei; ++i)
    LOG(cerr << (*i)->GetWorldCoor() << "\thp = " << (*i)->GetHP() << endl);

  for (PlayerTank** i = playerTanks, **ei = playerTanksEnd;
      i != ei; ++i) {
    EnemyObj* target = (*i)->GetTarget();
    if (target != NULL)
      LOG(cerr << "Attacking: " << target->GetWorldCoor() << " with HP = " << target->GetHP() << endl);
  }
#endif
}
