#include "DebugHeader.H"
#include "GeneralAI.H"
#include <algorithm>

DEF_FERR("GeneralAI");

using std::max;
using std::min;
using std::size_t;
using std::abs;

namespace {
  const sint4 INITIAL_SQUAD_NUMBER = 5;
  const sint4 CLUSTER_MERGE_DIST = 128*128;

  const MapCoor NO_TABOO(-1, -1);
  int BIT_COUNT[1 << MAX_SQUAD_NUMBER];

  int StaticInit() {
    BIT_COUNT[0] = 0;
    int n = 1 << MAX_SQUAD_NUMBER;
    for (int i = 1; i < n; ++i)
      BIT_COUNT[i] = (i & 1) + BIT_COUNT[i >> 1];
    return 0;
  }

  const int dummy = StaticInit();
}

#define CLEAR_DEAD(v) for (size_t i = 0; i < v.size(); ) if (!v[i]->IsAlive()) { v[i] = v.back(); v.pop_back(); } else ++i;

GeneralAI::GeneralAI() {
  clustering = new Clustering();
  memset(engagingMap, 0, MAX_SQUAD_NUMBER * MAX_SQUAD_NUMBER); 
  tabooCount = 0;
}

GeneralAI::~GeneralAI() {
  delete clustering;
  //TODO: delete all squads here
  for (int i = 0; i < squadAI.size(); i++) { 
    delete squadAI.at(i);
  }
}

void GeneralAI::Init() {
//1. Initially assign tanks to squads
  cerr << "Before everything in Init()" << endl;
  for (int i = 0; i < INITIAL_SQUAD_NUMBER; i++) {
    SquadAI* squad_ai = new SquadAI();
    squad_ai->SetID(i);
    squadAI.push_back(squad_ai);
  }
  //Decide which tank goes to which squad
  cerr << "Decide which tank goes to which squad" << endl;
  PlayerTank** player_tank = GlobalDataBase::GetPlayerTanks();
  clustering->InitAssignClosest(player_tank, GlobalDataBase::GetPlayerTankCount(), GlobalDataBase::GetPlayerCCs(), INITIAL_SQUAD_NUMBER);
  //Need to add tanks to each squad
  cerr << "Add tanks to each squad" << endl;
  for (int i = 0; i < GlobalDataBase::GetPlayerTankCount(); i++) {
    for (int j = 0; j < INITIAL_SQUAD_NUMBER; j++) {
      if (player_tank[i]->GetClusterID() == squadAI[j]->GetID()) {
        squadAI[j]->AddTank(player_tank[i]);
        break;
      }
    }
  }

  for (int j = 0; j < squadAI.size(); j++) {
    cerr << "Squad " << j << " contains: "; 
    for (int i = 0; i < GlobalDataBase::GetPlayerTankCount(); i++) {
      if (player_tank[i]->GetClusterID()== j ) {
        cerr << "(" << player_tank[i]->GetWorldCoor().x << ", " << player_tank[i]->GetWorldCoor().y << "),";
      }
      //cerr << "Squad" << squadAI[j]->GetID() << " contains" << squadAI[j]->GetTanks().size() << "tanks" << endl;
    }
    cerr << endl;
  }

//2. CC distribution analysis
   

}

namespace {
  void EngageVisitPlayer(const int &player_index, int &player_group, int &enemy_group, const int *graph, const int &num_player, const int &num_enemy);
  void EngageVisitEnemy(const int &enemy_index, int &player_group, int &enemy_group, const int *graph, const int &num_player, const int &num_enemy);

  void EngageVisitPlayer(const int &player_index, int &player_group, int &enemy_group, const int *graph, const int &num_player, const int &num_enemy) {
    player_group |= (1 << player_index);
    const int *offseted = graph + player_index * MAX_SQUAD_NUMBER;
    for (int i = 0; i < num_enemy; ++i)
      if ( (offseted[i] == 1) && !(enemy_group & (1 << i)))
        EngageVisitEnemy(i, player_group, enemy_group, graph, num_player, num_enemy);
  }

  void EngageVisitEnemy(const int &enemy_index, int &player_group, int &enemy_group, const int *graph, const int &num_player, const int &num_enemy) {
    enemy_group |= (1 << enemy_index);
    for (int i = 0; i < num_player; ++i)
      if ( graph[i * MAX_SQUAD_NUMBER + enemy_index] == 1
          && !(player_group & (1 << i)))
        EngageVisitPlayer(i, player_group, enemy_group, graph, num_player, num_enemy);
  }
  
  /*
  void CheckBattleStatus();
  void CheckBattleStatus() {
    for(int i = 0, ei = squadAI.size(); i < ei; i++) {
      squadAI.at(i)->CheckBattleStatus();
    }
  }
  */
}


// Build Group

void GeneralAI::Update() {
  //Upate squads status
  CLEAR_DEAD(squadAI);
  //CheckBattleStatus();
  //1. Get updated enemy clustering information
  //1.1 Decide how many clusters of enemy unit
  int enemy_count = GlobalDataBase::GetEnemyTankCount();
  if (enemy_count <= 5) { //TODO: Temporary fix, any improved proposal will be considered
    return;
  }
  EnemyTank** enemy_tanks = GlobalDataBase::GetEnemyTanks();
  int k = INITIAL_SQUAD_NUMBER;
  clustering->KMedoids(enemy_tanks, enemy_count, medoidsObj, k, medoidsSize);
  //1.2 Enemy cluster post_analysis with medoids_obj and medoids_size
  // in order to:
  // 1) prevent overfitting, since we set the k arbitrarily
  // 2) make use of the information
  for (int i = 0; i < medoidsObj.size()-1; i++) { //TODO: This merge part needs to be rewritten
    for (int j = i+1; j < medoidsObj.size(); ) {
      if (medoidsObj.at(i)->GetWorldCoor().SqrDist(medoidsObj.at(j)->GetWorldCoor()) < CLUSTER_MERGE_DIST) { 
        cerr << "Their distance is: " << medoidsObj.at(i)->GetWorldCoor().SqrDist(medoidsObj.at(j)->GetWorldCoor()) << endl;
        cerr << "Merging cluster " << i << "(size="<< medoidsSize.at(i)<<") with cluster "<< j << "(size="<<medoidsSize.at(j) << ")"<<endl;
        clustering->Merge(enemy_tanks, medoidsObj, medoidsSize, i, j);
      } else ++j;
    }
  }

  cerr << "medoids_obj.size() = " << medoidsObj.size() << endl;
  for (int i = 0, ei = medoidsObj.size(); i < ei; i++) {
    cerr << "Cluster "<< i << "is centered around: ( " << medoidsObj.at(i)->GetWorldCoor().x << 
      ", " << medoidsObj.at(i)->GetWorldCoor().y << ")." << "Number of tanks: " << medoidsSize.at(i) << endl;  
  }

  // TODO: Remove this hack...

#ifdef OLD_AI
  //n. Make strategic decision
  memset(engagingMap, 0, MAX_SQUAD_NUMBER*MAX_SQUAD_NUMBER);
  for (int i = 0, ei = squadAI.size(); i < ei; i++) {
    squadAI.at(i)->EngagingEnemyCluster(engagingMap, MAX_SQUAD_NUMBER); 
  }

  cerr << "The engaging matrix is:" << endl;
  for (int i = 0, ei = squadAI.size(); i < ei; i++) {
    for (int j = 0, ej = medoidsObj.size(); j < ej; j++) {
      cerr << engagingMap[i*MAX_SQUAD_NUMBER+j] << " ";
    }
    cerr << endl;
  }
#endif

  // Set taboo on where large enemy clusters locate
  MapCoor newTabooLeftTop[MAX_SQUAD_NUMBER];
  MapCoor newTabooRightBottom[MAX_SQUAD_NUMBER];
  int newTabooCount = 0, delta = 0;
  for (int i = 0, ei = medoidsObj.size(); i < ei; ++i)
    if (medoidsSize[i] >= 15) {
      delta = (medoidsSize[i] - 10) / 4 + 5;
      MapCoor center = medoidsObj[i]->GetMapCoor();
      newTabooLeftTop[newTabooCount]
        = MapCoor(max(center.r - delta, 0), max(center.c - delta, 0) );
      newTabooRightBottom[newTabooCount]
        = MapCoor(min(center.r + delta, 63), min(center.c + delta, 63));
      MapBox::AddTaboo(newTabooLeftTop[newTabooCount],
          newTabooRightBottom[newTabooCount]);
      ++newTabooCount;
    }
  // Delete all old ones
  while(tabooCount--) MapBox::RemoveTaboo(
      medoidsTabooLeftTop[tabooCount],
      medoidsTabooRightBottom[tabooCount]);
  tabooCount = newTabooCount;
  for (int i = 0; i < newTabooCount; ++i) {
    medoidsTabooLeftTop[i] = newTabooLeftTop[i];
    medoidsTabooRightBottom[i] = newTabooRightBottom[i];
  }
  
#ifdef OLD_AI

  int processed_player = 0;
  int processed_enemy = 0;
  int player_groups[MAX_SQUAD_NUMBER], enemy_groups[MAX_SQUAD_NUMBER];
  int player_group_sizes[MAX_SQUAD_NUMBER], enemy_group_sizes[MAX_SQUAD_NUMBER];
  int group_idx = 0;
  for (int i = 0, ei = squadAI.size(); i < ei; ++i) {
    // Have we dealt with this squad already?
    if (processed_player & (1 << i)) continue;
    player_groups[group_idx] = enemy_groups[group_idx] = 0;
    EngageVisitPlayer(i, player_groups[group_idx], enemy_groups[group_idx],
        engagingMap, squadAI.size(), medoidsObj.size());
    // So player_group and enemy_group are found
    processed_player |= player_groups[group_idx];
    processed_enemy |= enemy_groups[group_idx];
    // Calculate the squad sizes
    player_group_sizes[group_idx] = enemy_group_sizes[group_idx] = 0;
    for (int j = 0, ej = squadAI.size(); j < ej; ++j)
      if ( (1<<j) & player_groups[group_idx] ) // squad j in the group
        player_group_sizes[group_idx] += squadAI[j]->NumOfTanks();
    for (int j = 0, ej = medoidsObj.size(); j < ej; ++j)
      if ( (1<<j) & enemy_groups[group_idx] ) // Medoid j in the group
        enemy_group_sizes[group_idx] += medoidsSize[j];
    ++group_idx;
  }
  // Consider stray enemies
  for (int i = 0, ei = medoidsObj.size(); i < ei; ++i) {
    // Have we dealt with this squad already?
    if (processed_enemy & (1 << i)) continue;
    player_groups[group_idx] = 0;
    enemy_groups[group_idx] = (1 << i);
    //processed_enemy |= enemy_groups[group_idx]; // This is useless
    // Calculate the squad sizes
    player_group_sizes[group_idx] = 0;
    enemy_group_sizes[group_idx] = medoidsSize[i];
    ++group_idx;
  }
  

  // Group information and size found
  for (int i = 0; i < group_idx; i++) {
    if (!player_groups[i]) break;
    if (player_group_sizes[i] < enemy_group_sizes[i]) {
      //TODO: ask other available squads for help!!!!
      //if we have only one squad in this battle group, retreat
      if ( BIT_COUNT[player_groups[i]] == 1) {

      }
    }
  }
#endif
}



void GeneralAI::Act() {
  for (int i = 0; i < squadAI.size(); i++) {
    LOG(ferr << "Squad " << i << "::Act()" << endl);
    squadAI[i]->Act();
  }
}
