#ifndef FF_PRIMITIVE_H
#define FF_PRIMITIVE_H

#include "Primitive.H"
#include <complex>
#include <cmath>

namespace {

  typedef double real_type;
  typedef std::complex<real_type> Force; // Hopefully float is faster

  const int RESOLUTION = 8;
  const int MARINE_RADIUS = 4;
  const int NEUTRAL_RADIUS = 4;
  
  const int WORLD_WIDTH = 1024;
  const int WORLD_HEIGHT = 768;
  const int MAP_WIDTH = WORLD_WIDTH / RESOLUTION;
  const int MAP_HEIGHT = WORLD_HEIGHT / RESOLUTION;
  
  const int NEUTRAL_DISTANCE = 6 + MARINE_RADIUS + NEUTRAL_RADIUS;
  const int SQR_NEUTRAL_DISTANCE = NEUTRAL_DISTANCE * NEUTRAL_DISTANCE;
  const int PLAYER_DISTANCE = 4 * MARINE_RADIUS + MARINE_RADIUS;
  const int SQR_PLAYER_DISTANCE = PLAYER_DISTANCE * PLAYER_DISTANCE;
  const int EDGE_HULL_DISTANCE = 80;
  const int SQR_EDGE_HULL_DISTANCE
    = EDGE_HULL_DISTANCE * EDGE_HULL_DISTANCE;
  const int CORNER_HULL_DISTANCE = 68;
  const int SQR_CORNER_HULL_DISTANCE
    = CORNER_HULL_DISTANCE * CORNER_HULL_DISTANCE;
  const int WALL_DISTANCE = 5 + MARINE_RADIUS;

  const real_type PI = acos(-1.0);
  const real_type RANDOM_ANGLE = PI / 6;

  const real_type NEUTRAL_STRENGTH = 2.0;
  const real_type PLAYER_STRENGTH = 1.2;
  const real_type HULL_STRENGTH = 2.8;
  const real_type IN_HULL_STRENGTH = 4.5;
  const real_type DEST_STRENGTH = 1.0;
  const real_type WALL_STRENGTH = 7.0;
  const real_type CURL_STRENGTH = 0.8;
  const real_type CORNER_STRENGTH = 5.6;

  const real_type SPEED_1 = 0.001;
  const real_type SPEED_2 = 0.15;
  const real_type SPEED_3 = 0.35;
  
  const real_type EPS = 0.001;

  const Force ZERO(0, 0);
  const Force N_FORCE(0, -1);
  const Force E_FORCE(1, 0);
  const Force S_FORCE(0, 1);
  const Force W_FORCE(-1, 0);

  MapCoor WorldToMap(const WorldCoor& coor) {
    return MapCoor( coor.y / RESOLUTION, coor.x / RESOLUTION );
  }

}
#endif
