#include "GeneralAI.H"
#include <algorithm>

using std::max;
using std::min;
using std::size_t;
using std::abs;

namespace {
  const sint4 INITIAL_SQUAD_NUMBER = 5;
  const sint4 CLUSTER_MERGE_DIST = 64*64;

  const WorldCoor NULL_COOR(-1, -1);
  int BIT_COUNT[1 << MAX_SQUAD_NUMBER];

  int StaticInit() {
    BIT_COUNT[0] = 0;
    int n = 1 << MAX_SQUAD_NUMBER;
    for (int i = 1; i < n; ++i)
      BIT_COUNT[i] = (i & 1) + BIT_COUNT[i >> 1];
    return 0;
  }

  const int dummy = StaticInit();
}

#define CLEAR_DEAD(v) for (size_t i = 0; i < v.size(); ) if (!v[i]->IsAlive()) { v[i] = v.back(); v.pop_back(); } else ++i;

GeneralAI::GeneralAI(GlobalDataBase* global_data_base):globalDataBase(global_data_base) {
  //memset(engagingMap, 0, MAX_SQUAD_NUMBER * MAX_SQUAD_NUMBER); 
  //tabooCount = 0;
}

GeneralAI::~GeneralAI() {
  //TODO: delete all squads here
  for (int i = 0; i < squads.size(); i++) { 
    delete squads.at(i);
  }
}

void GeneralAI::AssignPlayerSquad() {
  PlayerMarine** player_marine = globalDataBase->GetPlayerMarines();
  for (int i = 0; i < globalDataBase->GetPlayerMarineCount(); i++)
    squads[i%5]->AddMarine(player_marine[i]);
    //player_marine[i]->SetSquad(squads[i%5]);
}

void GeneralAI::InitialAssemble() {
  assembling = false;
  return;
  assembling = true;
  PlayerMarine** player_marine = globalDataBase->GetPlayerMarines();
  int count  = globalDataBase->GetPlayerMarineCount();
  WorldCoor pos(0,0);
  for (int j = 0; j < count; j++) {
    pos.x += player_marine[j]->GetWorldCoor().x;
    pos.y += player_marine[j]->GetWorldCoor().y;
  }
  pos.x /= count;
  pos.y /= count;
  FORALL(squads, it)
    (*it)->SetDest(pos);
}

void GeneralAI::FinishAssemble() {
  if (!assembling) return;
  assembling = false;
}

/*
namespace {
  WorldCoor GetRelativePos(int index, int orign);
  WorldCoor GetRelativePos(int index, int orign) {
  }
}
*/

void GeneralAI::Init() {
//1. Initially assign marines to squads
  cerr << "Before everything in Init()" << endl;
  for (int i = 0; i < INITIAL_SQUAD_NUMBER; i++) {
    SquadAI* squad_ai = new SquadAI(globalDataBase);
    squad_ai->SetID(i);
    squads.push_back(squad_ai);
  }
  //Decide which marine goes to which squad
  cerr << "Decide which marine goes to which squad" << endl;
  //PlayerMarine** player_marine = globalDataBase->GetPlayerMarines();
  //clustering->InitAssignClosest(player_marine, globalDataBase->GetPlayerMarineCount(), globalDataBase->GetPlayerCCs(), INITIAL_SQUAD_NUMBER);
  AssignPlayerSquad();  //TODO: Temporary fix, assign marines to squads according to their index in for loop.
 
  //Need to add marines to each squad
  /*
  cerr << "Add marines to each squad" << endl;
  for (int i = 0; i < globalDataBase->GetPlayerMarineCount(); i++) {
    for (int j = 0; j < INITIAL_SQUAD_NUMBER; j++) {
      if (player_marine[i]->GetSquad() == squads[j]) {
        squads[j]->AddMarine(player_marine[i]);
        break;
      }
    }
  }

  for (int j = 0; j < squads.size(); j++) {
    cerr << "Squad " << j << " contains: "; 
    for (int i = 0; i < globalDataBase->GetPlayerMarineCount(); i++) {
      if (player_marine[i]->GetSquad() == squads[j] ) {
        cerr << "(" << player_marine[i]->GetWorldCoor().x << ", " << player_marine[i]->GetWorldCoor().y << "),";
      }
      //cerr << "Squad" << squads[j]->GetID() << " contains" << squads[j]->GetMarines().size() << "marines" << endl;
    }
    cerr << endl;
  }
  */

  InitialAssemble();
  /*

  //Select a marine in the center as the assemble target
  WorldCoor assemble_center;
  for (int i = 0; i < globalDataBase->GetPlayerMarineCount(); i++) {
    WorldCoor pos = player_marine[i]->GetWorldCoor();
    if (pos.y > 330 && pos.y < 430) {
      assemble_center = pos;
      player_marine[i]->AchieveInit();
      break;
    }
  }
  //Determine the relative position of the assemble target in the group
  int index = 0;
  for (int i = 0; i < globalDataBase->GetPlayerMarineCount(); i++) {
    if (player_marine[i]->GetWorldCoor().y <= assemble_center.y) {
      index++;
    }
  }
  */
}

namespace {
  void EngageVisitPlayer(const int &player_index, int &player_group, int &enemy_group, const int *graph, const int &num_player, const int &num_enemy);
  void EngageVisitEnemy(const int &enemy_index, int &player_group, int &enemy_group, const int *graph, const int &num_player, const int &num_enemy);

  void EngageVisitPlayer(const int &player_index, int &player_group, int &enemy_group, const int *graph, const int &num_player, const int &num_enemy) {
    player_group |= (1 << player_index);
    const int *offseted = graph + player_index * MAX_SQUAD_NUMBER;
    for (int i = 0; i < num_enemy; ++i)
      if ( (offseted[i] == 1) && !(enemy_group & (1 << i)))
        EngageVisitEnemy(i, player_group, enemy_group, graph, num_player, num_enemy);
  }

  void EngageVisitEnemy(const int &enemy_index, int &player_group, int &enemy_group, const int *graph, const int &num_player, const int &num_enemy) {
    enemy_group |= (1 << enemy_index);
    for (int i = 0; i < num_player; ++i)
      if ( graph[i * MAX_SQUAD_NUMBER + enemy_index] == 1
          && !(player_group & (1 << i)))
        EngageVisitPlayer(i, player_group, enemy_group, graph, num_player, num_enemy);
  }
  

}

void GeneralAI::CheckBattleStatus() {
  for(int i = 0, ei = squads.size(); i < ei; i++) {
    squads.at(i)->CheckBattleStatus();
  }
}

// Build Group

void GeneralAI::Update() {
  //cerr << "Start Update()" << endl;
  //Upate squads status
  CLEAR_DEAD(squads);

  if (assembling) return;

  cerr << "We have " << globalDataBase->GetPlayerMarineCount() << " marines left" << endl;
  cerr << "They have " << globalDataBase->GetEnemyMarineCount() << " marines left" << endl;

  //CheckBattleStatus();
  //1. Get updated enemy clustering information
  //1.1 Decide how many clusters of enemy unit
  int enemy_count = globalDataBase->GetEnemyMarineCount();

  EnemyMarine** enemy_marines = globalDataBase->GetEnemyMarines();

  Clustering* clustering = globalDataBase->GetClustering();
  const Cluster* const* cluster_begin = clustering->GetClusters();
  const Cluster* const* cluster_end = clustering->GetClustersEnd();

  /*
  int stupid = 0;
  cerr << "We have " << clustering->GetClusterCount() << " clusters" << endl;
  for (Cluster** i = cluster_begin; i != cluster_end; ++i) {
    cerr << "Clustering " << (*i)->GetID() << " has " << (*i)->GetSize()
      << " centered at " << (*i)->GetCenterOfMass() << endl;
    stupid += (*i)->GetSize();
  }
  cerr << stupid << " units clustered / "
    << globalDataBase->GetEnemyMarineCount() << " in total" << endl;
   */

  sint4 dist = 0;
  sint4 min_dist = 99999999;
  WorldCoor ref, temp;
  for (size_t i = 0; i < squads.size(); i++) {
    min_dist = 99999999;
    temp = NULL_COOR;
    /*
    ref.x = ref.y = 0;
    int n = squads[i]->NumOfMarines();
    for (int j = 0; j < n; ++j) {
      ref.x += squads[i]->GetMarines()[j]->GetWorldCoor().x;
      ref.y += squads[i]->GetMarines()[j]->GetWorldCoor().y;
    }
    ref.x /= n;
    ref.y /= n;
    */
    ref = squads[i]->GetMarines()[0]->GetWorldCoor();

    for (const Cluster* const* j = cluster_begin; j != cluster_end; ++j) {
      WorldCoor center_of_mass = (*j)->GetCenterOfMass();
      sint4 factor = 0;
      if ( (*j)->GetVariance() )
        factor = 80000 / (*j)->GetVariance();

      for (const WorldCoor *vertex = (*j)->GetHull(),
          *vertex_end = (*j)->GetHullEnd();
          vertex != vertex_end; ++vertex) {
        dist = vertex->sqr_dist(ref)
          - factor * vertex->sqr_dist(center_of_mass);
        //dist = vertex->sqr_dist(ref);
        if (dist < min_dist) {
          min_dist = dist;
          temp = *vertex;
        }
      }
    }

    if ( temp != NULL_COOR )
      squads[i]->SetDest(temp);
  }  

  /*
  //n. Make strategic decision
  memset(engagingMap, 0, MAX_SQUAD_NUMBER*MAX_SQUAD_NUMBER);
  for (int i = 0, ei = squads.size(); i < ei; i++) {
  squads.at(i)->EngagingEnemyCluster(engagingMap + i * MAX_SQUAD_NUMBER); 
  }

  cerr << "The engaging matrix is:" << endl;
  for (int i = 0, ei = squads.size(); i < ei; i++) {
  for (int j = 0, ej = medoidsObj.size(); j < ej; j++) {
  cerr << engagingMap[i*MAX_SQUAD_NUMBER+j] << " ";
  }
  cerr << endl;
  }

  // Set taboo on where large enemy clusters locate
  MapCoor newTabooLeftTop[MAX_SQUAD_NUMBER];
  MapCoor newTabooRightBottom[MAX_SQUAD_NUMBER];
  int newTabooCount = 0, delta = 0;
  for (int i = 0, ei = medoidsObj.size(); i < ei; ++i)
  if (medoidsSize[i] >= 15) {
  delta = (medoidsSize[i] - 10) / 4 + 5;
  MapCoor center = medoidsObj[i]->GetMapCoor();
  newTabooLeftTop[newTabooCount]
  = MapCoor(max(center.r - delta, 0), max(center.c - delta, 0) );
  newTabooRightBottom[newTabooCount]
  = MapCoor(min(center.r + delta, 63), min(center.c + delta, 63));
  globalDataBase->GetMapBox()->AddTaboo(newTabooLeftTop[newTabooCount],
  newTabooRightBottom[newTabooCount]);
  ++newTabooCount;
  }
  // Delete all old ones
  while(tabooCount--) globalDataBase->GetMapBox()->RemoveTaboo(
  medoidsTabooLeftTop[tabooCount], medoidsTabooRightBottom[tabooCount]);
  tabooCount = newTabooCount;
  for (int i = 0; i < newTabooCount; ++i) {
  medoidsTabooLeftTop[i] = newTabooLeftTop[i];
  medoidsTabooRightBottom[i] = newTabooRightBottom[i];
  }

  int processed_player = 0;
  int processed_enemy = 0;
  int player_groups[MAX_SQUAD_NUMBER], enemy_groups[MAX_SQUAD_NUMBER];
  int player_group_sizes[MAX_SQUAD_NUMBER], enemy_group_sizes[MAX_SQUAD_NUMBER];
  int group_idx = 0;
  for (int i = 0, ei = squads.size(); i < ei; ++i) {
// Have we dealt with this squad already?
if (processed_player & (1 << i)) continue;
player_groups[group_idx] = enemy_groups[group_idx] = 0;
EngageVisitPlayer(i, player_groups[group_idx], enemy_groups[group_idx],
engagingMap, squads.size(), medoidsObj.size());
// So player_group and enemy_group are found
processed_player |= player_groups[group_idx];
processed_enemy |= enemy_groups[group_idx];
// Calculate the squad sizes
player_group_sizes[group_idx] = enemy_group_sizes[group_idx] = 0;
for (int j = 0, ej = squads.size(); j < ej; ++j)
if ( (1<<j) & player_groups[group_idx] ) // squad j in the group
player_group_sizes[group_idx] += squads[j]->NumOfMarines();
for (int j = 0, ej = medoidsObj.size(); j < ej; ++j)
if ( (1<<j) & enemy_groups[group_idx] ) // Medoid j in the group
enemy_group_sizes[group_idx] += medoidsSize[j];
++group_idx;
}
// Consider stray enemies
for (int i = 0, ei = medoidsObj.size(); i < ei; ++i) {
// Have we dealt with this squad already?
if (processed_enemy & (1 << i)) continue;
player_groups[group_idx] = 0;
enemy_groups[group_idx] = (1 << i);
//processed_enemy |= enemy_groups[group_idx]; // This is useless
// Calculate the squad sizes
player_group_sizes[group_idx] = 0;
enemy_group_sizes[group_idx] = medoidsSize[i];
++group_idx;
}


// Group information and size found
for (int i = 0; i < group_idx; i++) {
  if (!player_groups[i]) break;
  if (player_group_sizes[i] < enemy_group_sizes[i]) {
    //TODO: ask other available squads for help!!!!
    //if we have only one squad in this battle group, retreat
      if ( BIT_COUNT[player_groups[i]] == 1) {

      }
    }
  }
*/

  //cerr << "Finish Update()" << endl;
}



void GeneralAI::Act() {
  for (int i = 0; i < squads.size(); i++) {
    squads[i]->Act();
  }
}
