// $Id: game4_main.C,v 1.7 2007/03/15 10:37:47 orts_furtak Exp $

// This is an ORTS file (c) Michael Buro, licensed under the GPL

// this is the main file of the sample ORTS AI client
// options are specified by the opt.put lines below
// to test it start the ORTS server (bin/orts) and then run bin/sampleai [options]

// Game4EventHandler.C contains the code for accessing the game state

#include "Global.H"
#include "Options.H"
#include "GameStateModule.H"
#include "ShutDown.H"
#include "SDLinit.H"
#include "Game.H"

#include "Game4EventHandler.H"

using namespace std;

//===================================================================

static char *VERSION = "game4_ubc v0.1";

static const sint4 GFX_X = 900;  // graphics window size
static const sint4 GFX_Y = 740;

//===================================================================

void add_options()
{
  Options opt("Game4Client");
  opt.put("-cinfo", "display game state changes");
  opt.put("-tinfo", "display tile info");
  opt.put("-oinfo", "display object info");
  opt.put("-minfo", "display map info");  
}


//===================================================================

int main(int argc, char **argv)
{
  signals_shut_down(true);  
  
  // populate opt with options from various modules

  add_options();
  GameStateModule::Options::add();
  Game::Options::add();

  // process command line arguments
  if (Options::process(argc, argv, cerr, VERSION)) exit(20);

  // initialize SDL
  SDLinit::video_init();
  SDLinit::network_init();
  
  // populate option objects with command line values
  // and create modules
  
  GameStateModule::Options gsmo;
  GameStateModule gsm(gsmo);
 
  // change seed value to time(0) if 0

  sint4 seed;
  Options::get("-seed", seed);
  if (!seed) seed = time(0);
  Options::set("-seed", seed);
  cout << "seed=" << seed << endl;

  // gives event handler access to the state
  Game4AIState state(&gsm);

  Game4EventHandler sev(state, seed);

  // gsm events are handled in sev
  gsm.add_handler(&sev);
  
  // connect game state module to server
  if (!gsm.connect()) ERR("connection problems");

  while(!state.quit)
    if (!state.gsm->recv_view())
      SDL_Delay(1);
  return 0;
}
