#include "Clustering.H"
#include <algorithm>

/*
   Copyright 2007 Man Hon Chan, Zhangbuo Liu, Simon Suyadi
   This code is licensed under GPL
 */

namespace {
  /*
     const int SQR_THRESHOLD = 96*96;
     const int SQR_THRESHOLD2 = 136*136;
   */
  ///*
  const int SQR_THRESHOLD = 72*72;
  const int SQR_THRESHOLD2 = 96*96;
  //*/
  int parent[MAX_UNIT_COUNT];
  int size[MAX_UNIT_COUNT];
  int renaming[MAX_UNIT_COUNT];

  inline void InitUnionFind(int n) {
    for (int i = 0; i < n; ++i)
      size[parent[i] = i] = 1;
  }

  int Find(int x) {
    return (x == parent[x])?x:(parent[x] = Find(parent[x]));
  }

  inline void Merge(int x, int y) {
    x = Find(x); y = Find(y);
    if (x == y) return; // We are all happy
    if (size[x] >= size[y]) {
      parent[y] = x;
      size[x] += size[y];
      size[y] = 0;
    } else {
      parent[x] = y;
      size[y] += size[x];
      size[x] = 0;
    }
  }

  // For point sorting
  WorldCoor ref;

  // Check if a should turn counterclockwise to reach b
  bool AngularOrder(const WorldCoor& a, const WorldCoor& b) {
    int dx_a = a.x - ref.x, dy_a = a.y - ref.y;
    int dx_b = b.x - ref.x, dy_b = b.y - ref.y;
    // Check if this is a left turn
    int cross = dx_a * dy_b - dx_b * dy_a;
    if (cross > 0) return true;
    else if (cross < 0) return false;
    else return (dx_a * dx_a + dy_a * dy_a < dx_b * dx_b + dy_b * dy_b);
  }

  // Check if a->b->c is a left turn
  bool IsLeftTurn(const WorldCoor& a,
      const WorldCoor& b, const WorldCoor& c) {
    int dx_ab = b.x - a.x, dy_ab = b.y - a.y;
    int dx_bc = c.x - b.x, dy_bc = c.y - b.y;
    return dx_ab * dy_bc - dx_bc * dy_ab > 0;
  }
}

Clustering::Clustering(GlobalDataBase* global_data_base):globalDataBase(global_data_base), clusterCount(0)/*, largestCluster(NULL)*/ {
  // Perform all the clustering now
  initEnemyCount = globalDataBase->GetEnemyMarineCount();
  for (int i = 0; i < initEnemyCount; ++i)
    clusters[i] = new Cluster(i);
}

Clustering::~Clustering() {
  for (int i = 0; i < initEnemyCount; ++i)
    delete clusters[i];
}

void Clustering::Update() {
  int n = globalDataBase->GetEnemyMarineCount();
  InitUnionFind(n);

  int i = 0, j = 0;
  for (EnemyMarine** marine = globalDataBase->GetEnemyMarines(),
      **marine_end = globalDataBase->GetEnemyMarinesEnd();
      marine != marine_end; ++marine, ++i) {
    j = i+1;
    for (EnemyMarine** next = marine+1; next != marine_end; ++next, ++j)
      if ( (*marine)->GetWorldCoor().sqr_dist( (*next)->GetWorldCoor() )
          < SQR_THRESHOLD )
        // Merge them
        Merge(i, j);
  }

  // We can now find the parent group of each enemy unit
  // So we want to rename all group
  clusterCount = 0;
  for (int k = 0; k < n; ++k)
    if (size[k]) renaming[k] = clusterCount++;

  if (clusterCount > MAX_CLUSTER_COUNT) { // That many clusters? Try a slightly bigger threshold
    i = 0, j = 0;
    for (EnemyMarine** marine = globalDataBase->GetEnemyMarines(),
        **marine_end = globalDataBase->GetEnemyMarinesEnd();
        marine != marine_end; ++marine, ++i) {
      j = i+1;
      for (EnemyMarine** next = marine+1; next != marine_end; ++next, ++j)
        if ( (*marine)->GetWorldCoor().sqr_dist( (*next)->GetWorldCoor() )
            < SQR_THRESHOLD2 )
          // Merge them
          Merge(i, j);
    }

    clusterCount = 0;
    for (int k = 0; k < n; ++k)
      if (size[k]) renaming[k] = clusterCount++;
  }

  // Now if there are really too many clusters
  if ( clusterCount > MAX_CLUSTER_COUNT ) {
    int threshold = MAX_CLUSTER_COUNT;
    while (clusterCount > MAX_CLUSTER_COUNT) {
      threshold += MAX_CLUSTER_COUNT;
      i = 0, j = 0;
      for (EnemyMarine** marine = globalDataBase->GetEnemyMarines(),
          **marine_end = globalDataBase->GetEnemyMarinesEnd();
          marine != marine_end; ++marine, ++i) {
        j = i+1;
        for (EnemyMarine** next = marine+1; next != marine_end; ++next, ++j)
          if ( (*marine)->GetWorldCoor().sqr_dist( (*next)->GetWorldCoor() )
              < threshold )
            // Merge them
            Merge(i, j);
      }

      clusterCount = 0;
      for (int k = 0; k < n; ++k)
        if (size[k]) renaming[k] = clusterCount++;
    }
  }


  for (int k = 0; k < clusterCount; ++k) clusters[k]->Clear();

  // So we can rename all of them
  // Add them to the clusters

  i = 0;
  for (EnemyMarine** marine = globalDataBase->GetEnemyMarines(),
      **marine_end = globalDataBase->GetEnemyMarinesEnd();
      marine != marine_end; ++marine, ++i) {
    int cluster_id = renaming[ Find(i) ];
    clusters[cluster_id]->AddUnit(*marine);
  }

  // Now ask them to do whatever they have to
  //int largest_cluster_size = 0;
  for (int k = 0; k < clusterCount; ++k) {
    clusters[k]->CalcStat();
    /*if (clusters[k]->GetSize() > largest_cluster_size) {
      largest_cluster_size = clusters[k]->GetSize();
      largestCluster = clusters[k];
      }*/
  }
}

Cluster* Clustering::GetLargestCluster() {
  int largest_cluster_size = 0;
  Cluster* largestCluster = NULL;
  for (int k = 0; k < clusterCount; ++k) {
    if (clusters[k]->GetSize() > largest_cluster_size) {
      largest_cluster_size = clusters[k]->GetSize();
      largestCluster = clusters[k];
    }
  }
  return largestCluster;
}

// Clusters
Cluster::Cluster(int id_): id(id_), clusterSize(0), hullSize(0), centerOfMass(-1, -1) {}

void Cluster::Clear() {
  variance = clusterSize = hullSize = 0;
  centerOfMass = WorldCoor(-1, -1);
}

void Cluster::AddUnit(EnemyMarine* unit) {
  units[clusterSize++] = unit;
  unit->SetCluster(this);
}

// Find out all relevant information about this cluster
// Should be called after all members of this cluster is set
void Cluster::CalcStat() {
  // First find their center of mass
  int cum_x = 0, cum_y = 0;
  for (int i = 0; i < clusterSize; ++i) {
    cum_x += units[i]->GetWorldCoor().x;
    cum_y += units[i]->GetWorldCoor().y;
  }
  centerOfMass = WorldCoor( cum_x / clusterSize, cum_y / clusterSize );
  variance = 0;
  for (int i = 0; i < clusterSize; ++i)
    variance += centerOfMass.sqr_dist(units[i]->GetWorldCoor());

  // Find the convex hull
  // Find the leftest toppest point
  int idx = 0;
  ref = units[0]->GetWorldCoor();
  hull[0] = ref;
  for (int i = 1; i < clusterSize; ++i) {
    hull[i] = units[i]->GetWorldCoor();
    if (hull[i] < ref)
      ref = hull[idx = i];
  }

  // Sort the points in place
  if (idx) std::swap(hull[0], hull[idx]);
  std::sort(hull+1, hull+clusterSize, AngularOrder);

  // Add them to the hull
  hullSize = 1;
  for (int i = 1; i < clusterSize; ++i) {
    while(hullSize > 1
        && !IsLeftTurn( hull[hullSize-2], hull[hullSize-1], hull[i]) )
      --hullSize;
    hull[hullSize++] = hull[i];
  }

  // Hull found
}

