#include "FFData.H"
#include "FFPrimitive.H"
#include "GameObjWrapper.H"
#include "FFEngine.H"
#include <cmath>
#include <iostream>
/* 
   Copyright 2007 Man Hon Chan, Zhangbo Liu, Simon Suyadi
   This code is Licensed under GPL.
 */

FFData::FFData(FFEngine* ff_engine, GameObjWrapper* game_obj_wrapper):ffEngine(ff_engine), gameObjWrapper(game_obj_wrapper) {
  worldCoor = gameObjWrapper->GetWorldCoor();
  mapCoor = WorldToMap(worldCoor);
  // Add myself to the engine
  ffEngine->Add(mapCoor, this);
}

FFData::~FFData() {
  ffEngine->Remove(mapCoor, this);
}

void FFData::Update() {
  worldCoor = gameObjWrapper->GetWorldCoor();
  MapCoor newMapCoor(WorldToMap(worldCoor));
  if (newMapCoor != mapCoor) {
    ffEngine->Remove(mapCoor, this);
    mapCoor = newMapCoor;
    ffEngine->Add(newMapCoor, this);
  }
}

FFPlayerData::FFPlayerData(FFEngine* ff_engine, GameObjWrapper* game_obj_wrapper):FFData(ff_engine, game_obj_wrapper) {}

void FFPlayerData::AddForce(const FFPlayerData* const &target,
    Force& player_force, Force& neutral_force) {
  int dx = target->worldCoor.x - worldCoor.x;
  int dy = target->worldCoor.y - worldCoor.y;
  int dist = dx * dx + dy * dy;
  if ( dist > SQR_PLAYER_DISTANCE ) return;
  player_force += Force(dx, dy) / sqrt(dist);
}

FFNeutralData::FFNeutralData(FFEngine* ff_engine, GameObjWrapper* game_obj_wrapper):FFData(ff_engine, game_obj_wrapper) {}

void FFNeutralData::AddForce(const FFPlayerData* const &target,
    Force& player_force, Force& neutral_force) {
  int dx = target->worldCoor.x - worldCoor.x;
  int dy = target->worldCoor.y - worldCoor.y;
  int dist = dx * dx + dy * dy;
  if ( dist > SQR_NEUTRAL_DISTANCE ) return;
  neutral_force += Force(dx, dy) / sqrt(dist);
}
