#include "GameObjWrapper.H"
#include "Game.H"
#include "ServerObjData.H"
#include "PlayerMarine.H"
#include "EnemyMarine.H"
#include "NeutralObj.H"
#include "GlobalDataBase.H"
#include "FFData.H"
/* 
   Copyright 2007 Man Hon Chan, Zhangbo Liu, Simon Suyadi
   This code is Licensed under GPL.
 */

namespace {
  template<typename T>
    class HashFunc_Pointer {
      public:
        inline size_t operator() (const T* const &x) const {
          return (size_t) x;
        }
    };

  EXT_NAMESPACE::hash_map<GameObj*, GameObjWrapper*, HashFunc_Pointer<GameObj> > gameObjMap;
}

GameObjWrapper::GameObjWrapper(GameObj* game_obj, GlobalDataBase* global_data_base):gameObj(game_obj){//, mapBox(global_data_base->GetMapBox()) {
  gameObjMap[gameObj] = this;
  ffData = NULL;
  Update(); // Update all information for this object
}

bool GameObjWrapper::Update() {
  // We assume that this function is not called if the object is dead
  dead = gameObj->is_dead() || (!gameObj->sod.in_game);

  worldCoor.x = *gameObj->sod.x;
  worldCoor.y = *gameObj->sod.y;

  //mapCoor = mapBox->GetMapCoor(worldCoor);
  return !dead;
}

GameObjWrapper::~GameObjWrapper() {
  gameObjMap.erase(gameObj);
}

GameObjWrapper* GameObjWrapper::GetWrapper(GameObj* game_obj, GlobalDataBase* global_data_base) {
  if (gameObjMap.find(game_obj) != gameObjMap.end())
    return gameObjMap[game_obj];

  // Create a suitable wrapper for this.
  // Now look at what it is
  sint4 owner = *game_obj->sod.owner;

  if (owner == global_data_base->GetPlayerID()) {
    if (game_obj->bp_name() == "marine")
      return new PlayerMarine(game_obj, global_data_base);
  } else if (owner == global_data_base->GetEnemyID()) {
    if (game_obj->bp_name() == "marine")
      return new EnemyMarine(game_obj, global_data_base);
  } else {
    return new NeutralObj(game_obj, global_data_base);
  }
  return NULL;
}

void GameObjWrapper::CleanUp() {
  dead = true; // This unit is dead.
  if (ffData) {
    delete ffData;
    ffData = NULL;
  }
}
