#include "GeneralAI.H"
#include <algorithm>
/* 
   Copyright 2007 Man Hon Chan, Zhangbo Liu, Simon Suyadi
   This code is Licensed under GPL.
 */

using std::max;
using std::min;
using std::size_t;
using std::abs;

namespace {

  const WorldCoor NULL_COOR(-1, -1);
  const int WIDTH = 1024;
  const int HEIGHT = 768;
  int BIT_COUNT[1 << 10];

  int StaticInit() {
    BIT_COUNT[0] = 0;
    int n = 1 << 10;
    for (int i = 1; i < n; ++i)
      BIT_COUNT[i] = (i & 1) + BIT_COUNT[i >> 1];
    return 0;
  }

  bool OrderByY(const WorldCoor& a, const WorldCoor& b) {
    if (a.y != b.y) return a.y < b.y;
    return a.x < b.x;
  }

  const int dummy = StaticInit();
}

#define CLEAR_DEAD(v) for (size_t i = 0; i < v.size(); ) if (!v[i]->IsAlive()) { v[i] = v.back(); v.pop_back(); } else ++i;

GeneralAI::GeneralAI(GlobalDataBase* global_data_base):globalDataBase(global_data_base) {
  //memset(engagingMap, 0, MAX_SQUAD_NUMBER * MAX_SQUAD_NUMBER); 
  //tabooCount = 0;
  regrouping = new Regrouping(globalDataBase);
}

GeneralAI::~GeneralAI() {
  //TODO: delete all squads here
  for (int i = 0; i < squads.size(); i++) { 
    delete squads.at(i);
  }
}

void GeneralAI::AssignPlayerSquad() {
  // We want to sort them by y
  // More like sort them by their world coordinate
  WorldCoor all_coor[50];
  PlayerMarine** player_marine = globalDataBase->GetPlayerMarines();
  int n = globalDataBase->GetPlayerMarineCount();
  for (int i = 0; i < n; i++)
    all_coor[i] = player_marine[i]->GetWorldCoor();
  //cerr << "There are initially " << n << " marines" << endl;
  std::sort(all_coor, all_coor + n, OrderByY);
  // We want five squads
  WorldCoor div[4] = {all_coor[10], all_coor[20], all_coor[30], all_coor[40], };

  for (int i = 0; i < n; ++i) { WorldCoor pos = player_marine[i]->GetWorldCoor();
    int squad_id = 0;
    while( squad_id < 4 && OrderByY(div[squad_id], pos) ) ++squad_id;
    squads[squad_id]->AddMarine(player_marine[i]);
  }
  squads[0]->SetMissionType(Intercept);
  squads[4]->SetMissionType(Intercept);

  //Test
  /*
  cerr << "Before spliting" << endl;
  for (int i = 0, ei = squads.size(); i < ei; ++i) {
  cerr << "Squad " << squads[i]->GetID() << " has " << squads[i]->NumOfMarines() << " marines" << endl;
  }
  for (int i = 0, ei = 5; i < ei; ++i) {
    SplitSquad(squads[i], 5);
  }
  cerr << "After spliting" << endl;
  for (int i = 0, ei = squads.size(); i < ei; ++i) {
  cerr << "Squad " << squads[i]->GetID() << " has " << squads[i]->NumOfMarines() << " marines" << endl;
  }
  
  cerr << "Then we merge....\n" << "---------------------" << endl;
  cerr << "After this, we have" << endl; 
  for (int i = 0; i < 5; i++) {
    MergeSquad(squads[0], squads[1]);
  }
  for (int i = 0; i < squads.size(); ++i) {
    cerr << "Squad " << i << " with " << squads[i]->NumOfMarines() << " marines." << endl;
  }
  */
}

void GeneralAI::InitialAssemble() {

  const WorldCoor FORMATION[] = {
    WorldCoor( 160, 144),
    WorldCoor( 130, 264),
    WorldCoor( 160, 384),
    WorldCoor( 130, 504),
    WorldCoor( 160, 624),
  };

  int idx = 0;
  assembling = true;
  FORALL(squads, it) {
    /*
    WorldCoor pos(0,0);
    cerr << "Squad has " << (*it)->NumOfMarines() << " marines" << endl;
    for (int j = (*it)->NumOfMarines(); j--; ) {
      pos.x += (*(*it)->GetMarines())[j]->GetWorldCoor().x;
      pos.y += (*(*it)->GetMarines())[j]->GetWorldCoor().y;
    }
    pos.x /= (*it)->NumOfMarines();
    pos.y /= (*it)->NumOfMarines();
    (*it)->SetDest(pos);
    cerr << "( " << pos.x << ", " << pos.y << " )" << endl;
    */
    WorldCoor tmp = FORMATION[idx++];
    if (globalDataBase->GetPlayerID() == 1)
      tmp.x = 1023 - tmp.x;
    (*it)->SetDest(tmp);
  }
}

void GeneralAI::FinishAssemble() {
  if (!assembling) return;
  assembling = false;
}

/*
namespace {
  WorldCoor GetRelativePos(int index, int orign);
  WorldCoor GetRelativePos(int index, int orign) {
  }
}
*/

void GeneralAI::Init() {
//1. Initially assign marines to squads
  cerr << "Before everything in Init()" << endl;
  for (int i = 0; i < INITIAL_SQUAD_NUMBER; i++) {
    SquadAI* squad_ai = new SquadAI(globalDataBase);
    squad_ai->SetID(i);
    squads.push_back(squad_ai);
  }
  //Decide which marine goes to which squad
  cerr << "Decide which marine goes to which squad" << endl;
  AssignPlayerSquad();  //TODO: Temporary fix, assign marines to squads according to their index in for loop.
 
  //Need to add marines to each squad
  /*
  cerr << "Add marines to each squad" << endl;
  for (int i = 0; i < globalDataBase->GetPlayerMarineCount(); i++) {
    for (int j = 0; j < INITIAL_SQUAD_NUMBER; j++) {
      if (player_marine[i]->GetSquad() == squads[j]) {
        squads[j]->AddMarine(player_marine[i]);
        break;
      }
    }
  }

  for (int j = 0; j < squads.size(); j++) {
    cerr << "Squad " << j << " contains: "; 
    for (int i = 0; i < globalDataBase->GetPlayerMarineCount(); i++) {
      if (player_marine[i]->GetSquad() == squads[j] ) {
        cerr << "(" << player_marine[i]->GetWorldCoor().x << ", " << player_marine[i]->GetWorldCoor().y << "),";
      }
      //cerr << "Squad" << squads[j]->GetID() << " contains" << squads[j]->GetMarines().size() << "marines" << endl;
    }
    cerr << endl;
  }
  */

  InitialAssemble();
  
  /*
  //Select a marine in the center as the assemble target
  WorldCoor assemble_center;
  for (int i = 0; i < globalDataBase->GetPlayerMarineCount(); i++) {
    WorldCoor pos = player_marine[i]->GetWorldCoor();
    if (pos.y > 330 && pos.y < 430) {
      assemble_center = pos;
      player_marine[i]->AchieveInit();
      break;
    }
  }
  //Determine the relative position of the assemble target in the group
  int index = 0;
  for (int i = 0; i < globalDataBase->GetPlayerMarineCount(); i++) {
    if (player_marine[i]->GetWorldCoor().y <= assemble_center.y) {
      index++;
    }
  }
  */
}

namespace {
  void EngageVisitPlayer(const int &player_index, int &player_group, int &enemy_group, const int *graph, const int &num_player, const int &num_enemy);
  void EngageVisitEnemy(const int &enemy_index, int &player_group, int &enemy_group, const int *graph, const int &num_player, const int &num_enemy);

  void EngageVisitPlayer(const int &player_index, int &player_group, int &enemy_group, const int *graph, const int &num_player, const int &num_enemy) {
    player_group |= (1 << player_index);
    const int *offseted = graph + player_index * MAX_SQUAD_NUMBER;
    for (int i = 0; i < num_enemy; ++i)
      if ( (offseted[i] == 1) && !(enemy_group & (1 << i)))
        EngageVisitEnemy(i, player_group, enemy_group, graph, num_player, num_enemy);
  }

  void EngageVisitEnemy(const int &enemy_index, int &player_group, int &enemy_group, const int *graph, const int &num_player, const int &num_enemy) {
    enemy_group |= (1 << enemy_index);
    for (int i = 0; i < num_player; ++i)
      if ( graph[i * MAX_SQUAD_NUMBER + enemy_index] == 1
          && !(player_group & (1 << i)))
        EngageVisitPlayer(i, player_group, enemy_group, graph, num_player, num_enemy);
  }

  const int SAFETY_BOUND = 25;
  const int SAFETY_PENALTY = 3;
  const int ENGAGE_RADIUS = 20;

  //const int ENGAGE_RADIUS = 10;

  // Begin the greedy matching algorithm for assigning vertices to groups
  int numVertices, numSquads; // The number of available vertices and number of squads

#ifdef SPAWN
  WorldCoor spawnFrom[64]; // Where does vertices[i] spawn from
  WorldCoor spawnPrev[64]; // Where does vertices[i]
  WorldCoor spawnNext[64]; // Where does vertices[i]
#endif

  WorldCoor vertices[64]; // All possible vertices
  bool occupied[64]; // Whether a vertex is occupied
  sint4 costOffsets[64]; // The offset from being not close to the center
  Cluster* assocClusters[64]; // The associated clusters of each vertices
  SquadAI* assocSquads[64];
  bool assigned[64]; // Whether the squad has been assigned
  sint4 costs[64][64]; // The cost of each squad attacking each vertex
  sint4 priorities[64][64]; // The sorted list of what units we should look at next
  sint4 readHead[64]; // Where they are pointing...

  int refSquad;
  bool RefOrder(const int a, const int b) {
    return costs[refSquad][a] < costs[refSquad][b];
  }


}

void GeneralAI::CheckBattleStatus() {
  for(int i = 0, ei = squads.size(); i < ei; i++) {
    squads.at(i)->CheckBattleStatus();
  }
}

// Build Group

void GeneralAI::Update() {

  // Update our squads

  //Upate squads status
  CLEAR_DEAD(squads);
  /*
     for (int i = 0 ; i < squads.size(); ++i ){
     cerr << "Squad " << i << ": " << squads[i]->GetMissionType() << endl;
     }
   */
  if (assembling) return;
  
  if (globalDataBase->GetEnemyMarineCount() == 0) return; // Why bother now?!

  // These squads are now USELESS!!!

  //cerr << "We have " << globalDataBase->GetPlayerMarineCount() << " marines left" << endl;
  //cerr << "They have " << globalDataBase->GetEnemyMarineCount() << " marines left" << endl;

  //int enemy_count = globalDataBase->GetEnemyMarineCount();

  EnemyMarine** enemy_marines = globalDataBase->GetEnemyMarines();

  Clustering* clustering = globalDataBase->GetClustering();
  Cluster* const* cluster_begin = clustering->GetClusters();
  Cluster* const* cluster_end = clustering->GetClustersEnd();
  /*
     cerr << "Enemy cluster information:" << endl;
     for (Cluster* const* j = cluster_begin; j != cluster_end; ++j) {
     cerr << "Cluster " << j-cluster_begin << " has " << (*j)->GetSize() << " marines and centered around (" << (*j)->GetCenterOfMass().x << ", " << (*j)->GetCenterOfMass().y << ")" << endl;
     }
   */

  // Merge or split squads

  for (Cluster* const* j = cluster_begin; j != cluster_end; ++j) {
    memset( (*j)->vertexSquadSet, 0, sizeof( (*j)->vertexSquadSet ) );
    (*j)->numMarineEngaged = 0;
  }

  sint4 dist = 0, min_dist = 99999999, best_idx;
  WorldCoor ref;

  static int regroup_cooldown = 0;
  if (regroup_cooldown == 0) {
    regrouping->Regroup();
    regroup_cooldown = 5;
  } else {
    --regroup_cooldown;
    regrouping->Update();
  }
  //cerr << "regroup_cooldown = " << regroup_cooldown << endl;
  //cerr << "number of squads = " << (regrouping->GetSquadsEnd() - regrouping->GetSquads()) << endl;

  // First build a list of attackable points
  numVertices = 0;
  for (Cluster* const* i = cluster_begin; i != cluster_end; ++i) {
    Cluster* cluster = *i;
    sint4 factor = 0;
    if ( cluster->GetVariance() )
      factor = 80000 / cluster->GetVariance();
    const WorldCoor& center = cluster->GetCenterOfMass();

    for (const WorldCoor *j = (*i)->GetHull(), *ej = (*i)->GetHullEnd();
        j != ej; ++j) {
      WorldCoor vertex = *j; // Find out where it should go

#ifdef SPAWN
      spawnFrom[numVertices] = vertex;
#endif

      if (cluster->GetSize() > 3) {
        //double dx = vertex.x - center.x, dy = vertex.y - center.y;
        WorldCoor prev, next;
        if (j == (*i)->GetHull()) prev = ej[-1]; else prev = j[-1];
        if (j == ((*i)->GetHullEnd() - 1) ) next = *((*i)->GetHull()); else next = j[1];
#ifdef SPAWN
        spawnPrev[numVertices] = prev;
        spawnNext[numVertices] = next;
#endif

        double dx_next = vertex.x - next.x, dy_next = vertex.y - next.y;
        double dx_prev = vertex.x - prev.x, dy_prev = vertex.y - prev.y;
        double next_dist = sqrt(dx_next * dx_next + dy_next * dy_next * 1.0);
        double prev_dist = sqrt(dx_prev * dx_prev + dy_prev * dy_prev * 1.0);
        if ( next_dist > 1e-3 && prev_dist > 1e-3) {
          vertex.x += (int) (ENGAGE_RADIUS * (dx_next / next_dist + dx_prev / prev_dist));
          vertex.y += (int) (ENGAGE_RADIUS * (dy_next / next_dist + dy_prev / prev_dist));

          /* Try to make all harass point same distance from the vertex
          double dx = dx_next / next_dist + dx_prev / prev_dist;
          double dy = dy_next / next_dist + dy_prev / prev_dist;
          double normalizer = 1.0 / sqrt(dx * dx + dy * dy);
          vertex.x += (int) (ENGAGE_RADIUS * dx * normalizer);
          vertex.y += (int) (ENGAGE_RADIUS * dy * normalizer);
          */
        }
      } else {
#ifdef SPAWN
        spawnPrev[numVertices] = vertex;
        spawnNext[numVertices] = vertex;
#endif
      }
      costOffsets[numVertices] = -factor * vertex.sqr_dist(center);

      // So they can really push
      if (vertex.x < 0) {
        vertex.x = 0;
        costOffsets[numVertices] += 1000 * cluster->GetSize() * cluster->GetSize();
      } else if (vertex.x >= WIDTH) {
        vertex.x = WIDTH - 1;
        costOffsets[numVertices] += 1000 * cluster->GetSize() * cluster->GetSize();
      }

      if (vertex.y < 0) {
        vertex.y = 0;
        costOffsets[numVertices] += 1000 * cluster->GetSize() * cluster->GetSize();
      } else if (vertex.y >= HEIGHT) {
        vertex.y = HEIGHT - 1;
        costOffsets[numVertices] += 1000 * cluster->GetSize() * cluster->GetSize();
      }

      vertices[numVertices] = vertex;
      /*
      // Check if it is near the boundary
      if (vertex.x <= SAFETY_BOUND)
        costOffsets[numVertices] +=
          SAFETY_PENALTY * (SAFETY_BOUND - vertex.x) * (SAFETY_BOUND - vertex.x);
      else if (vertex.x >= WIDTH - 1 - SAFETY_BOUND)
        costOffsets[numVertices] +=
          SAFETY_PENALTY * (vertex.x - WIDTH + 1 + SAFETY_BOUND)
          * (vertex.x - WIDTH + 1 + SAFETY_BOUND);

      if (vertex.y <= SAFETY_BOUND)
        costOffsets[numVertices] +=
          SAFETY_PENALTY * (SAFETY_BOUND - vertex.y) * (SAFETY_BOUND - vertex.y);
      else if (vertex.y >= HEIGHT - 1 - SAFETY_BOUND)
        costOffsets[numVertices] +=
          SAFETY_PENALTY * (vertex.y - HEIGHT + 1 + SAFETY_BOUND)
          * (vertex.y - HEIGHT + 1 + SAFETY_BOUND);
       */

      // I want to test if this would put me in unfavourable locations

      assocClusters[numVertices] = cluster;
      occupied[numVertices++] = false;
    }
  }

  //cerr << "We have a total of " << numVertices << " vertices" << endl;
  // Rate them now...
  for (int i = 0; i < numVertices; ++i) {
    WorldCoor vertex = vertices[i];
    for (const EnemyMarine* const* j = globalDataBase->GetEnemyMarines(),
        *const* ej = globalDataBase->GetEnemyMarinesEnd(); j != ej; ++j)
      if (vertex.sqr_dist( (*j)->GetWorldCoor() ) <= 80*80)
        costOffsets[i] += 64;
  }

  numSquads = 0;
  // Now, I have all vertices
  for (SquadAI** i = regrouping->GetSquads(), **ei = regrouping->GetSquadsEnd();
      i != ei; ++i) {
    SquadAI* squad = *i;

    ref.x = 0;
    ref.y = 0;
    for (int j = 0, ej = squad->NumOfMarines(); j != ej; ++j) {
      const WorldCoor marine_coor = (*squad->GetMarines())[j]->GetWorldCoor();
      ref.x += marine_coor.x;
      ref.y += marine_coor.y;
    }
    ref.x /= squad->NumOfMarines();
    ref.y /= squad->NumOfMarines();

    //cerr << "Reference point found: " << ref << endl;

    // For a small squad, it doesn't have to match
    // We only want intelligent matching for a large one
    if (squad->NumOfMarines() <= 3) {
      // Just do greedy
      min_dist = 99999999;
      best_idx = -1;
      // Find the best point
      for (int j = 0; j < numVertices; ++j) {
        dist = vertices[j].sqr_dist(ref) + costOffsets[j];
        if (dist < min_dist) {
          min_dist = dist;
          best_idx = j;
        }
      }
      // Fight this vertex
      if (best_idx != -1)
        squad->SetDest(vertices[best_idx]);
      // Register the number of people fighting this group
      assocClusters[best_idx]->numMarineEngaged += squad->NumOfMarines();
      if (assocClusters[best_idx]->numMarineEngaged * 2 / 3
          >= assocClusters[best_idx]->GetSize()) {
        // Mark all vertices as occupied
        for (int j = 0; j < numVertices; ++j)
          if (assocClusters[j] == assocClusters[best_idx])
            occupied[j] = true;
      }
      // Commented because I don't want these trail units to get in my way
      // If I can regroup, this may also be a good thing.
      // If I can't regroup, they need help
      continue;
    }

    sint4 squad_size = squad->NumOfMarines();

    // Since this is a sizable group, I want it to be a little bit more intelligent
    assocSquads[numSquads] = squad; // I have a squad, and I want to see how it reacts too all vertices
    for (int j = 0; j < numVertices; ++j) {
      const WorldCoor& vertex = vertices[j];
      const WorldCoor& center = assocClusters[j]->GetCenterOfMass();
      costs[numSquads][j] = vertex.sqr_dist(ref) + costOffsets[j];
      costs[numSquads][j] += std::max(assocClusters[j]->GetSize() - squad_size, -1) * 400;

      // Check if it is a dead point....
      if ((vertex.x <= SAFETY_BOUND && ref.x <= SAFETY_BOUND)
        || (vertex.x >= WIDTH - 1 - SAFETY_BOUND && ref.x >= WIDTH - 1 - SAFETY_BOUND))
        if ((ref.y - center.y)*(ref.y - vertex.y) < 0) costs[numSquads][j] += 5000;

      if ((vertex.y <= SAFETY_BOUND && ref.y <= SAFETY_BOUND)
        || (vertex.y >= HEIGHT - 1 - SAFETY_BOUND && ref.y >= HEIGHT - 1 - SAFETY_BOUND))
        if ((ref.x - center.x)*(ref.x - vertex.x) < 0) costs[numSquads][j] += 5000;

    }

    ++numSquads;
  }
  // Now for all reasonable sized squads... I have a matrix telling me what vertices I want to kill
  //cerr << "We have a total of " << numSquads << " remaining squads" << endl;

  // Sort them by their costs
  for (int i = 0; i < numSquads; ++i) {
    for (int j = 0; j < numVertices; ++j)
      priorities[i][j] = j;
    refSquad = i;
    // Sort it
    std::sort( &priorities[i][0], (&priorities[i][0]) + numVertices, RefOrder );
    readHead[i] = 0;
    assigned[i] = false;
  }
  //cerr << "All priorities set up" << endl;

  // Start assigning
  for (int i = 0; i < numSquads; ++i) {
    //cerr << "Doing the " << i << "-th assignment" << endl;
    // Find out what the best_idx is
    best_idx = -1;
    min_dist = 99999999;
    for (int j = 0; j < numSquads; ++j) {
      if (assigned[j]) continue;
      while( readHead[j] < numVertices && occupied[ priorities[j][readHead[j]] ] ) ++readHead[j];
      if (readHead[j] < numVertices) {
        dist = costs[j] [ priorities[j][readHead[j]] ];
        if (dist < min_dist) {
          min_dist = dist;
          best_idx = j;
        }
      }
    }
    if (best_idx == -1) break;
    // Assign best_idx then
    sint4 t = priorities[best_idx][readHead[best_idx]];
    assocSquads[best_idx]->SetDest( vertices[ t ] );
    assigned[best_idx] = true;
    occupied[ t ] = true;
    assocClusters[t]->numMarineEngaged += assocSquads[best_idx]->NumOfMarines();
    if (assocClusters[t]->numMarineEngaged * 2 / 3 >= assocClusters[t]->GetSize() ) {
      // This cluster is full now
      for (int j = 0; j < numVertices; ++j)
        if (assocClusters[j] == assocClusters[t])
          occupied[j] = true;
    }
  }

  //cerr << "Assigning remaining ones" << endl;
  for (int i = 0; i < numSquads; ++i)
    if (!assigned[i]) {// Go to the first one?
      assocSquads[i]->SetDest( vertices[priorities[i][0]] );
      //cerr << "Assinging left out squad to " << vertices[priorities[i][0]] << endl;
    }
  //cerr << "All done" << endl;

#ifdef ABCDEFG
  // Stuff below are something else

  sint4 dist = 0;
  sint4 min_dist = 99999999;
  WorldCoor ref, temp;
  WorldCoor targeted[MAX_SQUAD_NUMBER];

  for (size_t i = 0; i < squads.size(); i++) {

    if (squads[i]->GetMissionType() == Attack) {// Go towards enemy
      min_dist = 99999999;
      temp = NULL_COOR;

      ref = (*squads[i]->GetMarines())[0]->GetWorldCoor();

      for (const Cluster* const* j = cluster_begin; j != cluster_end; ++j) {
        WorldCoor center_of_mass = (*j)->GetCenterOfMass();
        sint4 factor = 0;
        if ( (*j)->GetVariance() )
          factor = 80000 / (*j)->GetVariance();

        for (const WorldCoor *vertex = (*j)->GetHull(),
            *vertex_end = (*j)->GetHullEnd();
            vertex != vertex_end; ++vertex) {

          // Check if this vertex has already been targeted
          bool doubled = false;
          for (size_t k = 0; k < i; ++k) doubled |= (targeted[k] == *vertex);
          if (doubled) continue;

          dist = vertex->sqr_dist(ref)
            - factor * vertex->sqr_dist(center_of_mass);
          //dist = vertex->sqr_dist(ref);
          if (dist < min_dist) {
            min_dist = dist;
            temp = *vertex;
          }
        }
      }
      if ( temp != NULL_COOR )
        squads[i]->SetDest(temp);
      targeted[i] = temp;
    }
    else if (squads[i]->GetMissionType() == Intercept) {// Run to enemy's back
      if (clustering->GetClusterCount() > 2 ) {
        squads[i]->SetMissionType(Attack);
      }
      else { 
        ref = (*squads[i]->GetMarines())[0]->GetWorldCoor();
        WorldCoor center_of_enemy(0,0);
        for (const Cluster* const* j = cluster_begin; j != cluster_end; ++j) {
          center_of_enemy.x += (*j)->GetCenterOfMass().x * (*j)->GetSize();
          center_of_enemy.y += (*j)->GetCenterOfMass().y * (*j)->GetSize();
        }
        center_of_enemy.x /= globalDataBase->GetEnemyMarineCount();
        center_of_enemy.y /= globalDataBase->GetEnemyMarineCount();

        if (globalDataBase->GetPlayerID() == 1) {
          if (ref.x > center_of_enemy.x - 30) {
            squads[i]->SetDest(WorldCoor(center_of_enemy.x - 75, ref.y));
          }
          else {
            squads[i]->SetMissionType(Attack);
          }
        } else {
          if (ref.x < center_of_enemy.x + 30) {
            squads[i]->SetDest(WorldCoor(center_of_enemy.x + 75, ref.y));
          }
          else {
            squads[i]->SetMissionType(Attack);
          }
        }

      }
    }
    else if (squads[i]->GetMissionType() == Relocation) {// Move!
      //If the squad achieves its dest, switch type to Attack
      ref = (*squads[i]->GetMarines())[0]->GetWorldCoor();
      if (ref.sqr_dist(squads[i]->GetDest()) <= HYPER_RANGE) {
        squads[i]->SetMissionType(Attack);
      }
    }

  }  
  //==========================================================================
  // Experiment Section  

  //Make strategic decision
  memset(engagingMap, 0, MAX_SQUAD_NUMBER*MAX_SQUAD_NUMBER);
  for (int i = 0, ei = squads.size(); i < ei; i++) {
    squads[i]->EngagingEnemyCluster(engagingMap + i * MAX_SQUAD_NUMBER); 
  }

  cerr << "The engaging matrix is:" << endl;
  for (int i = 0, ei = squads.size(); i < ei; i++) {
    for (int j = 0, ej = clustering->GetClusterCount(); j < ej; j++) {
      cerr << engagingMap[i*MAX_SQUAD_NUMBER+j] << " ";
    }
    cerr << endl;
  }
  // Compose engaging set
  int processed_player = 0;
  int processed_enemy = 0;
  int player_groups[MAX_SQUAD_NUMBER], enemy_groups[MAX_SQUAD_NUMBER];
  int player_group_sizes[MAX_SQUAD_NUMBER], enemy_group_sizes[MAX_SQUAD_NUMBER];
  int group_idx = 0;

  for (int i = 0, ei = squads.size(); i < ei; ++i) {
    // Have we dealt with this squad already?
    if (processed_player & (1 << i)) continue;
    player_groups[group_idx] = enemy_groups[group_idx] = 0;
    EngageVisitPlayer(i, player_groups[group_idx], enemy_groups[group_idx],
        engagingMap, squads.size(), clustering->GetClusterCount());
    // So player_group and enemy_group are found
    processed_player |= player_groups[group_idx];
    processed_enemy |= enemy_groups[group_idx];
    // Calculate the squad sizes
    player_group_sizes[group_idx] = enemy_group_sizes[group_idx] = 0;
    for (int j = 0, ej = squads.size(); j < ej; ++j)
      if ( (1<<j) & player_groups[group_idx] ) // squad j in the group
        player_group_sizes[group_idx] += squads[j]->NumOfMarines();
    for (int j = 0, ej = clustering->GetClusterCount(); j < ej; ++j)
      if ( (1<<j) & enemy_groups[group_idx] ) // Medoid j in the group
        enemy_group_sizes[group_idx] += clustering->GetClusters()[j]->GetSize();
    ++group_idx;
  }

  // Consider stray enemies
  for (int i = 0, ei = clustering->GetClusterCount(); i < ei; ++i) {
    // Have we dealt with this squad already?
    if (processed_enemy & (1 << i)) continue;
    player_groups[group_idx] = 0;
    enemy_groups[group_idx] = (1 << i);
    //processed_enemy |= enemy_groups[group_idx]; // This is useless
    // Calculate the squad sizes
    player_group_sizes[group_idx] = 0;
    enemy_group_sizes[group_idx] = clustering->GetClusters()[i]->GetSize();
    ++group_idx;
  }


  // Group information and size found
  int deduction = 0;
  for (int i = 0; i < group_idx; i++) {
    if (!player_groups[i] || enemy_group_sizes[i] == 0) continue;
    deduction = player_group_sizes[i] - enemy_group_sizes[i] * 1.5;
    if ( deduction >= 5) { // Split. But split to a very small squad makes no sense
      // Find whether there's a squad whose size is the deduction
      for (int j = 0, ej = squads.size(); j < ej; ++j) {
        if (player_groups[i] & (1 << j)) {
          if (squads[j]->NumOfMarines() > deduction - 1 
              && squads[j]->NumOfMarines() < deduction + 1) { //Match! No need to split
            squads[j]->SetDest(clustering->GetLargestCluster()->GetCenterOfMass());
            squads[j]->SetMissionType(Relocation);
            break;
          }
        }
      }
      // No match found


    }
  }

  //=================================================================================
#endif
  //cerr << "Finish Update()" << endl;
}



void GeneralAI::Act() {
  if (assembling) {
    for (int i = 0; i < squads.size(); i++)
      squads[i]->Act();
  } else {
    for (SquadAI** i = regrouping->GetSquads(), **ei = regrouping->GetSquadsEnd();
        i != ei; ++i) (*i)->Act();
  }
}

void GeneralAI::MergeSquad(const int& i, const int& j) {
  int big, small;
  if (squads[i]->NumOfMarines() > squads[j]->NumOfMarines()) {
    big = i;
    small = j;
  }
  else {
    big = j;
    small = i;
  }
  vector<PlayerMarine*>* player_marines = squads[small]->GetMarines();
  vector<PlayerMarine*>* player_marines_big = squads[big]->GetMarines();
  for (int m = 0; m < (*player_marines).size(); ++m) {
    (*player_marines_big).push_back((*player_marines)[m]);
    //(*squads[big]->GetMarines()).push_back();
  }
  squads.back()->SetID(small); //Pay attention to here
  squads[small] = squads.back();
  squads.pop_back();
}

void GeneralAI::MergeSquad(SquadAI* sq1, SquadAI* sq2) {
  if (sq1->NumOfMarines() > sq2->NumOfMarines()) {
    for (int i = 0, ei = sq2->NumOfMarines(); i < ei; ++i) {
      (*sq1->GetMarines()).push_back((*sq2->GetMarines())[i]);
    }
    squads.back()->SetID(sq2->GetID());
    sq2 = squads.back();
    squads.pop_back();
  }
  else {
    for (int i = 0, ei = sq1->NumOfMarines(); i < ei; ++i) {
      (*sq2->GetMarines()).push_back((*sq1->GetMarines())[i]);
    }
    squads.back()->SetID(sq1->GetID());
    sq1 = squads.back();
    squads.pop_back();
  }
}

void GeneralAI::SplitSquad(SquadAI* sq, const int& number) {//TODO: Temporary fix
  SquadAI* new_sq = new SquadAI(globalDataBase);
  new_sq->SetID(squads.size());
  cerr << "Before spliting this squad has " << sq->NumOfMarines() << " marines." << endl;
  vector<PlayerMarine*>* squad_vector = sq->GetMarines();
  for (int temp = number; temp > 0; --temp) {
    new_sq->AddMarine((*squad_vector).back());
    (*squad_vector).pop_back();
  }
  cerr << "After spliting this squad has " << sq->NumOfMarines() << " marines." << endl;

  squads.push_back(new_sq);
}

namespace {
  Vec3<real4> ORANGE(1.0, 0.5, 0.0);
  Vec3<real4> GREY(0.5, 0.5, 0.5);
  Vec3<real4> CYAN(0.0, 1.0, 1.0);
  Vec3<real4> BLACK(0.0, 0.0, 0.0);
  Vec3<real4> YELLOW(1.0, 1.0, 0.0);
}

#ifdef GRAPHICS
void GeneralAI::DrawAssignment(DrawOnTerrain2D& dot) {
  if (assembling) {
    for (int i = 0, ei = squads.size(); i != ei; ++i) {
      SquadAI* squad = squads[i];
      WorldCoor dest = squad->GetDest();
      std::vector<PlayerMarine*>* marines = squad->GetMarines();
      int n = marines->size();
      for (int j = 0; j < n; ++j) {
        // Draw the line
        WorldCoor marine_coor = (*marines)[j]->GetWorldCoor();
        dot.draw_line(marine_coor.x, marine_coor.y, dest.x, dest.y, GREY);
      }
    }
    return;
  }
  // Draw all strategic points
  for (int i = 0; i < numVertices; ++i) {
    dot.draw_circle(vertices[i].x, vertices[i].y, 5.0, ORANGE);
    // Draw a line...
#ifdef SPAWN
    dot.draw_line(spawnFrom[i].x, spawnFrom[i].y, vertices[i].x, vertices[i].y, BLACK);
    dot.draw_line(spawnPrev[i].x, spawnPrev[i].y, vertices[i].x, vertices[i].y, CYAN);
    dot.draw_line(spawnNext[i].x, spawnNext[i].y, vertices[i].x, vertices[i].y, YELLOW);
#endif
  }
  for (SquadAI** i = regrouping->GetSquads(), **ei = regrouping->GetSquadsEnd();
      i != ei; ++i) {
    SquadAI* squad = *i;
    WorldCoor dest = squad->GetDest();
    std::vector<PlayerMarine*>* marines = squad->GetMarines();
    int n = marines->size();
    for (int j = 0; j < n; ++j) {
      // Draw the line
      WorldCoor marine_coor = (*marines)[j]->GetWorldCoor();
      dot.draw_line(marine_coor.x, marine_coor.y, dest.x, dest.y, GREY);
    }
  }
}
#endif
