#include "GlobalDataBase.H"
#include "ServerObjData.H"
#include "Clustering.H"
#include <iostream>
/* 
   Copyright 2007 Man Hon Chan, Zhangbo Liu, Simon Suyadi
   This code is Licensed under GPL.
 */

using std::cerr;
using std::endl;

GlobalDataBase::GlobalDataBase(const Game& game_, Random* random_):game(game_), random(random_) {
  gameTick = game.tick();

  int tmp;
  playerMarineCount = enemyMarineCount = neutralObjCount = corpseCount = 0;
  playerID = game.get_client_player();
  enemyID = 1 - playerID;

  const Game::ObjCont &player_objs = game.get_objs(playerID);
  const Game::ObjCont &enemy_objs = game.get_objs(enemyID);

  FORALL (player_objs, it) {
    GameObj* game_obj = (*it)->get_GameObj();
    if (!game_obj->sod.in_game) continue;
    if ((game_obj->bp_name() == "marine") && *game_obj->sod.x > 0) {
      tmp = playerMarineCount++;
      playerMarines[tmp] = (PlayerMarine*) GameObjWrapper::GetWrapper(game_obj, this);
      playerMarines[tmp]->arrayIndex = tmp;
    }
  }
  
  FORALL (enemy_objs, it) {
    GameObj* game_obj = (*it)->get_GameObj();
    if (!game_obj->sod.in_game) continue;
    if ((game_obj->bp_name() == "marine") && *game_obj->sod.x > 0) {
      tmp = enemyMarineCount++;
      /*enemy = */enemyMarines[tmp] = (EnemyMarine*) GameObjWrapper::GetWrapper(game_obj, this);
      enemyMarines[tmp]->arrayIndex = tmp;
    }
  }

  const sint4 npcid = game.get_player_num();
  const Game::ObjCont &npc_objs = game.get_objs(npcid);
  FORALL (npc_objs, it) {
    GameObj *game_obj = (*it)->get_GameObj();
    if (game_obj->bp_name() == "sheep") {
      tmp = neutralObjCount++;
      neutralObjs[tmp] = (NeutralObj*) GameObjWrapper::GetWrapper(game_obj, this);
      neutralObjs[tmp]->arrayIndex = tmp;
    }
  }

  clustering = new Clustering(this);

}

GlobalDataBase::~GlobalDataBase() {
  for (int i = playerMarineCount; i--; ) delete playerMarines[i];
  for (int i = enemyMarineCount; i--; ) delete enemyMarines[i];
  for (int i = neutralObjCount; i--; ) delete neutralObjs[i];
  for (int i = corpseCount; i--; ) delete graveyard[i];
  playerMarineCount = enemyMarineCount = neutralObjCount = corpseCount = 0;

  delete clustering;
  clustering = NULL;
}

namespace {
  // How do I specify such that this only accepts derived classes of GameObjWrapper
  template<class T>
    inline void RemoveObj(T** array, int& count, GameObjWrapper* obj) {
      int idx = obj->arrayIndex;
      array[idx] = array[--count];
      array[idx]->arrayIndex = idx;
    }
}

void GlobalDataBase::Update(const GameChanges& game_changes) {
  gameTick = game.tick();

  // Deal with the changed objects
  const Vector<ScriptObj::WPtr>& changed_objs = game_changes.changed_objs;
  FORALL(changed_objs, it) {
    GameObj* game_obj = (*it)->get_GameObj();
    if (!game_obj) continue;
    GameObjWrapper* game_obj_wrapper = GameObjWrapper::GetWrapper(game_obj, this);
    if (game_obj_wrapper != NULL)
      // I am ignoring that I may want to check for death here
      game_obj_wrapper->Update();
  }

  // Deal with dead objects
  const Vector<ScriptObj::WPtr>& dead_objs = game_changes.dead_objs;
  FORALL(dead_objs, it) {
    GameObj* game_obj = (*it)->get_GameObj();
    if (!game_obj) continue;
    GameObjWrapper* game_obj_wrapper = GameObjWrapper::GetWrapper(game_obj, this);
    // Check what this is
    // I don't care about sheeps....
    if (game_obj->bp_name() == "marine") {
      // Mine
      if (*game_obj->sod.owner == playerID) {
        RemoveObj(playerMarines, playerMarineCount, game_obj_wrapper);
      } else {
        // Enemy
        RemoveObj(enemyMarines, enemyMarineCount, game_obj_wrapper);
      }
    }
    game_obj_wrapper->arrayIndex = corpseCount;
    graveyard[corpseCount++] = game_obj_wrapper;
    game_obj_wrapper->CleanUp();
  }

}

void GlobalDataBase::Display() {

  for (int i = 0; i < playerMarineCount; ++i) {
    PlayerMarine* marine = playerMarines[i];
    EnemyMarine* target = marine->GetTarget();
    if (target != NULL)
      cerr << "Attacking: " << target->GetWorldCoor() << " with HP = " << target->GetHP() << endl;
  }
}
