#include "PlayerMarine.H"
//#include "FFPlayerData.H"
#include "EnemyMarine.H"
#include "GlobalDataBase.H"
#include "ScriptObj.H"
#include <algorithm>
/* 
   Copyright 2007 Man Hon Chan, Zhangbo Liu, Simon Suyadi
   This code is Licensed under GPL.
 */

#include <iostream>
using std::cerr;
using std::endl;

namespace {
  const sint4 SQR_RANGE = 64 * 64;
}
Vector<sint4> PlayerMarine::attackArgs(1);
Vector<sint4> PlayerMarine::moveArgs(3);

PlayerMarine::PlayerMarine(GameObj* game_obj, GlobalDataBase* global_data_base):GameObjWrapper(game_obj, global_data_base), globalDataBase(global_data_base), target(NULL), nextFireTime(-1), inBattle(false), initAchieved(false), squad(NULL) {
  dest = worldCoor; // My own mapCoor
  wayPoint = worldCoor; // My current location
  speed = 3;
}

bool PlayerMarine::Update() {
  if (!GameObjWrapper::Update()) return false;
  hp = *gameObj->get_int_ptr("hp");
  return true;
}

sint4 PlayerMarine::GetCooldown() const {
  ScriptObj* weapon = gameObj->component("weapon");
  const sintptr *lf = weapon->get_int_ptr("last_fired");
  const sintptr *cd = weapon->get_int_ptr("cooldown");
  const sint4 game_tick = globalDataBase->GetGameTick();
  return std::max(sint4(*cd + *lf - game_tick), 0);
}

void PlayerMarine::Act() {
  if (target != NULL
      && !GetCooldown()
      && globalDataBase->GetGameTick() >= nextFireTime
      && (!target->IsDead()) ) {
    attackArgs[0] = target->GetID();
    gameObj->component("weapon")->set_action("attack", attackArgs);
    nextFireTime = globalDataBase->GetGameTick() + 2;
    //cerr << "Attacking " << target->GetWorldCoor() << " with HP " << target->GetHP() << endl;
  } else { // Move to the way point
    moveArgs[0] = wayPoint.x;
    moveArgs[1] = wayPoint.y;
    moveArgs[2] = speed;
    //if (target != NULL) moveArgs[2] <?= 2; // Can't move faster
    gameObj->set_action("move", moveArgs);
  }
}

EnemyMarine* PlayerMarine::WeakestInRange(sint4 range) {
  
  WorldCoor pos = GetWorldCoor();
  EnemyMarine* weakest = NULL;
  sint4 weakest_hp = 100;

  int tmp;
  for(EnemyMarine** enemy_unit = globalDataBase->GetEnemyMarines(),
      **enemy_units_end = globalDataBase->GetEnemyMarinesEnd();
      enemy_unit != enemy_units_end; ++enemy_unit) {

    tmp = (*enemy_unit)->GetWorldCoor().sqr_dist(pos);
    if (tmp < range && (*enemy_unit)->GetHP() < weakest_hp)
      weakest_hp = (weakest = (*enemy_unit))->GetHP();
  }
  if (weakest!=NULL) {
    inBattle = true;
  }
  else {
    inBattle = false;
  }
  return weakest;
}
