#include "Primitive.H"
#include <cmath>
/* 
   Copyright 2007 Man Hon Chan, Zhangbo Liu, Simon Suyadi
   This code is Licensed under GPL.
 */

/* MapCoor */
bool MapCoor::operator== (const MapCoor& other) const {
  return r == other.r && c == other.c;
}

bool MapCoor::operator!= (const MapCoor& other) const {
  return r != other.r || c != other.c;
}

bool MapCoor::operator< (const MapCoor& other) const {
  return (r < other.r) || (r == other.r && c < other.c);
}

/* End of MapCoor */


/* WorldCoor */
bool WorldCoor::operator== (const WorldCoor& other) const {
  return x == other.x && y == other.y;
}

bool WorldCoor::operator!= (const WorldCoor& other) const {
  return x != other.x || y != other.y;
}

bool WorldCoor::operator< (const WorldCoor& other) const {
  return (x < other.x) || (x == other.x && y < other.y);
}

sint4 WorldCoor::sqr_dist(const WorldCoor& other) const {
  sint4 dx(other.x - x), dy(other.y - y);
  return dx * dx + dy * dy;
}

double WorldCoor::dist(const WorldCoor& other) const {
  sint4 dx(other.x - x), dy(other.y - y);
  return sqrt(dx * dx + dy * dy);
}
/* End of WorldCoor */


/* MapDirection */
bool MapDirection::operator==(const MapDirection& other) const {
  return dr == other.dr && dc == other.dc;
}

bool MapDirection::operator!=(const MapDirection& other) const {
  return dr != other.dr && dc != other.dc;
}

bool MapDirection::operator<(const MapDirection& other) const {
  return (dr < other.dr) || (dr == other.dr && dc < other.dc);
}
/* End of MapDirection */

MapCoor operator+(const MapCoor& coor, const MapDirection& dir) {
  return MapCoor(coor.r + dir.dr, coor.c + dir.dc);
}


std::ostream& operator<<(std::ostream& os, const MapCoor& coor) {
  return os << "(r=" << coor.r << ", c=" << coor.c << ')';
}

std::ostream& operator<<(std::ostream& os, const WorldCoor& coor) {
  return os << "(x=" << coor.x << ", y=" << coor.y << ')';
}

std::ostream& operator<<(std::ostream& os, const MapDirection& dir) {
  return os << "(dr=" << dir.dr << ", dc=" << dir.dc << ')';
}

