#include "SquadAI.H"
#include "Clustering.H"
/* 
   Copyright 2007 Man Hon Chan, Zhangbo Liu, Simon Suyadi
   This code is Licensed under GPL.
 */

using namespace std;

#define CLEAR_DEAD(v) for (size_t i = 0; i < v.size(); ) if (v[i]->IsDead()) { v[i] = v.back(); v.pop_back(); } else ++i;

SquadAI::SquadAI(GlobalDataBase* global_data_base):globalDataBase(global_data_base), missionType(Attack), ID(-1), inBattle(false), squadDest(WorldCoor(0,0))/*, isAssembled(false) */ {};

void SquadAI::Reset() {
  // Go through all the marines and reset them
  /*
  for (int i = 0, ei = squadMarines.size(); i != ei; ++i)
    squadMarines[i]->SetSquad(NULL);
   */
  squadMarines.clear();
}

void SquadAI::AddMarine(PlayerMarine* marine) {
  if(marine!=NULL) {
    squadMarines.push_back(marine);
    marine->SetSquad(this);
  }
}

bool SquadAI::IsAlive() {
  CLEAR_DEAD(squadMarines);
  return !squadMarines.empty();
}


void SquadAI::Act() {

  //Scan and remove dead marines from squad
  CLEAR_DEAD(squadMarines);

  if (missionType==Attack) {
    //Set the closest enemyCC as the squad's dest. This should be one of the choices when the General makes decisions.
    //EnemyCC* closestEnemyCC = ClosestEnemyCC();
    
    for (int i = 0; i < squadMarines.size(); i++) {//marines are assumed to be alive
      EnemyMarine* temp_target = NULL;
      temp_target = squadMarines.at(i)->WeakestInRange(RANGE);
      squadMarines.at(i)->SetTarget(temp_target);
      if (squadDest.x >= 0)
        squadMarines.at(i)->SetDest(squadDest);
    }
  } 
  else if (missionType==Intercept) {  //TODO: Redesign needed
    
    for (int i = 0; i < squadMarines.size(); i++) {//marines are assumed to be alive
      EnemyMarine* temp_target = NULL;
      temp_target = squadMarines.at(i)->WeakestInRange(RANGE);
      squadMarines.at(i)->SetTarget(temp_target);
      if (squadDest.x >= 0)
        squadMarines.at(i)->SetDest(squadDest);
    }
    /*
    EnemyMarine* target = NULL;
    //EnemyMarine* potential_target = NULL;
    for (int i = 0; i < squadMarines.size(); i++) {
      EnemyMarine* temp_target = NULL;
      temp_target = squadMarines.at(i)->WeakestInRange(RANGE);
      if (temp_target!=NULL) {
        squadMarines.at(i)->SetTarget(temp_target);
        if (target==NULL || target->GetHP() > temp_target->GetHP()) {
          target = temp_target;
        }
      }
      else if (target!=NULL) {
        //squadMarines.at(i)->SetDest(target->GetMapCoor());
        squadMarines.at(i)->SetDest(target->GetWorldCoor());
      }
      else {
        //squadMarines.at(i)->SetDest(GetDest());
      }  
    }
    */
  }
  else if (missionType==Relocation) {
    for (int i = 0; i < squadMarines.size(); i++) {//marines are assumed to be alive
      EnemyMarine* temp_target = NULL;
      temp_target = squadMarines.at(i)->WeakestInRange(RANGE);
      squadMarines.at(i)->SetTarget(temp_target);
      if (squadDest.x >= 0)
        squadMarines.at(i)->SetDest(squadDest);
    }
  
  }
  else if (missionType==Retreat) {
    //TODO: FILL ME
  }
  else {
    cerr << "ERROR: MissionType undefined..." << endl;
  }

} // End of SquadAI::Act

void SquadAI::EngagingEnemyCluster(int* engaging_map) {
  for (int i = 0; i < squadMarines.size(); i++) {
    WorldCoor pos = squadMarines[i]->GetWorldCoor();
    //sint4 dist;
    for (EnemyMarine** enemy_marines = globalDataBase->GetEnemyMarines(), 
        **enemy_marines_end = globalDataBase->GetEnemyMarinesEnd();
        enemy_marines != enemy_marines_end; enemy_marines++) {
      if ((*enemy_marines)->GetWorldCoor().sqr_dist(pos) < HYPER_RANGE) {//HPER_RANGE for FF
        engaging_map[(*enemy_marines)->GetCluster()->GetID()] = 1;  
      }
    } 
  }
}

void SquadAI::CheckBattleStatus() {
  for (int i = 0, ei = squadMarines.size(); i < ei; i++) {
    if (squadMarines.at(i)->IsInBattle()) {
      inBattle = true;
      return;
    }
  }
}

/*
void SquadAI::DoAssemble() {
  SetDest(squadMarines[0]->GetWorldCoor());
  FORALL(squadMarines, it)
    (*it)->SetDest(squadDest);
  for (int i = 0, ei = squadMarines.size(); i < ei; i++) {
    squadMarines[i]->SetDest(squadDest);
  }
  isAssembled = false;
}
*/

/*
void SquadAI::DoProcess(vector< vector<int> >& player_sand, const int& pv_index, vector< vector<int> >& enemy_sand, const int& ev_index) {
  
  isProcessed = true;
}
*/



/*
WorldCoor SquadAI::RotateFormation(WorldCoor p, double angle) {
  double radius = sqrt(p.x*p.x + p.y*p.y);
  if (radius == 0) { // Is the central point
    return p;
  }
  else {
    WorldCoor newpoint;
    newpoint.x = p.x*cos(angle) - p.y*sin(angle);
    newpoint.y = p.x*sin(angle) + p.y*cos(angle);
    return newpoint;
  }
}
*/

/*
void SquadAI::GetGoal() { //For testing purpose
  vector<PlayerMarine*>::iterator it;
    cerr << "Our goals are: ";
  for (it = squadMarines.begin(); it != squadMarines.end(); it++) {
    cerr << "(" << (*it)->GetGoal().x << ", " << (*it)->GetGoal().y << "), ";
  }  
  cerr << endl;
}
*/
