//=======================================================
//TODO:
// 1. The value of k is given arbitrarily, which will 
//  not work when the number of enemy's tank become small
//=======================================================
#ifndef CLUSTERING_H
#define CLUSTERING_H

#include "Primitive.H"
#include "GameObjWrapper.H"
#include "GlobalDataBase.H"

using namespace std;

namespace {
  const int MAX_UNIT_COUNT = 64;
  const int MAX_CLUSTER_COUNT = 12;
}

class Clustering {
  public:
    Clustering(GlobalDataBase* global_data_base);
    ~Clustering();
    void Update(); // Group units and keep all the clusters.

    int GetClusterCount() const { return clusterCount; }
    Cluster* const* GetClusters() { return clusters; }
    Cluster* const* GetClustersEnd() { return clusters + clusterCount; }
    Cluster* GetLargestCluster();// const{ return largestCluster;}

  private:
    GlobalDataBase* globalDataBase;

    int initEnemyCount;

    // Hide these functions
    Clustering& operator=(const Clustering& ) { return *this; }
    Clustering(const Clustering& ) { }

    int clusterCount;
    Cluster* clusters[MAX_UNIT_COUNT];

    //Cluster* largestCluster;

};

class Cluster {
  public:
    Cluster(int id_);

    int GetID() const { return id; }

    int GetSize() const { return clusterSize; }
    EnemyMarine** GetUnits() { return units; }
    EnemyMarine** GetUnitsEnd() { return units + clusterSize; }

    WorldCoor GetCenterOfMass() const { return centerOfMass; }
    sint4 GetVariance() const { return variance; }
    
    int GetHullSize() const { return hullSize; }
    const WorldCoor* GetHull() const { return hull; }
    const WorldCoor* GetHullEnd() const { return hull + hullSize; }

    // This is for General AI
    bool vertexSquadSet[MAX_UNIT_COUNT];
    int numMarineEngaged;

  private:

    int id;

    int clusterSize;
    EnemyMarine* units[MAX_UNIT_COUNT];
    EnemyMarine* medoid;

    int hullSize;
    WorldCoor hull[MAX_UNIT_COUNT];

    WorldCoor centerOfMass;
    sint4 variance;

    void AddUnit(EnemyMarine* unit);
    void Clear(); // Clear any information related to this cluster
    void CalcStat(); // Calculate all relevant statistics

    friend class Clustering;
};
#endif
