#ifndef GLOBAL_DATA_BASE_H
#define GLOBAL_DATA_BASE_H

#include "Primitive.H"
#include "Game.H"
#include "GameChanges.H"
#include "GameObjWrapper.H"
#include "PlayerInfo.H"
#include "PlayerMarine.H"
#include "EnemyMarine.H"
#include "NeutralObj.H"

class Clustering;

class GlobalDataBase {
  public:
    GlobalDataBase(const Game &game_, Random* random_);
    ~GlobalDataBase();

    inline PlayerInfo& GetPlayerInfo() const { return game.get_cplayer_info(); }

    inline PlayerMarine** GetPlayerMarines() { return playerMarines; }
    inline EnemyMarine** GetEnemyMarines() { return enemyMarines; }
    inline NeutralObj** GetNeutralObjs() { return neutralObjs; }

    inline int GetPlayerMarineCount() const { return playerMarineCount; }
    inline int GetEnemyMarineCount() const { return enemyMarineCount; }
    inline int GetNeutralObjCount() const { return neutralObjCount; }

    inline PlayerMarine** GetPlayerMarinesEnd() { return playerMarines + playerMarineCount; }
    inline EnemyMarine** GetEnemyMarinesEnd() { return enemyMarines + enemyMarineCount; }
    inline NeutralObj** GetNeutralObjsEnd() { return neutralObjs + neutralObjCount; }

    inline const Game& GetGame() const { return game; }
    inline Random* GetRandom() const { return random; }

    inline sint4 GetGameTick() const { return gameTick; }
    inline sint4 GetPlayerID() const { return playerID; }
    inline sint4 GetEnemyID() const { return enemyID; }

    inline Clustering* GetClustering() const { return clustering; }

    void Update(const GameChanges& game_changes); // Update all information
    
    void Display();

  private:
    const Game& game;
    PlayerMarine* playerMarines[64];
    EnemyMarine* enemyMarines[64];
    NeutralObj* neutralObjs[128];

    int playerMarineCount, enemyMarineCount, neutralObjCount;
    GameObjWrapper* graveyard[128];
    int corpseCount;
    Random* random;

    sint4 gameTick;
    sint4 playerID, enemyID;

    Clustering* clustering;

};

#endif
