#ifndef GatherAI_H
#define GatherAI_H

// $Id: GatherAI.H,v 1.8 2007/04/13 15:03:44 orts_furtak Exp $

// This is an ORTS file (c) Michael Buro, licensed under the GPL

#include "Global.H"
#include "LowAI.H"
#include "EventHandler.H"
#include "ClientModule.H"

class GameStateModule;

//===================================================================

class GatherData {
public:
  GameObj *res;
  GameObj *base;
  enum State { MOVE_TO, GATHER, MOVE_BACK, RETURN, QUEUE };
  State state;
  sint4 last_updated; ///< last frame at which a base was selected
  GatherData() : last_updated(-1){}
};

//-------------------------------------------------------------------

class GatherAI : public LowAI<GatherData>, public EventHandler, public ClientModule
{
protected:

//   struct ResBase {
//     std::set<GameObj*> resources;
//     std::set<GameObj*> bases;
//     sint4 last_updated; ///< last frame at which a base was added or removed 
//     ResBase() : last_updated(-1) {}
//   };

  struct BaseData{
    std::set<GameObj*> bases;
    sint4 last_updated;
    BaseData() : last_updated(-1){};
  };

 

  // std::map<std::string, ResBase> res_map; ///< Resources and places to take them

  std::map<GameObj*, std::queue<GameObj*> > resources;
  GameStateModule &gsm;
  Vector<GameObj*> need_act;
  std::set<GameObj*> mining;
  std::set<GameObj*> returning;
  BaseData bases;
  
  void stop(GameObj *obj);

  /** Return the base in the ResBase struct that is closest to the given GameObj.
      Does not take into account terrain when calculating distance
  */ 
  GameObj* choose_base(GameObj* gatherer);

public:

  static const sint4 FROM;
  static const sint4 GATHER_START;
  static const sint4 GATHER_STOP;

  static bool is_extractor(GameObj* gob);

  GatherAI(GameStateModule &gsm);
  ~GatherAI();

  void act(GameStateModule &gsm);
  static const std::string bp_name;

  void add(GameObj*) { ERR("?"); }
  void add_res(GameObj *gob);

  //void add_base(const std::string &res, GameObj *gob);
  void add_base(GameObj *gob); ///<Adds the base to all resource maps

  void del(GameObj*) { ERR("?"); }
  void del_res(GameObj *gob);
  //  void del_base(const std::string &res, GameObj *gob);
  void del_base(GameObj *gob); ///<Removes the base from all resource maps

  bool handle_event(const Event &e);
};

//===================================================================

class GatherEvent : public Event
{
public:
  
  GatherEvent(sint4 who, sint4 what, GameObj *o, GameObj *t) :
    Event(who, what), obj(o), targ(t) {}

  ~GatherEvent() {}
  
  GameObj *obj, *targ;
};

//===================================================================

typedef std::map<GameObj*, GatherData> GatherDataMap;
typedef std::map<GameObj*, std::queue<GameObj*> > GameObjQueueMap;

REGISTER_TYPEOF(410, GatherDataMap::iterator);
REGISTER_TYPEOF(411, GameObjQueueMap::iterator);

#endif
