#ifndef MD3_H
#define MD3_H

// $Id: MD3.H,v 1.10 2005/08/26 20:35:25 orts_furtak Exp $

// This is an ORTS file (c) Keith Yerex, licensed under the GPL

#include "GfxGlobal.H"
#include "GfxAnimesh.H"

class MD3Header;

//===================================================================

class MD3_Model : public Model
{
  friend class GfxAnimesh;

public:

  MD3_Model();
  virtual ~MD3_Model();

  void strip(const std::string &modelfile);
  sint4 load(const std::string &modelfile, const std::string &texfile,
             real4 scale, real4 dy, real4 gamma);
  
  void draw_bbox(sint4 /*frame*/) {};
  virtual void draw(sint4 frame);
  virtual void draw(sint4 frame, sint4 team);
  virtual void draw_t(sint4 /*frame*/, sint4 /*team*/) {};
  virtual void draw_interp(sint4 f1, sint4 f2, real4 blend, sint4 team);
  virtual void select(sint4 frame);
  
  void draw_shadow(sint4 /*frame*/, real4 sc = 0.0);

  bool fast_save(const std::string &file);
  bool fast_load(const std::string &file);
  
protected:
  
  virtual void init();

  void build_shadows();

  bool load_header(FILE* &f, MD3Header &header);
  bool load_tags(FILE* &f, const MD3Header &header);
  void load_cfg(const std::string &fname);
  void compute_centroid();

  uint4 nvert;
  uint4 npoly;
  Vec3<real4> *vert;
  real4 *tvert;
  Vec3<real4> *vnorm;
  Poly *poly;
};

//-------------------------------------------------------------------

/** Display list MD3 model. */
class MD3_Model_DL : public MD3_Model
{
public:
  virtual void select(sint4 frame);
  virtual void draw(sint4 frame);
  virtual void draw_interp(sint4 f1, sint4 f2, real4 blend, sint4 team);
  
  virtual ~MD3_Model_DL();
  
protected:
  virtual void init();
  GLuint display_lists;
  GLuint select_list;
};

//-------------------------------------------------------------------

/** Vertex Buffer MD3 model. -- incomplete */
/*
class MD3_Model_VB : public MD3_Model
{
public:

  virtual void draw(sint4 frame);
  virtual void draw_interp(sint4 f1, sint4 f2, real4 blend, sint4 team);
  virtual ~MD3_Model_VB() {}  // fixme: delete anything?
  
protected:
  virtual void init();
  GLuint vertex_buffer;
  GLuint index_buffer;
};
*/
//-------------------------------------------------------------------

struct MD3Header
{
  sint4 id;
  sint4 version;
  uint1 name[64];
  sint4 flags;
  sint4 numFrames;
  sint4 numTags;
  sint4 numSurfaces;
  sint4 numSkins;
  sint4 surfaceSize;

  sint4 offsetTags;     // tag start
  sint4 offsetSurfaces; // surface start / tag end
  sint4 offsetEnd;      // end of file ~ file size

  void little2machine();
  void print() const;
};

//-------------------------------------------------------------------

struct MD3Surface
{
  sint4 id;
  uint1 name[64];
  sint4 flags;
  sint4 numFrames;
  sint4 numShaders;
  sint4 numVerts;
  sint4 numTriangles;
  sint4 offsetTriangles;
  sint4 offsetShaders;
  sint4 offsetST;
  sint4 offsetVerts;
  sint4 offsetEnd;
  
  void little2machine();
  void print() const;
};

//-------------------------------------------------------------------

struct MD3Vertex
{
  sint2 x;
  sint2 y;
  sint2 z;
  sint2 normal;

  void little2machine();
};

//-------------------------------------------------------------------

struct MD3Tag
{
  uint1 name[64];
  real4 origin[3];
  real4 axis[3][3];

  void little2machine();
};

//===================================================================

#endif
