#ifndef Object_H
#define Object_H

// $Id: Object.H,v 1.13 2007/05/02 16:16:39 orts_furtak Exp $
 
// This is an ORTS file (c) Michael Buro, licensed under the GPL

// Game object base class (interface to terrain code)

#include "Global.H"

class Object {

public:

  // CIRCLE:    center, radius
  // LINE:      (x1,y1)---(x2,y2)
  // RECTANGLE: (x1,y1) \ (x2,y2)  (axis aligned)
  enum Shape { SHAPE_UNDEF=0, CIRCLE, LINE, RECTANGLE };

  // Z categories
  enum ZCat { ZCAT_UNDEF=0, UNDER_WATER, ON_WATER, ON_LAND, IN_AIR };

  virtual Shape get_shape() const = 0;
  virtual sint4 get_max_speed() const = 0;

  // only valid if circle:
  virtual sint4 get_radius() const = 0;

  // attributes that can change
  // if pointer != 0 store previous value there
  
  virtual ZCat  get_zcat(ZCat *prev = 0) const = 0;  
  virtual sint4 get_speed(sint4 *prev=0) const = 0;
  virtual bool  get_moving(bool *prev=0) const = 0 ; 
  virtual void  get_center(sint4 &x, sint4 &y, sint4 *px=0, sint4 *py=0) const = 0;

  // only valid if rectangle or line:
  virtual void get_p1(sint4 &x, sint4 &y, sint4 *px=0, sint4 *py=0) const = 0; // rectangle: upper left corner
  virtual void get_p2(sint4 &x, sint4 &y, sint4 *px=0, sint4 *py=0) const = 0; // rectangle: lower right corner

  virtual bool is_pending_action(void) const = 0;

  virtual real8 distance_to(const Object &other) const = 0;

  inline virtual ~Object() { }
};

REGISTER_TYPEOF(110, Vector<Object*>::iterator);
REGISTER_TYPEOF(111, Vector<Object*>::const_iterator);

REGISTER_TYPEOF(112, Vector<const Object*>::iterator);
REGISTER_TYPEOF(113, Vector<const Object*>::const_iterator);

REGISTER_TYPEOF(114, std::set<Object*>::iterator);
REGISTER_TYPEOF(115, std::set<Object*>::const_iterator);

REGISTER_TYPEOF(116, std::set<const Object*>::iterator);
REGISTER_TYPEOF(117, std::set<const Object*>::const_iterator);

#endif
