#ifndef Random_H
#define Random_H

// $Id: Random.H,v 1.7 2006/04/26 01:38:37 orts_mburo Exp $

// This is an ORTS file (c) Nathan Sturtevant, licensed under the GPL

#include "Global.H"

class Random {

public:
  
  Random() : mti(N+1) {} // mti==N+1 means mt[N] is not initialized
  Random(uint4 seed) : mti(N+1) { srand(seed); }
  void srand(uint4);
  void lsrand(uint4[], sint4);
  uint4 rand_uint4();
  real8 rand_real8();
  uint4 ranged_uint4(uint4 minimum, uint4 maximum);
  uint4 ranged_sint4(sint4 minimum, sint4 maximum);  

  uint4 operator()(uint4 n) { assert(n > 0); return rand_uint4() % n; }
    
private:
  
  static const sint4 N = 624;
  static const sint4 M = 397;
  static const uint4 MATRIX_A = 0x9908b0dfUL;
  static const uint4 UPPER_MASK = 0x80000000UL;
  static const uint4 LOWER_MASK = 0x7fffffffUL;

  uint4 mt[N]; // the array for the state vector
  sint4 mti;
};


// random number source (functor)
struct RandSrc
{
  virtual sint4 operator()() = 0;
  virtual ~RandSrc() { }
};


// uniform distribution [0,uint4_max]
struct StdRandSrc : public RandSrc
{
  Random rand;
  StdRandSrc(uint4 seed) { rand.srand(seed); }
  sint4 operator()() { return rand.rand_uint4(); } // signed!
};


// uniform distribution over interval [min,max]
struct UniRandSrc : public RandSrc
{
  Random rand;
  sint4 min, max;
  
  UniRandSrc(uint4 seed, sint4 min_, sint4 max_) : min(min_), max(max_) { rand.srand(seed); }
  sint4 operator()();
};

#endif
