// $Id: LayoutHV.C,v 1.1 2006/06/27 02:32:30 orts_mburo Exp $

// This is an ORTS file (c) Michael Buro, licensed under the GPL

#include "LayoutHV.H"

using namespace std;

#define EXPECT(is,x,c) {                                         \
  if (!is) { ERR("unexpected end of string"); }                  \
  else if ((x) != (c)) {                                         \
    ostringstream os; os << c << " expected - got " << x << endl;\
    ERR(os.str());                                               \
  }                                                              \
}

#define EOS_CHECK(is) if (!is) ERR("unexpected end of string");


void LayoutHV::parse(const string &s, const R &r, sint4 space, Vector<R> &rects)
{
  cout << "parsing " << s << endl;

  istringstream is(s);
  rects.clear();
  parse(is, r, space, rects, ANY);
  char c; is >> c;
  if (is) ERR("end of string expected");
}


void LayoutHV::parse(istringstream &is,
		      const R &r, sint4 space, Vector<R> &rects, sint4 m)
{
  char c;
  
  is >> c; EOS_CHECK(is);
  c = toupper(c);

  if (c != 'H' && c != 'V') ERR("H or V expected");
  
  switch (m) {
  case ANY:
    if (c == 'H') m = H; else m = V;
    break;
  case H:
    EXPECT(is, c, 'H'); break;
  case V:
    EXPECT(is, c, 'V'); break;    
  }

  is >> c; EOS_CHECK(is); EXPECT(is, c, '(');
  sint4 nsucc;
  is >> nsucc; EOS_CHECK(is);
  if (nsucc < 0) ERR("nsucc < 0");

  is >> c; EOS_CHECK(is);

  if (c == ')') { // leaf  fixme: only when 0!
    rects.push_back(r); return;
  }
  
  Vector<sint4> weights;
  sint4 sum = 0;

  if (c == ':') { // weights

    FORS (i, nsucc) {

      EXPECT(is, c, ':');
      sint4 w;
      is >> w; EOS_CHECK(is);
      if (w < 1) ERR("weight < 1");
      weights.push_back(w);
      sum += w;
      is >> c; EOS_CHECK(is);
    }

  } else {

    // default weights
    FORS (i, nsucc) { weights.push_back(1); sum++; }

  }

  EXPECT(is, c, ',');

  sint4 x=r.x, y=r.y;
  sint4 total = (m==H) ? r.w : r.h;
  total -= (nsucc-1)*space;

  sint4 new_m = (m==H) ? V : H;
  
  FORS (i, nsucc) {

    R r2;
    r2.x = x; r2.y = y; r2.w = r.w; r2.h = r.h;

    sint4 part = (total*weights[i])/sum; 
    if (m == H) r2.w = part; else r2.h = part;

    parse(is, r2, space, rects, new_m);

    if (m == H) x += space + r2.w; else y += space + r2.h;

    is >> c;
    if (i < nsucc-1) EXPECT(is, c, ',');
  }

  EXPECT(is, c, ')');
}
