#include "ObjectManager.H"

//simple constructor.
ObjectManager::ObjectManager(){
  currentHighestNameNumber = 0; //Entity's in ogre are names this allows unique name creation.
}

//Adds Object to the Draw List. So that they will be displayed on screen.
BasicObject* ObjectManager::addObject( Ogre::String objName, GameObj *gameObj ){
  BasicObject *obj = new BasicObject(objName, gdSceneMgr, gameObj, bpName2MeshNameMap );  //initialize the object.
  objList.push_front( obj );
  sint4 xloc, yloc;
  obj->gameObj->get_center(xloc, yloc);
  int xloc1 = xloc;
  int yloc1 = yloc;
  obj->objNode->setPosition( Ogre::Vector3( xloc, 0, yloc) );								//add object to the list.
  return objList.front();	//get pointer to the pointer of the object in the list for Reference if needed by creator.
}

//This code is run every frame For every object
bool ObjectManager::frameUpdate(const Ogre::FrameEvent& evt){

  std::list<BasicObject*>::iterator iter = this->objList.begin();
	
  while( iter != objList.end() ){  //code for each object every frame.
    BasicObject *obj = (*iter);
    iter++;
  }
  return true;
}

//Generates a unique name as required by Ogre::Entity
Ogre::String ObjectManager::generateUniqueName(){
        std::cout << "Generating UNique Name" << std::endl;
	currentHighestNameNumber = currentHighestNameNumber + 1;
	return Ogre::String("&_"+Ogre::StringConverter::toString(currentHighestNameNumber));
}

//Gets a object by its ogre name.
BasicObject* ObjectManager::getObjectByName(Ogre::String name){
  //std::cout << "Attempting to compare names\n";
  std::list<BasicObject*>::iterator iter = this->objList.begin();
	while( iter != objList.end() ){  //code for each object every frame.
		BasicObject *obj = (*iter);  
		//now that we have click and have a name we want to check if it is a valid object henece begins w/ "&_"
		Ogre::String objBeg("&_");
		if( Ogre::StringUtil::startsWith( name, objBeg, false ) ){
			//std::cout << "Comparing names:" << name << " and " << (obj->objName) << std::endl;
			if( Ogre::StringUtil::match( name, (obj->objName), false) ){ 
				return obj;
			}
		}
		iter++;
	}
  return 0; 
}

//clears all objects currently selected.
bool ObjectManager::clearSelectedObjects(){
  //std::cout << "CLEAREING SELECTED OBJECTS->";
  std::list<BasicObject*>::iterator iter = this->selectedObjList.begin();
  while( iter != selectedObjList.end() ){  //for every selected objects.
    BasicObject *obj = (*iter);
    //1 since we are clearing the selected objects and selected objects have bonuding boxes lets destroy that.
    obj->objNode->showBoundingBox(false);
    iter++;   
  }//while
  selectedObjList.clear();
  //std::cout << "CLEAREING SELECTED OBJECTS COMPLETE" << std::endl;
  return true;
}

//adds a selected object.
bool ObjectManager::addSelectedObject(BasicObject *tempObj){
  //std::cout << "Attempting to add Selected object:" << tempObj->objName << std::endl; 

  //1 We are addign a selected object so make sure its not already in the list.
  std::list<BasicObject*>::iterator iter = this->selectedObjList.begin();
  while( iter != selectedObjList.end() ){  //for every selected objects.
    BasicObject *obj = (*iter);
    if( Ogre::StringUtil::match( tempObj->objName, obj->objName, false) ){ 
      return false; 
    }//if
    iter++;
  }//while
  //now that we have a valid unit selected 1) outline and 2) add to list.
  tempObj->objNode->showBoundingBox(true);
  //std::cout << "Obj Just added to selected list" << tempObj->objName << std::endl;
  selectedObjList.push_front(tempObj);
  //std::cout << "Object Added to Selected Object List" << std::endl;
  return true;
}

bool ObjectManager::doesSelectedObjectExist(BasicObject *tempObj){
  std::list<BasicObject*>::iterator iter = this->selectedObjList.begin();
  while( iter != selectedObjList.end() ){  //for every selected objects.
    BasicObject *obj = (*iter);
    if( Ogre::StringUtil::match( tempObj->objName, obj->objName, false) ){ 
      return true;
    }//if
    iter++;
  }//while
  return false;
}

/*===========================================================
The following Function takes the parameter tempObj and removes it from
the selectedObjList which is the objects user has selected. Returns true
when successful. Returns false when object is not found.
=============================================================*/
bool ObjectManager::removeSelectedObject(BasicObject *tempObj){
  std::list<BasicObject*>::iterator iter = this->selectedObjList.begin();
  while( iter != selectedObjList.end() ){  //for every selected objects.
    BasicObject *obj = (*iter);
    if( Ogre::StringUtil::match( tempObj->objName, obj->objName, false) ){ 
      //std::cout << "Selectd Obj list size:" << selectedObjList.size() << std::endl;
      //std::cout << "Object being removed:" << tempObj->objName << std::endl;
      obj->objNode->showBoundingBox(false);
      selectedObjList.erase(iter);
      //std::cout << "Selectd Obj list size:" << selectedObjList.size() << std::endl;
      return true;
    }//if
    iter++;
  }//while
  std::cout << "Error should never hit here: object found but not found??" << std::endl;
  return false;
}

bool ObjectManager::processMovementRequest( Ogre::Vector3 location ){
  std::list<BasicObject*>::iterator iter = this->selectedObjList.begin();
  while( iter != selectedObjList.end() ){  //for every selected objects.
    BasicObject *obj = (*iter);
    std::cout << "Movement request to pt:" << location.x << ", " << location.z << std::endl;
    Vector<sint4> param;
    param.push_back( (sint4)location.x );
    param.push_back( (sint4)location.z );
    param.push_back( 100000 );
    obj->gameObj->set_action("move", param );
    iter++;
  }//while
  return true;
}

bool ObjectManager::updateChangedObject( GameObj *gob ){
  if( gob->attr_changed("x") ){ std::cout << "X loc,"; }
  if( gob->attr_changed("y") ){ std::cout << "Y loc,"; }
  if( gob->attr_changed("heading") ){ std::cout << "heading,"; }
  if( gob->attr_changed("last_direction") ){ std::cout << "last direction,"; }
  if( gob->attr_changed("radius") ){ std::cout << "radius,"; }
  if( gob->attr_changed("width") ){ std::cout << "width,"; }
  if( gob->attr_changed("height") ){ std::cout << "height,"; }
  if( gob->attr_changed("zcat") ){ std::cout << "zcat,"; }
  std::cout << std::endl;

  if( gob->attr_changed("x") || gob->attr_changed("y") ){
    //update the objects position x and y.
    BasicObject *obj = getObjectByGameObj(  gob );
    if( obj == 0 ){ std::cout << "NO OBJ FOUND IN UPdATE" << std::endl; return false; }
    
    Ogre::Vector3 currentPosition = obj->objNode->getPosition();
    std::cout << "Old pos:" << currentPosition.x << ", " << currentPosition.y << ", " << currentPosition.z << std::endl;
    
    sint4 xloc, yloc;
    obj->gameObj->get_center(xloc, yloc);
    int xloc1 = xloc;
    int yloc1 = yloc;
    currentPosition.x = xloc1;
    currentPosition.z = yloc1;  //Y - Z collision.
    obj->objNode->setPosition( currentPosition );
    std::cout << "New pos:" << currentPosition.x << ", " << currentPosition.y << ", " << currentPosition.z << std::endl;
    std::cout << "gob_name()->" << gob->bp_name() << std::endl;
  }
  return true;
}

BasicObject* ObjectManager::getObjectByGameObj( GameObj *gob ){
  std::list<BasicObject*>::iterator iter = this->objList.begin();
  while( iter != objList.end() ){  //code for each object every frame.
    BasicObject *obj = (*iter);  
    if( obj->gameObj == gob ){ //points to same gob.
      return obj;
    }//if
    iter++;
  }//while	
  return 0;
}
