// $Id: Mem.C,v 1.1.1.1 2004/10/11 09:09:20 orts_mburo Exp $ 

// (c) Jonathan Yip, licensed under the GPL


#include "API.H"

#define DEBUG 1
#define GCPRINT errstr << "GC: "

void Mem::GC(sint4 elements)
{
#if DEBUG
  GCPRINT << "Starting garbage collection" << endl;
#endif

  cout << "BEFORE " << endl;
  FORU(i,m_stack.size())
    cout << i << " " << &m_stack[i] << endl;
  
  cout << "heap at " << &m_heap[0] << endl;

  stack<mem_unit*> live_objs;
  vector<mem_unit*> indices;

  map< mem_unit*, vector<mem_unit*> > obj_ptrs;

  // Find all objects on the stack
  FORU(i, m_is_obj.size())
    if (m_is_obj[i]) {
      live_objs.push(&m_stack[i]);

      while (!live_objs.empty()) {
	mem_unit* value_ptr = live_objs.top();
	mem_unit value = *value_ptr;
	
	cout << "INSIDE " << value << " " << value_ptr << endl;

	live_objs.pop();

	if (!value) continue;
	ScriptObject* o = *reinterpret_cast<ScriptObject**>(&value);

	cout << "CONT " << o << endl;


	mem_unit* obj_ptr = *reinterpret_cast<mem_unit**>(&value);

	cout << "OBJECT POINTER " << obj_ptr << endl;
	if (obj_ptrs.find(obj_ptr) != obj_ptrs.end()) {
	  obj_ptrs[obj_ptr].push_back(value_ptr);
	}
	else {
	  vector<mem_unit*> v;
	  v.push_back(value_ptr);
	  obj_ptrs[obj_ptr] = v;
	}

	if (o->m_flags & FLAG_GC) continue;
      
	o->m_flags |= FLAG_GC;

	//	vector<mem_types> members = o->m_definition->get_types();

	vector<uint4> mbs = o->m_definition->get_objs();

	cout << "** " << mbs.size() << endl;
	FORU(j, mbs.size()) {
	  cout << "pushing live obj " << obj_ptr << " " << mbs[j] << " " 
	       << obj_ptr + ScriptObjectType::members_offset + mbs[j] << endl;
	  live_objs.push(obj_ptr + ScriptObjectType::members_offset + mbs[j]);
	}

	/* old version
	FORU(j, members.size())
	  if (members[j] == OBJECT) {
	    cout << "pushing live obj " << obj_ptr << " " << j << " " 
		 << obj_ptr + ScriptObjectType::members_offset + j << endl;
	    live_objs.push(obj_ptr + ScriptObjectType::members_offset + j);
	  }
	*/
      }
  }

  // Resize the heap first, and copy-compact
  //vector<mem_unit> new_heap = m_heap;
  mem_unit* new_heap = (mem_unit*) malloc (m_heap_size * sizeof(mem_unit));
  uint4 old_heap_size = m_heap_size;
  while (m_heap_index + elements > m_heap_size) {
    // Grow heap by 2
    m_heap_size *= 2;
    new_heap = (mem_unit*) realloc(new_heap, m_heap_size * sizeof(mem_unit));
  }

  FORU(i, old_heap_size) new_heap[i] = m_heap[i];
  cout << "New heap at " << &new_heap << endl;

  map< mem_unit*, vector<mem_unit*> >::iterator obj_itr = obj_ptrs.begin();
  map< mem_unit*, vector<mem_unit*> > new_obj_ptrs;

  sint4 heap_diff = &new_heap[0] - &m_heap[0];

  // adjust the pointers
  while (obj_itr != obj_ptrs.end()) {
    new_obj_ptrs[(*obj_itr).first + heap_diff] = (*obj_itr).second;
    indices.push_back((*obj_itr).first + heap_diff);
    ++obj_itr;
  }

  // Need to fix up pointers pointing to old heap
  obj_itr = obj_ptrs.begin();
  while (obj_itr != obj_ptrs.end()) {
    map< mem_unit*, vector<mem_unit*> >::iterator new_obj_itr = new_obj_ptrs.begin();
    while (new_obj_itr != new_obj_ptrs.end()) {
      FORU(i, (*new_obj_itr).second.size()) {
	FORU(j, old_heap_size)
	  if ((*new_obj_itr).second[i] == &m_heap[j]) {
	    cout << "Setting from " << &m_heap[j] << " to " << &new_heap[j] 
		 << endl;
	    (*new_obj_itr).second[i] = &new_heap[j];
	  }
      }
      ++new_obj_itr;
    }
    ++obj_itr;
  }
  
  // fix pointers 
  
  map< mem_unit*, vector<mem_unit*> >::iterator new_obj_itr = new_obj_ptrs.begin();
  while (new_obj_itr != new_obj_ptrs.end()) {
    FORU(i, (*new_obj_itr).second.size()) {
      cout << "IN HERE " << i << " " 
	   << reinterpret_cast<mem_unit*>(*(*new_obj_itr).second[i]) << " to ";
      *(*new_obj_itr).second[i] = reinterpret_cast<mem_unit>((*new_obj_itr).first);
      cout << reinterpret_cast<mem_unit*>(*(*new_obj_itr).second[i]) << endl;
    }
    ++new_obj_itr;
  }

  mem_unit* current_index = &new_heap[0];
  mem_unit* heap_index = &new_heap[0];
  uint4 c_index = 0;

  FOREVER {
    cout << "CURRENT INDEX " << current_index << " obj index " << heap_index
	 << endl;

    if (*heap_index == 0) break;

    ScriptObject* o = reinterpret_cast<ScriptObject*>(heap_index);
    cout << "On object " << o  << endl;
    uint4 size = o->m_definition->size();
    cout << "DO I GET HERE " << endl;

    if (o->m_flags & FLAG_GC && current_index != heap_index) {
#if DEBUG
      GCPRINT << "Found a hole, moving object starting at index " << heap_index
	      << " to index at " << current_index << endl;
#endif
      
      map < mem_unit*, vector<mem_unit*> >:: iterator new_obj_itr = new_obj_ptrs.begin();
      
      while (new_obj_itr != new_obj_ptrs.end()) {
	if ((*new_obj_itr).second.size())
	  FORU(i, (*new_obj_itr).second.size()) {
	  cout << "IN HERE " << i << " " << (*new_obj_itr).second[i] << endl;
	  *(*new_obj_itr).second[i] = reinterpret_cast<mem_unit>(current_index);
	}
	++new_obj_itr;
      }

      FORU(j, size) {
	*current_index = *heap_index;
	++current_index;
	++heap_index;
      }
    }
    else if (o->m_flags & FLAG_GC && current_index == heap_index) {
      current_index += size;
      heap_index += size;
      c_index += size;
    }
    else {
      heap_index += size;
    }
      
    // Reset the gc flag
    o->m_flags &= ~FLAG_GC;      
  }


  // Set the next index to the right spot

#if DEBUG
  GCPRINT << "Setting heap index to " << c_index << endl;
#endif

  free(m_heap);
  m_heap = new_heap;

  m_heap_index = c_index;
  


}


