#include "Bucket.H"


Bucket::Bucket(const Vector<Point2i> &points_) { points = points_; }

Bucket::~Bucket() { }

/**
 * checks movement of this bucket in the given direction on the given map
 * is allawed, and stores the cost of that movement in cost
 * */
bool Bucket::check_direction(const SimpleMap<SizeCell> &map, sint4 d, sint4& cost)
{
  switch (d){
  case 0: //E
    return check_vector(map, pointsE, 1, 0, cost);
    break;

  case 1: //SE
    return check_vector(map, pointsS, 1, 1, cost) && 
	   check_vector(map, pointsE, 1, 1, cost) &&
	   check_vector(map, pointsS, 0, 1, cost) && 
	   check_vector(map, pointsE, 1, 0, cost);
    break;

  case 2: //S
    return check_vector(map, pointsS,0,1, cost);
    break;

  case 3: //SW
    return check_vector(map, pointsS, -1, 1, cost) && 
    	   check_vector(map, pointsW, -1, 1, cost) &&
	   check_vector(map, pointsS,  0, 1, cost) && 
	   check_vector(map, pointsW, -1, 0, cost);
    break;

  case 4: //W
    return check_vector(map, pointsW, -1, 0, cost);
    break;

  case 5: //NW
    return check_vector(map, pointsN, -1, -1, cost) && 
    	   check_vector(map, pointsW, -1, -1, cost) &&
	   check_vector(map, pointsN,  0, -1, cost) && 
	   check_vector(map, pointsW, -1,  0, cost);
    break;

  case 6: //N
    return check_vector(map, pointsN, 0, -1, cost);
    break;

  case 7: //NE
    return check_vector(map, pointsN, 1, -1, cost) && 
    	   check_vector(map, pointsE, 1, -1, cost) &&
	   check_vector(map, pointsN, 0, -1, cost) && 
	   check_vector(map, pointsE, 1,  0, cost);
    break;
  }

  return false;
}


bool Bucket::check_vector(const SimpleMap<SizeCell> &map,
			  const Vector<Point2i>& points,
			  sint4 dx, sint4 dy, sint4 &cost)
{
  FORALL (points, i){
    if (center.x+i->x+dx < 0
	|| center.x+i->x+dx >= map.get_w()
	|| center.y+i->y+dy < 0
	|| center.y+i->y+dy >= map.get_h() ) return false;
    if (map(center.x+i->x+dx,center.y+i->y+dy).max_size < 1) return false;
    cost += map(center.x+i->x+dx, center.y+i->y+dy).cost;
  }

  return true;

}
